;
; ==========================================================================
;
;	Scroll bar routines
;
; ==========================================================================
;
;	vscroll_init - Initialize scroll bar
;
;	Entry:
;		IY - pointer to scroll parameter block
;	Exit:
;		Blank scroll bar
;
;	vscroll_display - Display scroll bar
;
;	Entry:
;		IY - pointer to scroll parameter block
;		A - Pixel offset for cursor
;	Exit:
;		Updated display
;
; ==========================================================================
;
;	vscroll_setpos - Sets the position in the bar, does not do any
;		bounds checking.
;
;	Entry:
;		IY - pointer to scroll parameter block
;		A - Pixel offset for cursor
;		B - Old position (0xff for full redraw)
;	Exit:
;		Updated display
;	
; ==========================================================================
;
;	Scroll parameter block:
;
;	Each scroll bar is defined by a scroll parameter block which
;	has the following format:
;
;	Byte	Contents
;	0	Bar column
;	1	Bar starting row
;	2	Bar ending row
;
; ==========================================================================
;
vscroll_init				;init scroll bar
	ld	hl,vscroll_vccdata	;define custom characters
	sys_call	gn_sop
vscroll_newbar
	ld	b,(iy+2)		;set ending row
vscroll_init_1
	ld	a,08h		;blank bar
	call	scroll_xdisbar
	dec	b
	ld	a,(iy+1)		;get starting row
	cp	b		;and check if done
	jr	nz,vscroll_init_1
	ret			;and return
;
vscroll_setpos			;set cursor position
	push	af		;save new position
	ld	b,a		;get old position
	vscroll_map
;
; ==========================================================================
;
;	Internal routines
;
;	scroll_xdisbar - Output a character
;
;	Entry:
;		A - Character to display
;		B - Row offset
;		IY - parameter block
;
;	vscroll_map - Map offset and display
;
;	Entry:
;		A - Offset (0 to nrow * 8 - 4)
;		B - 0 to draw bar
;		    1 to reset bar
;	Exit:
;		Updated display
;
; ==========================================================================
;
vscroll_map				;output character
	push	bc		;save flag
	ld	e,a		;save character
	rr	a		;divide by 8
	rr	a
	rr	a
	and	07h		;mask out unused bits
	ld	d,a		;and save in d
	ld	b,a		;set row
	
scroll_xdisbar
	push	bc		;save location
	push	af		;and character
	ld	a,(iy)
	add	a,b		;make correct row
	ld	c,a
	ld	b,(iy+2)
	call	gotoxy		;set position
	ld	hl,scroll_ccprefix	;output prefix
	sys_call	gn_sop
	pop	af		;get character back
	add	a,40h		;offset to specific value
	sys_call	os_out		;and output
	pop	bc		;get pos back
	ret			;and return
;
; ==========================================================================
;
;	Static data for scroll bar routines
;
; ==========================================================================
;
vscroll_vccdata			;custom char data (vbar)
	db	1,138,61,64+0,0bfh,0bfh,0bfh,0bfh,0abh,0b5h,0abh,0b5h
	db	1,138,61,64+1,0abh,0bfh,0bfh,0bfh,0bfh,0b5h,0abh,0b5h
	db	1,138,61,64+2,0abh,0b5h,0bfh,0bfh,0bfh,0bfh,0abh,0b5h
	db	1,138,61,64+3,0abh,0b5h,0abh,0bfh,0bfh,0bfh,0bfh,0b5h
	db	1,138,61,64+4,0abh,0b5h,0abh,0b5h,0bfh,0bfh,0bfh,0bfh
	db	1,138,61,64+5,0abh,0b5h,0abh,0b5h,0abh,0bfh,0bfh,0bfh
	db	1,138,61,64+6,0abh,0b5h,0abh,0b5h,0abh,0b5h,0bfh,0bfh
	db	1,138,61,64+7,0abh,0b5h,0abh,0b5h,0abh,0b5h,0abh,0bfh
	db	1,138,61,64+8,0abh,0b5h,0abh,0b5h,0abh,0b5h,0abh,0b5h
	db	1,138,61,64+9,0bfh,0b5h,0abh,0b5h,0abh,0b5h,0abh,0b5h
	db	1,138,61,64+10,0bfh,0bfh,0abh,0b5h,0abh,0b5h,0abh,0b5h
	db	1,138,61,64+11,0bfh,0bfh,0bfh,0b5h,0abh,0b5h,0abh,0b5h
	db	0
;
scroll_ccprefix			;custom char prefix
	db	1,'2','?',0