#ifdef DOCUMENTATION
 
/File:tokgen.c
 
    This program generates a list of token frequencies in a text
file. Used to determine optimal token types for help files.
 
Valid call formats are as follows:
 
TOKGEN input_file token_file
 
Where:
 
     input_file - source file to read data from
     token_file - output file of tokens
 
#endif
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
 
#define MAX_LINE 128
#define MIN_WORDTOK 128
#define MAX_WORDTOK 512
#define WORDTOK_OUT 128
 
    struct word {
        struct word *lower;     /* lower pointer */
        struct word *upper;     /* upper pointer */
        char *text;             /* text */
        int count;           /* word count */
        };
 
    static FILE *sfile, *tfile;
    static char card[MAX_LINE];
    static long file_size;
    static long num_words;                      /* number of unique words */
    static char *word_tokens[MAX_WORDTOK];      /* pointers to word tokens */
    static int word_tok_count[MAX_WORDTOK];     /* resulting frequency */
    static double word_size[MAX_WORDTOK];       /* size factor for each word */
    static int free_mem;                        /* flag to free mem in t_tree */
    static long size_factor;                    /* size factor for word accum. */
    static int num_word_tok;                    /* number of word tokens */
 
    int proc_line(char *card);
    void proc_words(void);
    void aerror(void);
    void t_tree(struct word *wordt);
 
main(int argc,char *argv[])
 
{
 
/*
    Get the file size
*/
    printf("Text input file to read : ");
    scanf("%s",card);
    sfile = fopen(card,"rt");
    if (sfile == NULL)
    {
        printf("Unable to open file %s\n",card);
        exit(0);
    }
    fseek(sfile,(long) 0,SEEK_END);     /* set to end of file */
    file_size = ftell(sfile);           /* then ask where that is */
    printf("Processing file %s; %ld bytes\n",card,file_size);
 
/*
    Open the token file
*/
    printf("Token file to create : ");
    scanf("%s",card);
    tfile = fopen(card,"wt");
    if (tfile == NULL)
    {
        printf("Unable to open file %s\n",card);
        exit(0);
    }
 
/*
    Process the file for the number of words present
*/
    proc_words();
 
    exit(0);
    return(0);
}
 
void proc_words(void)
{
    char wtemp[80];
    char *c;
    int i, j, ic, icmp, lwork, idone;
    double dtemp;
    struct word *word1, *wordt;
 
    rewind(sfile);
 
    word1 = malloc(sizeof(struct word));
    if (word1 == NULL)
        aerror();
    word1->upper = NULL;     /* initialize first structure */
    word1->lower = NULL;
    *word1->text = 0;
 
    lwork = 0;
    num_words = 0;
    c = wtemp;
    while ((ic=fgetc(sfile)) != EOF)    /* go through the entire file */
    {
        if ((isspace(ic) == 0) && (ispunct(ic) == 0))   /* not a space */
        {
            if (lwork == 0)     /* not working on a word */
                lwork = 1;      /* but now we are */
            *c = (char) ic;     /* save the character and incr. */
            c++;
        }
        else
            if (lwork == 1)     /* working on a word, so must process */
            {
                *c = 0;         /* terminate current word */
                wordt = word1;  /* set to start of tree */
                idone = 0;
                while (idone == 0)  /* loop till a problem */
                {
                    icmp = strcmp(wtemp,wordt->text);
                    if (icmp < 0)
                    {
                        if (wordt->lower == NULL)
                        {
                            num_words++;
                            wordt->lower = malloc(sizeof(struct word));
                            if (wordt->lower == NULL)
                                aerror();
                            wordt = wordt->lower;
                            wordt->lower = NULL;
                            wordt->upper = NULL;
                            wordt->text = malloc((strlen(wtemp)+1) * sizeof(char));
                            if (wordt->text == NULL)
                                aerror();
                            strcpy(wordt->text,wtemp);
                            wordt->count = 0;
                            idone = 1;
                        }
                        else
                            wordt = wordt->lower;
                    }
                    else
                        if (icmp > 0)
                        {
                            if (wordt->upper == NULL)
                            {
                                num_words++;
                                wordt->upper = malloc(sizeof(struct word));
                                if (wordt->upper == NULL)
                                    aerror();
                                wordt = wordt->upper;
                                wordt->lower = NULL;
                                wordt->upper = NULL;
                                wordt->text = malloc((strlen(wtemp)+1) * sizeof(char));
                                if (wordt->text == NULL)
                                    aerror();
                                strcpy(wordt->text,wtemp);
                                wordt->count = 0;
                                idone = 1;
                            }
                            else
                                wordt = wordt->upper;
                        }
                        else        /* its a match */
                        {
                            idone = 1;
                            wordt->count = wordt->count + 1;
                        }
                }               /* end of word search loop */
            c = wtemp;          /* reset pointer */
            lwork = 0;          /* not working on a word */
            }
    }
/*
    Now that all of the words are in a tree, go through the tree and
    look for frequent words. Accumulate all of these in the word_tokens
    array for later processing.
*/
    idone = 0;
    size_factor = 200;
    printf("Word list generated, total of %ld unique words\n",num_words);
    printf("Selecting frequent words from list\n");
    while (idone == 0)
    {
        num_word_tok = 0;       /* zero counter */
        free_mem = 0;           /* don't free up memory */
        t_tree(word1);          /* traverse tree */
        if (num_word_tok > MAX_WORDTOK )    /* too many at this size */
            size_factor = size_factor - size_factor / 5;
        else
        {
            if (num_word_tok < (MIN_WORDTOK))   /* not enough words found */
                size_factor = size_factor + size_factor / 5;
            else                                /* all done with this */
            {
                free_mem = 1;
                num_word_tok = 0;
                t_tree(word1);
                idone = 1;
            }
        }
    }
 
/*
    Set up the size reduction array, and sort based on reduction
*/
 
    printf("Sizing based on word length and sorting by reduction\n");
 
    for (i=0;i<num_word_tok;i++)
        word_size[i] = (double) (strlen(word_tokens[i]) * word_tok_count[i]);
 
    for (i=0;i<(num_word_tok-1);i++)
        for (j=i+1;j<num_word_tok;j++)
            if (word_size[i] < word_size[j])    /* swap em */
            {
                dtemp = word_size[i];
                word_size[i] = word_size[j];
                word_size[j] = dtemp;
                c = word_tokens[i];
                word_tokens[i] = word_tokens[j];
                word_tokens[j] = c;
            }
 
/*
    Now write out this set of keywords.
*/
 
    printf("Writing out word tokens\n");
 
    for (i=0;i<WORDTOK_OUT;i++)
    {
        dtemp = word_size[i] / ((double) file_size) * 100.0;
        fprintf(tfile,"/%s/\n",word_tokens[i]);
        free(word_tokens[i]);
    }
 
}
 
void t_tree(struct word *wordt)
 
{
    if (wordt->lower != NULL)   /* nothing lower */
        t_tree(wordt->lower);
 
    if (wordt->upper != NULL)   /* nothing upper */
        t_tree(wordt->upper);
 
/*
   Now figure out if we want it or not, criteria is based on 50%
   reduction of file size (roughly)
*/
 
    if ((wordt->count * strlen(wordt->text)) > (int)(file_size / size_factor))
    {
        if (num_word_tok < MAX_WORDTOK)
        {
            word_tok_count[num_word_tok] = wordt->count;
            word_tokens[num_word_tok] = wordt->text;
        }
        num_word_tok++;
        if (free_mem == 1)
            free(wordt);            /* free memory for this guy */
    }
    else
        if (free_mem == 1)
        {
            free(wordt->text);      /* free text string memory */
            free(wordt);            /* free memory for this guy */
        }
 
    return;
 
}
 
 
void aerror(void)
 
{
    printf("\n\nUnable to allocate memory - aborted.....\n\n");
    exit(0);
}
