	title	Hex Import
;
; ==========================================================================
;
;	This module is the Intel Hex Import Program for the Z88
;
; ==========================================================================
;
;	Error codes for hxi_error exit routine:
;
;	FF - Block checksum error
;	FE - No command block found
;	FD - Verify error burning EPROM
;	FC - Command block error
;
; ==========================================================================
;
hex_import
	ld	hl,0		;save the stack state
	add	hl,sp
	ld	(hxi_stack),hl
	ld	hl,0		;zero out handles
	ld	(hxi_handin),hl
	ld	(hxi_handout),hl
	ld	(hxi_rupdn),hl	;zero block counter
	xor	a		;zero bank counter
	ld	(hxi_bank),a
	call	hxi_bdatazero	;zero out data block
;
	IFDEF	DISTRIBUTE
	z88_name	'HX Version 1.00'	;set program name
	ld	hl,hex_imp_win	;print out window header
	sys_call	gn_sop
	ld	hl,hex_initmsg	;put out initial screen
	sys_call	gn_sop
:
	sys_call	os_in		;get a key
	jr	c,:-1
	call	a_to_lower		;make lower case
	cp	'b'		;check for BASIC dump request
	jr	z,hxi_bascomin
	cp	'y'		;check for yes
	jr	z,:+1
	jp	c,hxi_done
:
	ld	hl,hex_imp_win	;put out main window
	sys_call	gn_sop
	ld	hl,hxi_epvwin	;and verify message
	sys_call	gn_sop
	call	ep_verify		;call verify EPROM routine
	jr	z,:+1		;it is erased
	ld	hl,hxi_epvfail
	sys_call	gn_sop
	call	pause
	jp	hxi_done
:
	ld	hl,hxi_prmpt0	;call get file name routine
	ld	de,hxi_fname	;set buffer
	ld	c,020h		;set buffer length
	call	get_file_name	;get file name
	ENDIF
;
hxi_bascomin
;
	xor	a		;set up for open call
	ld	b,a		;set page 0 for filename
	inc	a		;set a=1 (open for input)
	ld	c,020h		;set scratch length
	ld	de,hxi_scratch	;scratch for expanded file name
	ld	hl,hxi_fname	;set file name address
	sys_call	gn_opf		;open file
	jp	c,hxi_done		;exit if an open error
	ld	(hxi_handin),ix	;save file handle
	ld	hl,hex_imp_win	;and redraw main window
	sys_call	gn_sop
;
;	Now get the first record to figure out what is going on
;
	ld	hl,hxi_waitprmpt	;put out waiting message
	sys_call	gn_sop
	call	hxi_grec		;get the first record
	ld	a,(hxi_btype)	;get type
	cp	1		;check for type 1 record
	jp	z,hxi_done
	ld	hl,(hxi_baddr)	;get address
	ld	a,h		;and test for a 1
	and	a
	jr	nz,:+1
	ld	a,l
	cp	1		;test for basic file
	jp	z,hxi_basfile
	cp	2		;test for EPROM burn
	jp	z,hxi_eprom
	ld	a,0fch		;unknown command
	jp	hxi_error
:				;not a command block
	ld	a,254		;set error and error exit
	jp	hxi_error
;
; ==========================================================================
;
;	Write the data out to a BASIC file
;
; ==========================================================================
;
hxi_basfile
	ld	hl,hxi_baprmpt	;set type prompt
	call	hxi_workwin		;set up working window
	ld	hl,hxi_baprmptf	;put out file message
	sys_call	gn_sop
	ld	hl,(hxi_bdata)	;get offset
	ld	(hxi_boffset),hl	;and save it
	ld	hl,hxi_bdata+2	;put out file name
	sys_call	gn_sop
	ld	a,2		;set a=1 (open for output)
	ld	b,0		;set page
	ld	c,01Fh		;set scratch length
	ld	de,hxi_scratch	;scratch for expanded file name
	ld	hl,hxi_bdata+2	;set file name address
	sys_call	gn_opf		;open file
	jp	c,hxi_error		;exit if an open error
	ld	(hxi_handout),ix	;set output handle
;
hxi_basfa
	call	hxi_grec		;get another record
	call	hxi_rupd		;update display
	ld	a,(hxi_btype)
	cp	1		;check for end of data
	jp	z,hxi_done
	ld	a,(hxi_blen)	;check length
	and	a		;and loop if zero
	jr	z,hxi_basfa
	ld	hl,hxi_bdata	;set buffer address
:
	push	af		;save count
	push	hl		;and address
	ld	a,(hl)		;get byte
	ld	ix,(hxi_handout)	;set handle
	sys_call	os_pb		;and output it
	jp	c,hxi_error
	pop	hl		;get address back
	inc	hl		;and count it
	pop	af		;get count
	dec	a
	jr	nz,:-1		;loop till done with this rec.
	jp	hxi_basfa		;and get another record
;
; ==========================================================================
;
;	Write the data out to the EPROM
;
; ==========================================================================
;
hxi_eprom
	ld	hl,hxi_epprmpt	;set transfer prompt
	call	hxi_workwin		;set up working window
	ld	a,(hxi_bdata)	;get eprom size and save it
	ld	(hxi_epsize),a
	ld	a,(hxi_bdata+1)	;get bank number
	ld	(hxi_bank),a	;and save it
	ld	hl,(hxi_bdata+2)	;get offset
	ld	(hxi_boffset),hl	;and save it
	xor	a		;zero error flag
	ld	(hxi_epberr),a
;
hxi_eploop
	call	hxi_grec		;get another record
	call	hxi_rupd		;update display
	ld	a,(hxi_btype)
	cp	1		;check for end of data
	jp	z,hxi_epdone
	ld	hl,(hxi_baddr)	;get address
	ld	a,h		;and test for a 2
	and	a
	jr	nz,:+2		;not a command
	ld	a,l
	cp	2		;check for EPROM command
	jr	z,:+1		;yep
	ld	a,0fch		;set invalid command error
	jp	hxi_error
:
	ld	a,(hxi_bdata)	;get eprom size and save it
	ld	(hxi_epsize),a
	ld	a,(hxi_bdata+1)	;get bank number
	ld	(hxi_bank),a	;and save it
	ld	hl,(hxi_bdata+2)	;get offset
	ld	(hxi_boffset),hl	;and save it
	jp	hxi_eploop		;and get next record
:
	ld	a,(hxi_blen)	;check length
	and	a		;and loop if zero
	jp	z,hxi_eploop
	call	hxi_epburn		;write block to EPROM
	jp	hxi_eploop		;and go back for more
;
hxi_epdone				;exit from EPROM burn
	ld	a,(hxi_epberr)	;check burn error flag
	and	a
	jr	z,:+2
	ld	hl,hxi_eperrmsg	;put out error message
	sys_call	gn_sop
:
   	sys_call	os_in		;wait for a key
	jr	c,:-1
:
	IFDEF	DISTRIBUTE
	ld	hl,hxi_ependmsg	;put out ending message
	sys_call	gn_sop
:
	sys_call	os_in		;wait for a key
	jr	c,:-1
	ENDIF
;
	jp	hxi_done
;
; ==========================================================================
;
;	Burn the current block of bytes into the EPROM
;
; ==========================================================================
;
hxi_epburn
	ld	a,(hxi_bank)	;set bank
	or	0c0h		;make third slot
	ld	b,a		;and map to address space
	ld	c,3
	sys_call	os_mpb		;map it
	push	bc		;save mapping
	ld	de,(hxi_boffset)	;get offset
	ld	hl,(hxi_baddr)	;get block address
	xor	a		;clear carry flag
	sbc	hl,de		;convert to offset in bank
	ld	de,0c000h		;convert to mapped address
	add	hl,de
	ex	de,hl		;put offset into de
	ld	hl,hxi_bdata	;and data start into hl
	call	hxi_epbwrite	;write out this record
	pop	bc		;get mapping back
	sys_call	os_mpb		;map bank back
	ret			;and return
;
hxi_epbwrite			;write out block of bytes
	xor	a		;zero c
	ld	c,a
	ld	b,75		;number of trys for EPROM
	ld	a,48h		;set ep signal (68h for 128k)
	out	(0b3h),a
:
	ld	a,0eh		;set Vpp and program bits
	out	(0b0h),a
	call	hxi_epbw1		;write block
	ld	a,04h		;reset Vpp and program bits
	out	(0b0h),a		;(but not screen)
	call	hxi_epbvfy		;verify
	jr	z,:+1		;block OK
	djnz	:-1		;loop
:
	ld	a,76		;now go back and reburn the
	sub	b		;the same number of times
	ld	b,a
:
	ld	a,02eh		;set Vpp and program bits
	out	(0b0h),a
	call	hxi_epbw1
	ld	a,04h		;reset Vpp and program bits
	out	(0b0h),a		;(but not screen)
	djnz	:-1		;loop till done
	call	hxi_epbvfy		;verify final write
	jr	z,:+1
	ld	a,0ffh		;write error, set flag
	ld	(hxi_epberr),a
:
	ld	a,05h		;turn screen back on
	out	(0b0h),a
	ret			;and return
;
hxi_epbw1				;copy block of data
	push3			;save registers
	ld	a,(hxi_blen)	;get length
	ld	c,a		;and put in BC
	xor	a
	ld	b,a
	ldir			;copy bytes
	pop3			;get registers back
	ret			;and return
;
hxi_epbvfy				;verify data
	push3			;save registers
	ld	a,(hxi_blen)	;get length
	ld	b,a		;and put in B
:
	ld	a,(de)		;get data byte
	cp	(hl)		;compare to ROM
	jr	nz,:+1		;is bad
	inc	de		;next byte
	inc	hl
	dec	b		;loop till done
	jr	nz,:-1
:
	pop3			;get registers back
	ret			;and return
;
hxi_done
	ld	hl,hxi_handin	;check file and close if need be
	call	close_file
	ld	hl,hxi_handout
	call	close_file
	call	reset_win
;
	ret			;return
;
; ==========================================================================
;
;	Support routines
;
; ==========================================================================
;
hxi_grec			;get a hex record
	call	hxi_gchar	;get a character
	cp	':'	;check for start of record,
	jr	nz,hxi_grec	;if not loop back for more
	xor	a	;initialize checksum
	ld	(hxi_chk),a
	call	hxi_gb	;get record length
	ld	(hxi_blen),a
	push	af	;save it for a moment
	call	hxi_gb	;get address
	ld	(hxi_baddr+1),a
	call	hxi_gb
	ld	(hxi_baddr),a
	call	hxi_gb	;get record type
	ld	(hxi_btype),a
	pop	af	;get length back
	and	a	;and check for zero length
	jr	z,:+2
;
	ld	hl,hxi_bdata	;set buffer start
:
	push	af	;save count
	push	hl	;and address
	call	hxi_gb	;get the next byte
	pop	hl	;get address back
	ld	(hl),a	;save this byte
	inc	hl	;next address
	pop	af	;and count
	dec	a	;count down
	jr	nz,:-1
:
	call	hxi_gb	;get checksum byte
	ld	a,(hxi_chk)	;now check checksum (should be zero with
	and	a	;the checksum value added in)
	jr	z,:+1
	ld	a,255	;set error number
	jp	hxi_error	;and branch to error exit routine
:
	ret		;return
;
hxi_gb			;get a hex pair as a single byte
	call	hxi_gchar	;get a character
	push	af	;and save it
	call	hxi_gchar	;get the second
	ld	l,a	;put into hl
	pop	af
	ld	h,a
	call	hex_to_a	;and convert
	push	af	;save char
	ld	hl,hxi_chk	;get checksum address
	add	a,(hl)	;and count this value
	ld	(hl),a
	pop	af	;get char back
	ret		;and return
;
hxi_gchar			;get a character from the file
	ld	ix,(hxi_handin)	;get file handle
:
	sys_call	os_gb	;get byte
	ret	nc	;return if no error
	cp	rc_eof	;check for eof
	jr	z,:+1
	cp	rc_hand	;bad handle
	jr	z,:+1
	cp	rc_rp	;read protect
	jr	z,:+1
	jr	:-1	;loop back for more
:
	jp	hxi_error	;branch to error handler
;
hxi_rupd				;update transfer display
	ld	bc,1004h		;position cursor
	call	gotoxy
	ld	hl,(hxi_rupdn)	;get block number
	inc	hl		;count and save
	ld	(hxi_rupdn),hl
	call	hl_to_hexdis	;and display
	ld	bc,1c04h		;position cursor
	call	gotoxy
	ld	a,(hxi_bank)	;display bank
	call	a_to_hexdis
	ld	bc,2904h		;set cursor
	call	gotoxy
	ld	hl,(hxi_boffset)	;display offset
	call	hl_to_hexdis
	ld	bc,3904h		;position cursor
	call	gotoxy
	ld	hl,(hxi_baddr)	;get address
	call	hl_to_hexdis	;and display
	ret			;all done
;
hxi_bdatazero			;zero out the code/data storage
	ld	hl,hxi_bdata	;set address
	ld	b,40h		;and length
	xor	a		;zero a
:
	ld	(hl),a		;and zero byte
	inc	hl
	djnz	:-1		;loop
	ret			;and return
;
hxi_error				;error handler
	push	af		;save error type
	ld	hl,hxi_handin	;close files if open
	call	close_file
	ld	hl,hxi_handout
	call	close_file
	pop	af		;get error code back
	cp	0ffh		;check for error
	jr	nz,:+1
	ld	hl,hxi_emsgff
	jp	hxi_error1
:
	cp	0feh		;check for error
	jr	nz,:+1
	ld	hl,hxi_emsgfe
	jp	hxi_error1
:
	cp	0fdh		;check for error
	jr	nz,:+1
	ld	hl,hxi_emsgfd
	jp	hxi_error1
:
	cp	0fch		;check for error
	jr	nz,:+1
	ld	hl,hxi_emsgfc
	jp	hxi_error1
:
	and	a		;must be a system error
	sys_call	gn_err
hxi_error1
	push	hl		;save hl
	ld	hl,hxi_emsg		;and setup screen
	sys_call	gn_sop
	pop	hl		;get error message text back
	sys_call	gn_sop
	sys_call	os_in		;get a keystroke
;
	call	reset_win
	ld	hl,(hxi_stack)	;restore stack
	ld	sp,hl
	ret			;and return to caller
;
hxi_workwin				;put out working screen info
	push	hl		;save message address
	ld	hl,hex_imp_scr0	;set data address
	sys_call	gn_sop
	pop	hl
	sys_call	gn_sop
	ret
;
; ==========================================================================
;
;	Static data area
;
; ==========================================================================
;
hex_imp_win				;main window
	vid_win_1	1,0,0,88,8
	db	'HEX IMPORT VERSION 1.00'
	vid_win_2	1,0,0,88,8
	db	0
;
hxi_epvwin				;verify window
	db	12
	vid_just_center
	vid_cursor_off
	db	10,10,'Verifying EPROM Erase',10
	db	0
;
hxi_epvfail				;verify failed
	db	12,10
	vid_bold_toggle
	db	'EPROM Verify Failed',10,10
	vid_bold_toggle
	db	'The EPROM is not completely erased or is defective',10
	db	0
;
	IFDEF	DISTRIBUTE
hex_initmsg				;initial message screen
	db	12
	vid_just_center
	vid_cursor_off
	db	'HX - Z88 Hex Import Program    Version 1.00',10
	copyright_message
	publicdomain_message
	db	10,'This program burns a hex data file into a blank '
	db	'EPROM.',10
	db	10,'Is there a freshly erased EPROM in the third '
	db	'slot [Y/N]',13
	db	0
hxi_ependmsg			;end of burn message
	db	12
	vid_just_center
	vid_cursor_off
	vid_bold_toggle
	db	'EPROM BURN COMPLETED',10
	vid_bold_toggle
	db	10,'To make the system aware of the EPROM '
	db	'go to the INDEX and with the computer',10
	db	'on open the front door and remove the '
	db	'EPROM, close the front door, then open',10
	db	'the front door again, replace the EPROM '
	db	'and close the front door.',13
	db	0
hxi_prmpt0				;Get file name prompt
	db	'Hex Data Source',0
	ENDIF
;
hxi_eperrmsg			;burn error message
	db	12
	vid_just_center
	vid_cursor_off
	vid_bold_toggle
	db	'Verify error burning EPROM',10
	vid_bold_toggle
	db	10,'There was a verify error during the EPROM '
	db	'burn, the EPROM should be erased and reburned.',10
	db	0
;
hex_imp_scr0			;main working screen
	db	12
	vid_just_normal
	vid_gotoxy	10,4
	db	'Block'
	vid_gotoxy	22,4
	db	'Bank'
	vid_gotoxy	33,4
	db	'Offset'
	vid_gotoxy	48,4
	db	'Address'
	vid_gotoxy	5,1
	db	'Converting HEX data to '
	db	0
hxi_waitprmpt			;waiting for data message
	db	12
	db	0
hxi_baprmpt				;basic transfer message
	db	'a BASIC program',0
hxi_baprmptf			;File prompt
	vid_gotoxy	5,2
	db	'Saving file '
	db	0
hxi_epprmpt				;EPROM transfer message
	db	'EPROM',0
;
;	Error messages
;
hxi_emsg				;error message prefix
	db	12
	vid_just_center
	vid_gotoxy	1,5
	db	'Hit any key to continue',13
	vid_gotoxy	1,3
	db	'Error - ',0
hxi_emsgff	db	'Block checksum error',0
hxi_emsgfe	db	'First block not a command block',0
hxi_emsgfd	db	'EPROM write verify error',0
hxi_emsgfc	db	'Illegal command type',0
;
; ==========================================================================
;
;	Dynamic data area
;
; ==========================================================================
;
hxi_fname			;file name to open
	db	':COM.0'
	bsz	20h
hxi_scratch			;scratch space
	bsz	20h
hxi_rupdn			;block number
	dw	0
hxi_handin			;file handle for hex input file
	dw	0
hxi_handout			;file handle for hex output file
	dw	0
hxi_stack			;entry level stack
	dw	0
hxi_epberr			;error code for eprom burn
	db	0
hxi_epsize			;eprom size (banks)
	db	0
hxi_chk			;checksum for records
	dw	0
hxi_blen			;block length
	db	0
hxi_baddr			;block address
	dw	0
hxi_btype			;block type
	db	0
hxi_bank			;bank for EPROM
	db	0
hxi_boffset			;offset for basic or bank for EPROM burn
	dw	0
hxi_bdata			;block data
	bsz	40h
;
