	title	Utilities
;
; ==========================================================================
;
;	Utilties module
;
; ==========================================================================
;
;	Memory Status
;
; ==========================================================================
;
util_memstat			;memory statistics
	xor	a		;Initialize for main entry
	ld	(util_mdrdrawflg),a	;set redraw flag
	call	util_msredraw
	call	pause
	ret
;
util_msredraw
	pushall			;save all registers
	ld	hl,util_mswin	;set up main window
	call	title_win_full
	popall			;get all registers back
	ret			;and return
;
; ==========================================================================
;
;	Memory support routines
;
; ==========================================================================
;
;	util_msgramsize - Get ram size
;		C - Slot to examine
;		A - returned as number of banks, 0 for no RAM, Z flag
;		set if this is the case.
;
;	util_msgromsize - Get rom size
;		C - Slot to check
;		A - returned as number of banks, 0 for no ROM, Z flag
;		set if this is the case.
;		B - if a ROM, returned as 0 for application, 1 for file
;		DEHL - if application returns ID code
;
;	util_msgromchk - Get rom checksum
;		C - Slot to check
;		HL - Pointer to checksum
;		ZF set if no ROM, otherwise CRC in (HL)
;
; ==========================================================================
;
util_msgramsize			;get ram size
	ld	a,c		;get bank number
	and	03h		;and check if bank zero
	jr	nz,:+1
	ld	a,021h		;set for bank 21 (special case
	jr	:+2		;for the first bank
:
	rlca			;multiply by 64
	rlca
	rlca
	rlca
	rlca
	rlca
:
	ld	b,a		;save bank number
	ld	c,2		;map into bank two
	sys_call	os_mpb		;map it
	push	bc		;save old mapping
	ld	hl,08000h		;address to start of memory
	ld	a,(hl)		;get first byte
	cp	05ah		;check for valid RAM header
	jr	nz,:+1
	inc	hl		;get next byte
	ld	a,(hl)
	cp	0a5h		;check for header byte two
	jr	nz,:+1
	inc	hl		;its valid, get size in banks
	ld	a,(hl)
	jr	:+2
:
	xor	a		;zero out size
:
	pop	bc		;get mapping back
	push	af		;save size
	sys_call	os_mpb		;map bank back
	pop	af		;get size back
	and	a		;set flags
	ret			;and return	
;
util_msgromsize			;get ram size
	call	util_msgross	;get bank offset
	ld	b,a		;save bank number
	ld	c,2		;map into bank two
	sys_call	os_mpb		;map it
	push	bc		;save old mapping
	ld	hl,0bfffh		;address to top of EPROM
	ld	a,(hl)		;check top byte
	cp	'Z'		;check if application
	jr	z,util_msgrosa
	cp	'z'		;check if file rom
	jr	z,util_msgrosf
	xor	a		;zero a
	jr	util_msgrosd	;and exit
;
util_msgrosa			;handle application
	dec	hl		;next byte down
	cp	'O'		;check byte two
	jr	nz,util_msgrosd	;nope
	dec	hl		;get size
	dec	hl
	ld	c,(hl)
	dec	hl		;get id bytes (3-4)
	ld	e,(hl)
	dec	hl
	ld	d,(hl)
	push	de		;save bytes 3-4
	dec	hl		;get bytes 1-2
	ld	e,(hl)
	dec	hl
	ld	d,(hl)
	pop	hl		;get bytes 3-4 in hl
	xor	a		;set rom type
	ld	b,a
	ld	a,c		;get size back
	and	a		;set flags
	jr	util_msgrosd	;and exit
;
util_msgrosf			;handle files
	dec	hl		;next byte down
	cp	'o'		;check byte two
	jr	nz,util_msgrosd	;nope
	dec	hl		;get size
	dec	hl
	ld	a,0ffh		;set rom type
	ld	b,a
	ld	a,(hl)
	and	a		;set flags
	jr	util_msgrosd	;and exit
;
util_msgrosd			;done
	pop	iy		;get mapping back in IY
	pushall			;save registers
	push	iy		;then put iy back in bc
	pop	bc
	sys_call	os_mpb		;remap
	popall			;get all registers back
	ret			;and return
;
util_msgross			;get ROM bank in A given C=Slot
	ld	a,c		;get bank number
	and	03h		;and check if bank zero
	jr	nz,:+1
	ld	a,01fh		;set for bank 21 (special case
	ret			;for the first bank
:
	rlca			;multiply by 64
	rlca
	rlca
	rlca
	rlca
	rlca
	add	a,3fh		;and offset to end of bank
	ret			;return
;
util_msgromchk			;get ROM checksum
	push	bc		;save slot number
	call	util_msgromsize	;check rom
	jr	nz,:+1		;has a rom chip
:
	ret	
;
; ==========================================================================
;
;	Memory Dump
;
; ==========================================================================
;
util_memdump
	xor	a		;Initialize for main entry
	ld	(util_bank),a	;zero bank
	ld	hl,0		;and offset
	ld	(util_offset),hl
	ld	(util_mdrdrawflg),a	;set redraw flag
	call	util_mdredraw
util_mdmain
	ld	a,0
	ld	(util_mdrdrawflg),a	;set redraw flag
	call	util_erasecmd	;erase command line
	ld	hl,util_cmdmsg	;put out command message
	sys_call	gn_sop
:
	sys_call	os_in		;get command
	jr	c,:-1		;loop on preempt or whatever
	cp	00h		;check for arrow keys
	jr	nz,:+2
:
	sys_call	os_in		;get next key
	cp	0ffh		;up arrow
	jp	z,util_mddisdn
	cp	0feh		;down arrow
	jp	z,util_mddisup
	jr	:-2
:
	call	a_to_lower		;conver to lower case
	cp	'b'		;change memory display
	jp	z,util_getbank
	cp	'o'		;update offset
	jp	z,util_getoffset
	cp	'q'		;basic exit routine
	jp	z,util_mddone
	jp	:-3
;
util_mddone				;exit point
	call	reset_win		;zero out window
	ret			;return to caller
;
util_mderrhan			;error handler
	ret	z		;return if fatal
	cp	rc_esc		;check for esc
	jr	nz,:+1
	sys_call	os_esc		;acknowledge esc
	cp	a		;set flags
	ret			;and return
:
	cp	rc_quit		;check for quit request
	jr	nz,:+1
	xor	a
	sys_call	os_bye		;exit appl
:
	cp	rc_draw		;check for redraw
	jr	nz,:+1
	call	util_mdredraw	;redraw screen
	cp	a		;set flags and return
	ret
:
	cp	a		;set flags
	ret			;and return
;
util_mdredraw
	pushall			;save all registers
	ld	hl,util_mdwin	;set up main window
	call	title_win_full
	call	util_mddis		;draw out intial screen
	call	util_erasecmd	;erase command line
	ld	a,(util_mdrdrawflg)	;get redraw flag
	cp	00h		;main command
	jr	z,:+1
	cp	01h		;get byte message
	jr	z,:+2
	cp	02h		;get offset message
	jr	z,:+3
:
	ld	hl,util_cmdmsg
	jr	util_mdredraw1
:
	ld	hl,util_gbmsg
	jr	util_mdredraw1
:
	ld	hl,util_gomsg
	jr	util_mdredraw1
util_mdredraw1
	sys_call	gn_sop		;output the text
	popall			;get all registers back
	ret			;and return
;
util_getbank
	ld	a,01h		;set display point
	ld	(util_mdrdrawflg),a
	call	util_erasecmd	;erase command line
	ld	hl,util_gbmsg	;output prompt
	sys_call	gn_sop
:
	ld	de,util_bufin	;set buffer address
	ld	a,02h		;set mode
	ld	c,0		;and starting position
	ld	b,3
	sys_call	gn_sip		;get input line
;	jr	c,:-1		;loop on error
	sys_call	os_pur
	ld	hl,util_bufin	;get characters and convert
	ld	d,(hl)		;to hex
	inc	hl
	ld	e,(hl)
	ex	de,hl
	call	hex_to_a		;convert it
	ld	(util_bank),a	;save it
	call	util_mddis		;and update display
	jp	util_mdmain
;
util_getoffset
	ld	a,02h		;set display point
	ld	(util_mdrdrawflg),a
	call	util_erasecmd	;erase command line
	ld	hl,util_gomsg	;output prompt
	sys_call	gn_sop
:
	ld	de,util_bufin	;set buffer address
	ld	a,02h		;set mode
	ld	c,0		;and starting position
	ld	b,5
	sys_call	gn_sip		;get input line
;	jr	c,:-1		;redo if error
	sys_call	os_pur
	ld	hl,util_bufin	;get characters and convert
	ld	b,(hl)		;to hex
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	e,(hl)
	inc	hl
	call	hex_to_hl		;convert it
	ld	a,h		;adjust
	and	3fh
	ld	h,a
	ld	a,l
	and	0f0h
	ld	l,a
	ld	(util_offset),hl	;save it
	call	util_mddis		;and update display
	jp	util_mdmain
;
util_erasecmd			;erase the command line
	ld	bc,0106h		;set cursor
	call	gotoxy
	ld	b,80		;set cursor count
	ld	a,' '		;set for space
:
	sys_call	os_out		;print the space
	dec	b
	jr	nz,:-1
	ld	bc,0106h		;set cursor
	call	gotoxy
	ret			;return
;
util_mddisup			;up a block of memory
	ld	hl,(util_offset)	;get offset
	ld	de,0030h		;set up count
	add	hl,de		;up count
	ld	a,h		;get high byte
	and	0c0h		;get top two bits
	jr	z,util_mddisscroll	;no carry
	push	hl		;save address
	ld	hl,util_bank	;next bank
	inc	(hl)
	pop	hl
	ld	de,0c000h		;and decrement offset
	add	hl,de
	jr	util_mddisscroll
;
util_mddisdn			;down a block of memory
	ld	hl,(util_offset)	;get offset
	ld	de,0ffd0h		;set up count (rollover)
	add	hl,de		;adjust
	ld	a,h		;get high bits
	and	0c0h
	jr	z,util_mddisscroll	;no underflow
	push	hl		;save hl and decrement bank
	ld	hl,util_bank
	dec	(hl)
	pop	hl
	ld	de,04000h		;and update offset
	add	hl,de
	jr	util_mddisscroll
;
util_mddisscroll			;update offset and display
	ld	(util_offset),hl	;save updated address
	call	util_mddis		;update display
	jp	util_mdmain		;and back to main menu
;
;	Display a block of memory
;
util_mddis				;display memory block
	ld	bc,0200h		;set initial cursor position
	ld	hl,(util_offset)	;set buffer address
	ld	a,(util_bank)	;save a copy of the bank
	push	af
	push	hl
	ld	a,6		;set count
:
	push	af		;save registers
	push	bc
	call	gotoxy		;set cursor
	ld	a,(util_bank)	;display bank number
	call	a_to_hexdis
	ld	a,':'		;and put out a colon
	sys_call	os_out
	ld	hl,(util_offset)
	call	hl_to_hexdis
	ld	a,20h		;and a space
	sys_call	os_out
	call	util_mdcopy		;move block of data
	ld	de,0		;prefix
	ld	hl,util_buf		;and get address
	call	display_hex_line
	ld	de,0010h		;next line
	ld	hl,(util_offset)	;get offset
	add	hl,de		;increment offset
	ld	(util_offset),hl	;and save
	ld	a,h		;and check for overflow
	and	0c0h		;to next bank
	jr	z,:+1
	ld	hl,util_bank	;increment bank number
	inc	(hl)
	ld	hl,(util_offset)	;get offset back
	ld	de,0c000h
	add	hl,de
	ld	(util_offset),hl	;and update
:
	pop	bc		;get bc and a back
	pop	af
	inc	c		;next row
	dec	a		;count this
	jr	nz,:-2		;loop till done
	pop	hl		;get offset and bank back
	pop	af
	ld	(util_bank),a	;and save
	ld	(util_offset),hl
	ret			;return
;
util_mdcopy				;copy 16 bytes to buffer
	ld	a,(util_bank)	;get bank
	ld	b,a		;and set in b
	ld	c,2		;to bank 2 mapping
	sys_call	os_mpb
	push	bc		;save old binding
	ld	bc,0010h		;set count
	ld	de,08000h		;set offset to bank
	ld	hl,(util_offset)	;get offset in bank
	add	hl,de		;mapped address
	ld	de,util_buf		;destination
	ldir			;copy bytes
	pop	bc		;get mapping back
	sys_call	os_mpb		;and do it
	ret			;all done
;
; ==========================================================================
;
;	Static Data Area (Memory Dump)
;
; ==========================================================================
;
util_mdwin				;memory dump window
	db	'MEMORY DISPLAY v1.00'
	db	0
util_cmdmsg				;command line prompt
	db	'Command: '
	vid_bold_toggle
	db	'B)'
	vid_bold_toggle
	db	'ank, '
	vid_bold_toggle
	db	'O)'
	vid_bold_toggle
	db	'ffset, '
	vid_bold_toggle
	db	'Q)'
	vid_bold_toggle
	db	'uit '
	db	0
util_gbmsg				;get bank message
	db	'Bank (xx) : ',0
util_gomsg				;get offset message
	db	'Offset in bank (xxxx) : ',0
;
; ==========================================================================
;
;	Static Data Area (Memory Status)
;
; ==========================================================================
;
util_mswin				;main window
	db	'MEMORY STATUS v1.00'
	db	0
;
; ==========================================================================
;
;	Dynamic Data Area
;
; ==========================================================================
;
	IFDEF	BASIC
util_dstart	equ	$
	bsz	030h
	ENDIF
;
	IFDEF	ROM
util_dstart	equ	(01ffdh-030h)	;allocated down from 01ff
	ENDIF
;
util_buf	equ	util_dstart		;data buffer (16 bytes)
util_bufin	equ	util_buf+010h	;input buffer (6 bytes)
util_bank	equ	util_bufin+010h	;bank to map
util_offset	equ	util_bank+01h	;offset
util_mdrdrawflg	equ	util_offset+02h	;redraw status
;