;
; ==========================================================================
;
;	Utilities ROM.asm
;
;	This file contains the ROM DOR and related table information
;	for the Utilities application.
;
; ==========================================================================
;
	set_epseg_1			;set to top segment
util_dor
	db	0,0,0		;link to parent
	db	0,0,0		;link to brother
	db	0,0,0		;link to son
	db	083h		;dor type
	db	:+8-:+1		;dor length
:
	db	'@'		;key to info section
	db	:+2-:+1		;length of info section
:
	dw	0		;reserved
	db	'U'		;access key
	db	0		;contiguous ram required
	dw	0		;environment overhead
	dw	0		;unsafe space needed
	dw	0100h		;save space needed
	dw	util_apl_entry	;entry point
	db	0		;binding on entry; segment 1
	db	0		;binding on entry; segment 2
	db	0		;binding on entry; segment 3
	db	03eh		;binding on entry; segment 4
	db	01h		;application type byte 1
	db	00h		;application type byte 2
:
	db	'H'		;key to help section
	db	:+2-:+1		;length of help section
:
	dw	util_topics&3fffh	;pointer to topics
	db	03fh
	dw	util_commands&3fffh	;pointer to commands
	db	03fh
	dw	util_help&3fffh	;pointer to appl. help
	db	03fh
	dw	util_tokens&3fffh	;pointer to appl. tokens
	db	03fh
:
	db	'N'		;key to name section
	db	:+2-:+1		;length of name section
:
	db	'Utilities',0
:
	db	0ffh		;dor terminator
:
	include	'::HelpGen:utility.asm'
;
	set_epseg_2			;set to code segment
util_apl_entry
	call	util_entry		;call entry point
	scf			;enquiry entry point
	ret
;
util_entry
	xor	a		;zero a
	ld	b,a		;and b
	ld	hl,util_errhan	;set up error handler
	sys_call	os_erh
	ld	a,5		;enable escape detection
	sys_call	os_esc
	call	util_main

;
util_exit
	xor	a		;no error exit
	sys_call	os_bye
;
util_errhan				;error handler
	ret	z		;return if fatal
	cp	rc_esc		;check for esc
	jr	nz,:+1
	sys_call	os_esc		;acknowledge esc
	cp	a		;set flags
	ret			;and return
:
	cp	rc_quit		;check for quit request
	jr	nz,:+1
	xor	a
	sys_call	os_bye		;exit appl
:
	cp	rc_draw		;check for redraw
	jr	nz,:+1
	call	util_mainredraw	;redraw screen
	cp	a		;set flags and return
	ret
:
	cp	a		;set flags
	ret			;and return
;
util_main				;main start for util
	call	util_mainredraw	;draw out screen
util_mainloop
	sys_call	os_in		;get a key
	jr	nc,:+1		;no error
	cp	rc_susp		;suspended task
	jr	z,util_mainloop	;get another
:
	cp	0		;check for command
	jr	nz,util_mainloop
	sys_call	os_in		;get command byte
;
	cp	COMMANDMD		;check for memory dump request
	jr	nz,:+1
	call	util_mdentry
	jp	util_main
:
	cp	COMMANDQUIT		;check for quit
	ret	z
:
	jp	util_mainloop
;
; ==========================================================================
;
;	Entry for Memory Dump program
;
; ==========================================================================
;
util_mdentry
	xor	a		;zero a
	ld	b,a		;and b
	ld	hl,util_mderrhan	;set up error handler
	sys_call	os_erh
	ld	a,5		;enable escape detection
	sys_call	os_esc
	call	util_memdump	;call memory dump
	xor	a		;zero a
	ld	b,a		;and b
	ld	hl,util_errhan	;set up error handler
	sys_call	os_erh
	ld	a,5		;enable escape detection
	sys_call	os_esc
	ret			;return to utility level
;
; ==========================================================================
;
;	Utility screen main redraw routine
;
; ==========================================================================
;
util_mainredraw			;redraw screen
	pushall			;save all registers
	ld	hl,util_mainscrn	;just put out a message
	sys_call	gn_sop
	popall			;get all registers back
	ret
;
util_mainscrn
	vid_win_1	1,0,0,60,8
	db	'Z88 UTILITIES - VERSION 1.00'
	db	'      '
	db	'COPYRIGHT 1990 RICHARD HAW'
	vid_win_2	1,0,0,60,8
	db	0
