;
; ==========================================================================
;
;	This file is the Z-Graphics program.
;
; ==========================================================================
;
zgraph
	ld	hl,zg_buffer	;erase graphics buffer
	call	grph_erase
	xor	a		;set redraw mode
	ld	(zg_rdraw),a
	call	zg_redraw		;and output main screen
	ld	bc,0000h
	ld	hl,zg_buffer
:
	push	bc
	call	grph_setdot
	pop	bc
	inc	b
	inc	b
	inc	c
	ld	a,03fh
	cp	c
	jr	nz,:-1
	call	grph_updatef
	call	pause
	ld	hl,zg_buffer
	call	grph_invert
	call	grph_updatef
	call	pause
;
zg_done
	call	grph_delete		;delete graphics area
	call	reset_win		;reset window
	ret
;
zg_redraw				;redraw screen
	ld	hl,zg_mainwin	;output titled window
	call	title_win_half
	ld	hl,zg_wintextptr	;get start of table
	ld	a,(zg_rdraw)	;and redraw mode
	rlca			;make word offset
	ld	d,0		;and put in de
	ld	e,a
	add	hl,de		;adjust pointer
	ld	e,(hl)		;get address
	inc	hl		;for text
	ld	d,(hl)
	ex	de,hl		;put address in hl
	sys_call	gn_sop		;and output window
	call	grph_create		;redefine graphics area
	ld	hl,zg_buffer	;set buffer
	call	grph_updatef	;and update
	ret
;
; ==========================================================================
;
;	Static data
;
; ==========================================================================
;
zg_mainwin				;main window title
	db	'Z-GRAPH v 1.00',0
;
zg_wintextptr			;table of window text pointers
	dw	zg_win0
	dw	zg_win1
	dw	0
zg_win0				;main command window
	db	12
	db	10,10,'    Z-Graph graphics program'
	db	0
zg_win1				;window 1
	db	12,0
;
; ==========================================================================
;
;	Dynamic data
;
; ==========================================================================
;
	IFDEF	BASIC
zg_dstart	equ	$
	bsz	0850h
	ENDIF
;
	IFDEF	ROM
zg_dstart	equ	(01ffdh-080h)	;allocated down from 01ff
	ENDIF
;
zg_dstatus	equ	zg_dstart		;draw status
zg_rdraw	equ	zg_dstatus+1	;redraw flag
zg_curx	equ	zg_rdraw+1		;current x location
zg_cury	equ	zg_curx+1		;current y location
zg_gxoff	equ	zg_cury+1		;global x offset
zg_gyoff	equ	zg_gxoff+2		;global y offset
zg_buffer	equ	zg_dstart+020h	;graphics buffer
;
; ==========================================================================
;
;	Graphics support utilities
;
;	grph_create - Create a graphics window (number 6)
;
;	grph_delete - Delete a graphics window (number 6)
;
;	grph_update - Update output window
;		HL - Graphics buffer
;		B - Starting row
;		C - Ending row
;
;	grph_updatef - Full screen update entry point of grph_update
;
;	grph_erase - Erase graphics window
;		HL - Graphics buffer
;
;	grph_invert - Invert graphics image
;		HL - Graphics buffer
;
;	grph_outline - Output a single line
;		HL - Graphics buffer
;		A - Line number
;
;	grph_setdot - Set a graphics dot
;		HL - Graphics buffer
;		B - X (column)
;		C - Y (row)
;
;	grph_line - Draw a line
;		HL - Graphics buffer
;		BC - x,y of start of line
;		DE - x,y of end of line
;
;	grph_compadd - Compute buffer address of line E
;		HL - Graphics buffer
;		E - Line to draw
;		returns HL as start of line E, D reset to 0.
;
;	grph_print - Print out the current graphics image
;		HL - Graphics buffer
;		IX - Handle for output file
;
; ==========================================================================
;
grph_create				;create a graphics output window
	ld	bc,mp_gra		;set to define a map
	ld	a,'6'		;set as window 6
	ld	hl,0ffh		;make 255 wide
	sys_call	os_map		;do it
	ret
;
grph_delete				;delete a graphics window
	ld	a,'6'		;window 6
	ld	bc,mp_del		;set for delete
	sys_call	os_map		;do it
	ret
;
grph_erase
	push	hl		;save buffer address
	ld	c,8		;main count
	xor	a		;zero a
:
	ld	b,0		;do 256 times
:
	ld	(hl),a		;zero buffer
	inc	hl		;next byte
	djnz	:-1		;loop till b=0
	dec	c		;next block
	jr	nz,:-2		;loop till done
	pop	hl		;get buffer address back
	ret			;and return
;
grph_updatef			;update full display
	ld	bc,003fh		;full screen
grph_update				;update display
	ld	e,b		;starting row
	call	grph_outline	;output line
	ld	a,b		;get line
	cp	c		;check for done
	ret	z		;and return if so
	inc	b		;next row
	jr	grph_update		;and loop for more
;
grph_outline
	pushall			;save all registers
	call	grph_compadd	;get address of row E
	ld	a,'6'		;window 6
	ld	bc,mp_wr
	sys_call	os_map
	popall			;get registers back
	ret			;return
;
grph_invert				;invert graphics image
	push	hl		;save buffer address
	ld	c,8		;main count
:
	ld	b,0		;do 256 times
:
	ld	a,(hl)		;get byte
	cpl			;complement
	ld	(hl),a		;and put it back
	inc	hl		;next byte
	djnz	:-1		;loop till b=0
	dec	c		;next block
	jr	nz,:-2		;loop till done
	pop	hl		;get buffer address back
	ret			;and return
;
grph_setdot				;set a graphics dot
	push	hl		;save buffer
	ld	e,c		;get y
	call	grph_compadd	;and compute offset
	ld	a,b		;get x
	and	0f8h		;clear lower three bits
	rrca			;and rotate right 3
	rrca			;to divide by 8
	rrca
	ld	d,0		;then put in DE and adjust
	ld	e,a		;buffer address
	add	hl,de
;
;	Now set up the bit mask
;
	ld	a,b		;get x
	and	07h		;get bit number
	ld	b,a		;save it
	ld	a,08h		;set rotate count
	sub	b
	ld	b,a		;and save in b
	ld	a,80h		;set bit 7
:
	rlca			;shift left
	djnz	:-1		;and count
	ld	b,(hl)		;get byte desired
	or	b		;and set bit
	ld	(hl),a		;then save it
	pop	hl		;get buffer address back
	ret			;and return
;
grph_line				;output a line
	push	hl		;save buffer address
	ld	a,d		;get x2
	sub	b		;and subtract x1
	jr	nc,grph_line1	;x2>x1
	push	bc		;swap bc/de
	push	de
	pop	bc
	pop	de
grph_line1
	
	pop	hl		;get buffer back
	ret
;
grph_compadd
	ld	d,0		;set offset into buffer
	push	de		;save row number
	xor	a		;clear carry
	rl	e		;left shift DE 5, doing
	rl	d		;a multiply by 32.
	rl	e
	rl	d
	rl	e
	rl	d
	rl	e
	rl	d
	rl	e
	rl	d
	add	hl,de		;and adjust buffer
	pop	de		;and get de back
	ret			;return
;
grph_print				;print out graphics buffer
	push	hl		;save buffer address
	
	pop	hl		;get buffer address back
	ret