	title	Z-Image routines
;
; ==========================================================================
;
;	This module does an graphics page display
;
; ==========================================================================
;
graph_zientry			;main entry
	ld	hl,0		;save the stack state
	add	hl,sp
	ld	(zi_stack),hl
	ld	hl,0		;zero out handles
	ld	(zi_handle),hl
	ld	(zi_fileptr),hl
	ld	(zi_fileptr+2),hl
	xor	a		;and region status flag
	ld	(graph_defined),a
	
	z88_name	'Z-Image v1.00'	;set program name
	ld	hl,zi_mainwin	;print out window header
	call	title_win_half
:
	ld	hl,zi_gfprompt	;call get file name routine
	ld	de,zi_fname	;set buffer
	ld	c,020h		;set buffer length
	call	get_file_name	;get file name
;
	xor	a		;set up for open call
	ld	b,a		;set page 0 for filename
	inc	a		;set a=1 (open for input)
	ld	c,020h		;set scratch length
	ld	de,zi_scratch	;scratch for expanded file name
	ld	hl,zi_fname		;set file name address
	sys_call	gn_opf		;open file
	jp	c,zi_done		;exit if an open error
	ld	(zi_handle),ix	;save file handle
	ld	hl,zi_mainwin	;and redraw main window
	call	title_win_half
;
;	Now check file to make sure it is a valid Z-Image file
;
zi_start				;start of file read
	ld	ix,(zi_handle)	;get file handle
	ld	a,01h		;set pointer to start of file
	ld	hl,zi_fileptr
	sys_call	os_fwm
	jp	c,zi_error
	call	zi_getbyte
	cp	0ffh
	jr	nz,:+1
	call	zi_getbyte		;get next byte
	cp	'Z'
	jr	nz,:+1
	call	zi_getbyte
	cp	'G'
	jr	nz,:+1
	jr	:+2		;all OK
:
	ld	hl,zi_notfilemsg	;put out message saying not
	sys_call	gn_sop		;a graphics file
	ld	a,sr_pwt		;page wait
	sys_call	os_sr
	jr	c,:-1		;loop on carry
:
	call	graph_create	;create a graphics window
	ld	a,0ffh		;set window flag
	ld	(graph_defined),a
;
;	Process a page loop
;
zi_page
	call	zi_getbyte		;get a byte
	cp	0ffh		;check for EOF
	jp	z,zi_done

	ld	b,64		;line counter
	ld	c,0		;scan line
zi_lineloop
	push	bc		;save scan counter
	ld	hl,graph_buf	;set buffer address
	push	hl		;and graph buffer
	ld	b,32		;load buffer array
:
	push	hl		;save address
	push	bc		;and counter
	call	zi_getbyte		;get a byte
	ld	(hl),a		;and save it
	pop	bc		;restore counters
	pop	hl
	inc	hl		;next buffer location
	djnz	:-1		;loop for 32 words
	pop	hl		;get buffer address back
	pop	bc		;and line number
	ld	d,0		;set row
	ld	e,c
	push	bc		;save counter
	call	graph_disbuf	;and output buffer
	pop	bc		;get counters
	inc	c		;next line
	djnz	zi_lineloop		;loop for next line
;
;	Done with this page, pause and then loop back
;
	ld	a,sr_pwt		;page wait
	sys_call	os_sr
	jp	zi_page		;next page if any
;
zi_done				;exit point
	call	zi_cleanup		;clean up
	ld	hl,(zi_stack)	;put stack back
	ld	sp,hl		;and return
	ret
;
zi_getbyte				;get a byte from the file
	ld	ix,(zi_handle)	;set handle
	sys_call	os_gb		;get byte
	ret	nc
	cp	rc_eof		;test for bad errors
	jr	z,:+1
	cp	rc_hand
	jr	z,:+1
	ret
:
	jp	zi_error
;
zi_error				;error exit routine
	push	af		;save a
	call	zi_cleanup		;cleanup all stuff
	ld	hl,(zi_stack)	;get saved stack
	ld	sp,hl		;restore stack
	pop	af		;must be a system error
	sys_call	gn_err
	ret			;and return
;
zi_cleanup				;cleanup stuff
	ld	a,(graph_defined)	;get graphics flag
	and	a
	jr	z,:+1
	call	graph_delete	;delete graphics region
:
	ld	hl,zi_handle	;close file if need be
	call	close_file
	call	reset_win		;fixup window
	ret			;return
;
util_zierrhan			;error handler
	ret	z		;return if fatal
	cp	rc_esc		;check for esc
	jr	nz,:+1
	sys_call	os_esc		;acknowledge esc
	cp	a		;set flags
	ret			;and return
:
	cp	rc_quit		;check for quit request
	jr	nz,:+1
	call	zi_cleanup
	xor	a
	sys_call	os_bye		;exit appl
:
	cp	rc_draw		;check for redraw
	jr	nz,:+1
	call	util_ziredraw	;redraw screen
	cp	a		;set flags and return
	ret
:
	cp	a		;set flags
	ret			;and return
;
util_ziredraw
	pushall			;save all registers
	ld	hl,zi_mainwin	;set up main window
	call	title_win_half
	popall			;get registers back
	ret
;
;	Display a line on the screen
;
;	de - row to write
;	hl - buffer address
;
graph_disbuf			;display a line
	ld	bc,mp_wr		;set for write
	ld	a,'6'		;and window number
	sys_call	os_map		;draw it
	ret
;
graph_create
	ld	bc,mp_gra		;set to define a map
	ld	a,'6'		;set as window 6
	ld	hl,0ffh		;make 255 wide
	sys_call	os_map		;do it
	ret
;
graph_delete			;delete a graphics window
	ld	a,'6'		;window 6
	ld	bc,mp_del		;set for delete
	sys_call	os_map		;do it
	ret
;
graph_erase
	ld	a,00h		;set mask
	call	graph_buf_fill	;fill buffer
	ld	b,64		;set row count
	ld	d,0
	ld	e,0		;start at row 0
:
	push3
	call	graph_disbuf	;draw out line
	pop3
	inc	e		;next row
	djnz	:-1		;loop till done
	ret			;all done, return
;
graph_buf_fill			;fill graphics buffer with A
	push	bc		;save registers
	push	hl
	ld	b,32		;set count
	ld	hl,graph_buf	;and address
:
	ld	(hl),a		;save mask
	djnz	:-1		;and loop till done
	pop	hl		;and restore registers
	pop	bc
	ret			;then return
;
; ==========================================================================
;
;	Static data for windows and such
;
; ==========================================================================
;
zi_mainwin				;main titled window
	db	'Z-IMAGE GRAPHICS DISPLAY v1.00'
	db	0
zi_gfprompt				;get file prompt
	db	'Z-Image File to Display'
	db	0
zi_notfilemsg			;not a Z-Image file
	db	12,10,10
	vid_just_center
	db	'Not a Z-Image Graphics File',10
	vid_just_normal
	db	0
;
; ==========================================================================
;
;	Dynamic data
;
; ==========================================================================
;
	IFDEF	BASIC
zi_dstart	equ	$
	bsz	080h
	ENDIF
;
	IFDEF	ROM
zi_dstart	equ	(01ffdh-080h)	;allocated down from 01ff
	ENDIF
;
graph_defined	equ	zi_dstart		;defined flag
graph_buf	equ	zi_dstart+1		;graphics buffer
zi_stack	equ	zi_dstart+030h	;stack save
zi_fileptr	equ	zi_dstart+032h	;file pointer
zi_handle	equ	zi_dstart+036h	;file handle
zi_fname	equ	zi_dstart+040h	;File name
zi_scratch	equ	zi_dstart+060h	;scratch space