	title	ROM Utility
;
; ==========================================================================
;
;	ROM Utilities
;
; ==========================================================================
;
	jp	rom_dump
rom_check
	ld	hl,rmc_mainwin	;draw out main window
	sys_call	gn_sop
	ld	hl,rmu_workhdr
	sys_call	gn_sop
;
	ld	b,0feh		;set starting bank
	ld	c,02h		;set number of banks
	call	rmc_chkslot
	ld	hl,rmc_chkmsg	;print out message
	sys_call	gn_sop
	ld	hl,(rmu_crc)	;get checksum
	call	hl_to_hexdis	;display it
	sys_call	os_in
	call	reset_win		;clear display
	ret			;and exit
;
;	Routine to perform a checksum on a ROM slot, b is the starting
;	bank and c is the number of banks to process.
;
rmc_chkslot
	ld	hl,0		;zero out checksum
	ld	(rmu_crc),hl
	ld	a,10h		;set length of buffers
	ld	(rmu_blen),a
:
	ld	hl,0		;zero out offset
	ld	(rmu_offset),hl	;and offset
	ld	de,400h		;set for 1024 loops
:
	call	rmu_workdis		;update working display
	call	rmu_getbuf		;get buffer
	push	bc		;save offset and count
	ld	hl,rmu_buf		;set address
	ld	b,10h		;and count
	push	de		;save offset and count
:
	push	hl		;save buffer address
	push	bc		;and counter
	ld	a,(hl)		;get byte
	ld	hl,(rmu_crc)	;get checksum
	call	crc_update		;update crc
	ld	(rmu_crc),hl	;save crc
	pop	bc		;get counter and address
	pop	hl
	inc	hl
	djnz	:-1		;loop till EOB
	ld	hl,(rmu_offset)	;index offset
	ld	de,0010h
	add	hl,de
	ld	(rmu_offset),hl
	pop	de		;get offset counter back
	pop	bc		;and bank counter
	dec	de		;count this buffer
	ld	a,e
	or	d
	jr	nz,:-2		;loop
	inc	b		;increment bank
	dec	c		;count down bank number
	jr	nz,:-3		;loop for more
	ret			;return
;
;	Get a buffer from EPROM, b is the bank, de is the offset.
;	Data is left in rmu_buf. Buffer length is set by (rmu_blen).
;
rmu_getbuf
	pushall			;save all registers
	ld	c,02		;set segment to map into
	sys_call	os_mpb		;map it in
	push	bc		;save mapping
	ld	hl,(rmu_offset)	;set up offset
	ld	a,h		;and adjust to mapped area
	or	80h
	ld	h,a
	ld	b,0		;set buffer length
	ld	a,(rmu_blen)
	ld	c,a
	ld	de,rmu_buf		;set target buffer
	ldir			;move bytes
	pop	bc		;get mapping back
	sys_call	os_mpb		;and remap
	popall			;return all registers
	ret			;and return from routine
;
;	Put out block and offset display, block is assumed in b, only
;	display offset with 00h as the lower byte (to avoid taking up
;	a lot of time in display).
;
rmu_workdis
	pushall			;save all register
	ld	a,(rmu_offset)	;get lower byte
	and	a
	jr	nz,:+1		;if not zero, skip
	push	bc		;save block
	ld	bc,01501h		;set cursor
	call	gotoxy
	pop	bc		;get block back
	ld	a,b		;get block
	call	a_to_hexdis		;and display it
	ld	bc,02001h		;set cursor
	call	gotoxy
	ld	hl,(rmu_offset)	;get offset
	call	hl_to_hexdis	;and display
:
	popall			;get regs back
	ret			;and return
;
; ==========================================================================
;
;	ROM Dump program
;
; ==========================================================================
;
rom_dump
	ld	hl,rmd_mainwin	;put out main window
	sys_call	gn_sop
;
	z88_name	'ROM Dump v1.00'	;set program name
	ld	hl,rmd_prmpt0	;call get file name routine
	ld	de,rmu_buf		;set buffer
	ld	c,020h		;set buffer length
	call	get_file_name	;get file name
;
	xor	a		;set up for open call
	ld	b,a		;set page 0 for filename
	ld	a,02h		;set a=2 (open for input)
	ld	c,020h		;set scratch length
	ld	de,rmu_scratch	;scratch for expanded file name
	ld	hl,rmu_buf		;set file name address
	sys_call	gn_opf		;open file
	jp	c,rmd_done		;exit if an open error
	ld	(rmu_fhandle),ix	;save file handle
	ld	hl,rmd_mainwin	;and redraw main window
	sys_call	gn_sop
;
	ld	b,0f8h		;set starting bank
	ld	c,08h		;set number of banks
	call	rmd_dumpbank
	jp	rmd_done		;and exit
;
;	Routine to dump a ROM bank, b is the starting
;	bank and c is the number of banks to process.
;
rmd_dumpbank
:
	push	bc		;save bank and whatnot
	ld	a,04h		;write out EPROM header
	ld	(rmu_blen),a
	ld	hl,04002h		;set address (EPROM command)
	ld	(rmu_offset),hl	;(0c000h added in rmd_outbuf)
	ld	a,00h		;set block type
	ld	(rmu_btype),a
	ld	a,b		;set bank
	ld	(rmu_bank),a
	ld	hl,rmu_buf		;initialize header info
	ld	a,02h		;eprom size
	ld	(hl),a
	inc	hl
	ld	a,b		;get bank back
	ld	(hl),a
	ld	hl,0c000h		;set 3rd bank offset
	ld	(rmu_buf+2),hl
	call	rmd_outbuf		;output buffer
	pop	bc		;get bank info back
	ld	hl,0		;zero out offset
	ld	(rmu_offset),hl
	ld	de,0400h		;set for 1024 loops
	ld	a,10h		;set buffer length
	ld	(rmu_blen),a
:
	ld	hl,(rmu_offset)	;get offset
	call	rmu_workdis		;update working display
	call	rmu_getbuf		;get buffer
	call	rmd_outbuf		;output buffer
	push	de		;save de
	ld	de,0010h		;set increment
	add	hl,de		;next block
	ld	(rmu_offset),hl	;save offset
	pop	de		;get counte back
	dec	de		;count this buffer
	ld	a,e
	or	d
	jr	nz,:-1		;loop
	inc	b		;increment bank
	dec	c		;count down bank number
	jr	nz,:-2		;loop for more
	ld	a,01h		;set record type for terminal
	ld	(rmu_btype),a	;hex record (type=1)
	ld	a,00h		;buffer length
	ld	(rmu_blen),a
	ld	hl,04002h		;and offset
	ld	(rmu_offset),hl	;(0c000h added in rmd_outbuf)
	call	rmd_outbuf		;and output
	ret			;return
;
;	Output the data in memory as an INTEL hex record, the offset
;	is biased by 0c000h to eliminate confusion with commands when
;	reading in data.
;
rmd_outbuf
	pushall			;save all registers
	ld	a,':'
	call	rmd_outbyte		;output byte
	xor	a		;zero checksum (1 byte)
	ld	(rmu_crc),a
	ld	a,(rmu_blen)	;get data length
	ld	b,a		;and save a copy
	push	bc
	call	rmd_outhex
	ld	hl,(rmu_offset)	;output offset
	ld	de,0c000h		;offset the offset
	add	hl,de
	ld	a,h
	call	rmd_outhex
	ld	a,l
	call	rmd_outhex
	ld	a,(rmu_btype)	;output type
	call	rmd_outhex
	pop	bc		;get length back as b
	ld	hl,rmu_buf		;and buffer address
	ld	a,b		;check for zero
	and	a
	jr	z,:+2
:
	push	bc		;save counter
	ld	a,(hl)		;get byte
	call	rmd_outhex		;output it
	pop	bc		;get counter
	inc	hl		;up address
	djnz	:-1		;loop till b=0
:
	ld	a,(rmu_crc)		;get checksum
	cpl			;complement it
	inc	a		;and up one
	call	rmd_outhex		;and output it
	ld	a,13		;then a terminal CR
	call	rmd_outbyte
	ld	a,10		;LF
	call	rmd_outbyte
	popall			;get back all registers
	ret			;return
;
rmd_outhex				;output byte as hex
	push	hl		;save hl
	ld	hl,rmu_crc
	push	af		;save a
	add	a,(hl)		;add checksum
	ld	(hl),a		;and save it
	pop	af		;get byte back
	call	a_to_hex		;convert to hex and output
	push	hl
	ld	a,h
	call	rmd_outbyte
	pop	hl
	ld	a,l
	call	rmd_outbyte
	pop	hl		;get hl back
	ret			;and return
;
rmd_outbyte			;output byte in a
	ld	ix,(rmu_fhandle)	;get file handle
	sys_call	os_pb		;put byte
	ret	nc		;return if no error
	jp	rmd_error		;jump to error routine
;
rmd_done				;exit branch for rmd
	call	rmd_cleanup
	call	reset_win	;clear display
	ret
rmd_error				;error branch
	call	rmd_cleanup
	call	reset_win
	jp	exit
;
rmd_cleanup				;cleanup for rmd
	ld	hl,rmu_fhandle	;close file if need be
	call	close_file
	ret
;
; ==========================================================================
;
;	Static Data Area
;
; ==========================================================================
;
rmc_mainwin				;main window
	vid_win_1	1,0,0,88,8
	db	'MEMORY MODULE STATUS - VERSION 1.00'
	vid_win_2	1,0,0,88,8
	db	0
rmc_chkmsg				;checksum message
	db	'Checksum is ',0
rmu_workhdr				;working header
	vid_gotoxy	5,1
	db	'Processing Bank    Offset',0
	db	0
rmd_mainwin				;main window
	vid_win_1	1,0,0,88,8
	db	'ROM DUMP VERSION 1.00'
	vid_win_2	1,0,0,88,8
	db	0
rmd_prmpt0				;output file prompt
	db	'Hex File to Create',0
;
; ==========================================================================
;
;	Dynamic Data Area
;
; ==========================================================================
;
	IFDEF	BASIC
rmu_dstart	equ	$
	bsz	048h
	ENDIF
;
	IFDEF	ROM
rmu_dstart	equ	(01ffdh-048h)	;allocated down from 01ff
	ENDIF
;
rmu_crc	equ	rmu_dstart		;checksum
rmu_fhandle	equ	rmu_dstart+2	;file handle
rmu_bank	equ	rmu_dstart+4
rmu_btype	equ	rmu_dstart+5	;block type
rmu_blen	equ	rmu_dstart+6	;block length
rmu_offset	equ	rmu_dstart+7	;offset
rmu_buf	equ	rmu_dstart+9	;buffer
rmu_scratch	equ	rmu_dstart+29h	;scratch
;
