;
; ==========================================================================
;
;	File: BasicHdr.asm
;
;	Standard header for BASIC programs, this should be included
;	as the first file in the assembly process.
;
;	Two macros MUST be defined before this file is included
;	as indicated below, the first to hold the name of the file
;	to be created when the output file is passed to HX and the
;	second to hold the text for the first line of the BASIC program
;	which is a comment.
;
;	basic_progname	macro
;		db	'filename'
;		endm
;
;	basic_remark	macro
;		db	'BASIC program comment line'
;		endm
;
;	This file generates a header at address 0x0001 to indicate
;	that this is a BASIC program. INTEL HEX output should be used
;	and then passed to the Z88 program HX to generate the program
;	on the Z88.
;
;	This file requires the existance of a routine in the program
;	called 'exit_cleanup'. This routine is called on error exits
;	and should perform any needed cleanup, such as closing all
;	open files.
;
; ==========================================================================
;
;	This version of the header checks for a file with the name of
;	/zmm.bas, and if one is found the program tries to run that
;	program at termination, otherwise the instantation of BASIC
;	is closed out.
;
; ==========================================================================
;
;	Macro section
;
;	basic_line - Define a basic line
;
;	Call form:
;
;		basic_line	line_num,line_data
;
; ==========================================================================
;
basic_line	macro
:start
	db	:end-:start		;length of line
	db	:0&0ffh,(:0<<8)&0ffh	;line number (lsb first)
	db	:1		;actual line
	db	13		;end of line
:end
	endm
;
; ==========================================================================
;
bastok_rem	equ	0f4h	;remark
bastok_page	equ	090h	;page (read)
bastok_if	equ	0e7h	;if
bastok_then	equ	08ch	;then
bastok_print	equ	0f1h	;print
bastok_end	equ	0e0h	;end
bastok_lomem	equ	0d2h	;lomem
bastok_call	equ	0d6h	;call
bastok_run	equ	0f9h	;run
;
	org	0001		;Set zero origin
	dw	2300H		;Set basic offset
	basic_progname		;Set file name
;
;	Set up basic program header
;
	org	2300H	;Set for basic program start
;
;	First line (comment), macro not used here to avoid
;	recursion problems.
;
:
	db	:+1-:-1		;length of line
	db	1&0ffh,(1<<8)&0ffh	;line number (lsb first)
	db	bastok_rem,32
	basic_remark		;comment text
	db	13		;end of line
:
;
	basic_line	0,'bastok_if,32,bastok_page,"<>&2300 ",bastok_then,32,bastok_print,34,"Unable to load",34,":",bastok_end'
	basic_line	0,'bastok_if,32,"$&23C0<>",34,"Kwc8#cPWQ@",34,32,bastok_then,32,bastok_print,34,"Corrupted image",34,":",bastok_end'
	basic_line	0,'bastok_lomem,"=32767"'
	basic_line	0,'bastok_call,32,"&23D0"'
	basic_line	0,'bastok_run,34,"/ZMM.BAS",34'
	db	0,0ffh,0ffh,13	;end of program
	bsz	023c0h-$
	db	'Kwc8#cPWQ@',13
	bsz	023d0h-$
;
;	BASIC error handler
;
	ld	hl,0
	add	hl,sp
	ld	(bstk),hl	;save basic stack
	ld	sp,(01FFEH)	;select safe stack
	xor	a	;new call level
	ld	b,a	;b must be zero
	ld	hl,errhan	;address of this error handler
	sys_call	os_erh	;install new error handler
	ld	(obou),a	;save old error handler call level
	ld	(oerr),hl	;save old error handler address
	call	basic_entry	;call main routine
;
;	Check for the existance of '/ZMM.BAS', and if it is found,
;	simply return to BASIC, otherwise issue fatal error to BASIC
;	error handler.
;
	ld	hl,bas_menuname	;get menu name
	ld	b,0		;local string
	ld	de,bas_menubuf	;scratch
	ld	c,08h
	ld	a,06h		;open DOR
	sys_call	gn_opf		;open it
	jr	c,exit_all_1		;abort from basic
	ld	a,05h		;free DOR
	sys_call	os_dor
	ld	hl,(oerr)		;reinstall old error handler
	ld	a,(obou)
	ld	b,0
	sys_call	os_erh
	ld	sp,(bstk)		;restore stack
	ret			;return to BASIC
bas_menuname
	db	'/ZMM.BAS',0
bas_menubuf
	bsz	9
errhan
	ret	z		;fatal error
	cp	rc_esc		;check for escape
	jr	nz,err1
	sys_call	os_esc		;acknowledge escape
	jr	exit_all		;and exit
err1
	cp	rc_quit		;KILL request
	jr	nz,err2
exit_all
	call	exit_cleanup	;call exit cleanup routine
exit_all_1
	ld	hl,(oerr)		;reinstall old error handler
	ld	a,(obou)
	ld	b,0
	sys_call	os_erh
	ld	sp,(bstk)
	ld	hl,(oerr)
	ld	a,rc_quit-1
	inc	a	;reset zero flag
	scf		;set carry flag
	jp	(hl)	;jump to basics error handler
err2
	cp	a	;reset carry flag
	ret
;
;	Error handler data storage
;
bstk	dw	0
obou	db	0
oerr	dw	0
;
basic_entry
;