;
; ----------------------------------------------------------------------
; ----------------------------------------------------------------------
;
;	file.asm
;
;	System library - file.asm
;
;	File support routines
;
; ----------------------------------------------------------------------
; ----------------------------------------------------------------------
;
; ----------------------------------------------------------------------
;
;	Routine to prompt for and read in a file name.
;
;	Mail support is provided to
;	allow the File utility to be used.
;
;	Window 5 is used for the prompt session.
;
; ----------------------------------------------------------------------
;
;	get_file_name
;
;	Get a file name and return it in buffer
;
;	Entry:
;		HL - pointer to prompt text
;		DE - pointer to buffer
;		C - buffer length (must be at least 20 bytes)
;	Exit:
;		HL - pointer to start of file name
;
; ----------------------------------------------------------------------
;
gfn_init	equ	022H	;Initial gn_sip mode
;
get_file_name
	push3		;save stuff
;
;	Zero out any existing mail
;
	ld	a,sr_wpd
	ld	de,0	;set to 0 to null data
	sys_call	os_sr		;null it
;
	xor	a	;make sure the buffer is empty
	ld	(de),a	;for redraw purposes
;
;	Put out a window and read in file name information
;
	ld	hl,:+6		;output window
	call	:+4
;
	ld	a,gfn_init	;set to scroll if long line
:
	pop3		;get data back
	push3
	ld	b,c	;put buffer length in b
	ld	l,50	;set max visible length
	sys_call	gn_sip	;get input line
	jp	nc,:+2	;exit if all done
	cp	rc_susp	;check is suspended or needs redraw
	jr	z,:+1	;and redraw window if so.
	cp	rc_draw
	jr	z,:+1
	jp	:+2		;other error
:
	ld	hl,:+5		;init window
	call	:+2
;
;	Check for mail
;
	pop3		;get data back
	push3
	ex	de,hl	;put address into hl
	ld	de,:+3		;set for type of mail
	ld	a,sr_rpd	;read mail
	sys_call	os_sr		;do the call
;
	ld	a,gfn_init|1	;existing text
	jr	:-2
;
:
	sys_call	os_pur		;purge keyboard buffer
	pop3
	ret
;
;	Support routines for get_file_name
;
;	Print out (HL) followed by text at (HL) on stack, followed
;	by prompt
;
:
	sys_call	gn_sop
	pop	de	;get return address
	pop	hl	;get data address
	push	hl
	push	de	;put return address back
	sys_call	gn_sop		;and output text
	ld	hl,:+4
	sys_call	gn_sop		;and finish prompt
	ret		;and return
;
;	Data storage
;
:
	db	'NAME',0
:
	vid_win	5,10,0,60,8,1
:
	vid_win_sel_ic	5
	vid_just_center
	vid_bold_toggle
	vid_cursor_toggle
	db	0
:
	vid_bold_toggle
	db	13,10,10
	vid_just_normal
	db	'  File Name ',0
;
; ----------------------------------------------------------------------
;
;	Close file routine, call with address of file handle in HL,
;	will try and close the file if the handle is non-zero, otherwise
;	will return with no action taken.
;
; ----------------------------------------------------------------------
;
close_file
	push	hl	;save address
	ld	e,(hl)	;put handle into de
	inc	hl
	ld	d,(hl)
	ld	a,e	;check for zero
	and	a
	jr	z,:+1
	ld	a,d
	and	a
	jr	z,:+1
	push	de	;put handle in IX
	pop	ix
	sys_call	gn_cl	;close file
	xor	a	;zero a
	ld	(hl),a	;and zero out address
	inc	hl
	ld	(hl),a
:
	pop	hl	;get address back
	ret		;return
;
;
; ----------------------------------------------------------------------
;
;	Close wildcard routine, call with address of wildcard handle in HL,
;	will try and close the wildcard if the handle is non-zero, otherwise
;	will return with no action taken.
;
; ----------------------------------------------------------------------
;
close_wildcard
	push	hl	;save address
	ld	e,(hl)	;put handle into de
	inc	hl
	ld	d,(hl)
	ld	a,e	;check for zero
	and	a
	jr	z,:+1
	ld	a,d
	and	a
	jr	z,:+1
	push	de	;put handle in IX
	pop	ix
	sys_call	gn_wcl	;close wildcard
	xor	a	;zero a
	ld	(hl),a	;and zero out address
	inc	hl
	ld	(hl),a
:
	pop	hl	;get address back
	ret		;return
;