;
; ----------------------------------------------------------------------
;
;	STRING.ASM
;
;	Character and string support routines
;
; ----------------------------------------------------------------------
;
;	The following routines are available:
;
;	a_to_lower - Convert character in A to lower case
;	a_print_out - Print a, making printable
;	a_to_hex - Convert A to hex in HL
;	a_to_hexdis - a_to_hex and display
;	hex_to_a - Convert hex in HL to A
;	hexmem_to_a - Convert hex at (HL) to A
;	hex_to_hl - Convert hex in BC,DE to value in HL
;	hl_to_hex - Convert hl binary to hex in HL
;	hl_to_hexdis - hl_to_hex and print
;	string_to_lower - convert string at (HL) to lower case
;	get_string_length - return A as length of string (HL)
;	increment_file_name - Increment file name at (HL), (DE) is scratch
;	copy_string - Copy (HL) to (DE) till null
;
; ----------------------------------------------------------------------
;
a_to_lower
	cp	'A'	;check if too low
	ret	c
	cp	'Z'+1	;check if too high
	jr	c,:+1
	ret
:
	or	32	;set bit
	ret
;
a_print_out
	cp	20h	;check for control
	jr	nc,:+1
	ld	a,'.'	;make a period
:
	sys_call	os_out	;and output it
	ret
;
a_to_hex
	push	af	;save a copy
	call	a_to_hex_1	;get low 4 bits
	ld	l,a	;and save in l
	pop	af	;get a back
	rrca		;rotate 4 bits down
	rrca
	rrca
	rrca
	call	a_to_hex_1	;convert to ASCII
	ld	h,a	;and put in h
	ret		;all done
a_to_hex_1
	and	0fh	;clear high bits
	cp	0ah	;if greater than 9 increment
	jr	c,:+1
	add	a,07h	;add 7
:
	add	a,030h	;make ASCII
	ret		;and return
;
a_to_hexdis			;a_to_hex and display
	call	a_to_hex	;convert
	ld	a,h	;and display
	sys_call	os_out
	ld	a,l
	sys_call	os_out
	ret		;return
;
hl_to_hex
	push	hl	;save hl
	ld	a,h	;first byte
	call	a_to_hex	;convert
	push	hl
	pop	bc
	pop	hl
	ld	a,l	;second byte
	call	a_to_hex	;convert
	ex	de,hl	;and put in DE
	ret
;
hl_to_hexdis
	push	hl	;save hl
	ld	a,h	;first byte
	call	a_to_hexdis	;convert
	push	hl
	pop	bc
	pop	hl
	ld	a,l	;second byte
	call	a_to_hexdis	;convert
	ex	de,hl	;and put in DE
	ret
;
hexmem_to_a				;convert hex at (HL) to A
	push	de		;save a copy of DE
	ld	a,(hl)		;get first byte
	ld	d,a		;save in d
	inc	hl		;next byte
	ld	a,(hl)		;get next byte
	ld	e,a		;and save as e
	inc	hl		;point to next byte
	push	hl		;save address
	ex	de,hl		;put in hl
	call	hex_to_a		;convert hex
	pop	hl		;get hl and de back
	pop	de
	ret			;and return
;
hex_to_a
	ld	a,h	;get high nibble
	call	hex_to_a_1	;and convert
	and	a	;clear carry
	rla		;and shift over
	rla
	rla
	rla
	ld	h,a	;save back in h
	ld	a,l	;get next nibble
	call	hex_to_a_1	;and convert
	add	a,h	;then add high 4 bits
	ret		;and return
hex_to_a_1
	cp	60h	;check for lower case
	jr	c,:+1	;jump if upper
	sub	020h	;make lower
:
	sub	030h	;Convert from ASCII
	cp	0ah	;check if > 9
	jr	c,:+1	;jump if < 10
	sub	07h	;decrement by 7
:
	ret		;return
;
hex_to_hl			;convert BC,DE in hex to binary in HL
	push	bc	;get high byte
	pop	hl
	call	hex_to_a	;convert
	push	af	;save
	push	de	;get second set
	pop	hl
	call	hex_to_a	;convert
	ld	l,a	;and save
	pop	af
	ld	h,a
	ret		;return
;
string_to_lower			;convert string to lower
	push	hl		;save hl
string_to_lower_1
	ld	a,(hl)		;get byte
	cp	' '		;check for control
	jr	c,string_to_lower_99
	cp	'A'		;check if too low
	jr	c,string_to_lower_2
	cp	'Z'+1		;check if too high
	jr	nc,string_to_lower_2
	or	32		;set bit
	ld	(hl),a		;save it back
string_to_lower_2
	inc	hl		;next byte
	jr	string_to_lower_1	;and loop
string_to_lower_99
	pop	hl		;restore hl
	ret			;and return
;
get_string_length			;return A as length of string (HL)
	xor	a		;zero a and store in B
	ld	b,a
get_string_length1
	ld	a,(hl)		;get byte
	inc	b		;count it
	inc	hl		;next byte
	and	a		;test
	jr	nz,get_string_length1
	dec	b		;adjust count
	ld	a,b		;save in a
	dec	hl		;adjust hl
	ret			;and return
;
increment_file_name			;adjust file name up one
	push	de		;save buffer
	push	hl		;save file address
	call	string_to_lower	;convert file name to lower
	ld	b,0		;get number of segments
	sys_call	gn_prs		;returns segments in B
	pop	hl		;get file name and buffer
	pop	de
	push	hl		;and save back again
	push	de
	xor	a		;set for file name read
	dec	b		;file name segment (zero offset)
	sys_call	gn_esa		;read file name
	jr	c,increment_file_name_err1	;error
	pop	hl		;get buffer address
	push	bc		;save segment number
	push	hl		;and buffer address
	call	get_string_length	;get string length
	ld	c,a		;save in c
	dec	hl		;get last character
increment_file_name_1
	ld	a,(hl)		;get last byte
	cp	'z'		;check for z
	jr	c,increment_file_name_2	;can increment
	ld	a,c		;get length
	cp	0ch		;check length
	jr	nc,increment_file_name_err1	;error
	inc	hl		;next byte
	ld	(hl),'a'-1		;set a
	inc	c		;up length
	jr	increment_file_name_1	;loop
increment_file_name_2			;just count up one char
	inc	(hl)		;up hl
	pop	hl		;get buffer address
	pop	bc		;segment number
	pop	de		;and file name
	ld	a,80h		;set for write
	ld	c,060h		;max length to write
	sys_call	gn_esa		;and write file name
	ret			;and return
;
increment_file_name_err			;error exit
	pop	hl		;get stack back
increment_file_name_err1		;error exit with 2 on stack
	pop	hl
	pop	hl
	scf			;set carry
	ret			;and return
;
copy_string				;copy string
	ld	a,(hl)		;get a
	and	a		;check for zero
	ld	(de),a		;save at (DE) (includes NULL)
	ret	z		;return on zero
	inc	de		;and increment
	inc	hl
	jr	copy_string		;and loop
	
