;
; ==========================================================================
;
;	Z88 Standard library
;
; ==========================================================================
;
;	A set of library routines for the Z88. This library should
;	be included after all other code is assembled.
;
;	Macros defined in sysmac.asm define logical constants which
;	enable compilation of code if the routines are needed, sort
;	of a poor mans linker.
;
; ==========================================================================
;
	ifdef	q$gotoxy
qgotoxy
	ld	a,1	;set up prefix
	sys_call	os_out
	ld	a,'3'
	sys_call	os_out
	ld	a,'@'
	sys_call	os_out
	ld	a,b	;set position
	add	a,32
	sys_call	os_out
	ld	a,c
	add	a,32
	sys_call	os_out
	ret
	endif
;
; ==========================================================================
;
	ifdef	link_force_z88fn
;
force_z88fn			;make sure z88 file name
	ld	b,0bh	;max file name length
force_zfn_nloop
	ld	a,(hl)	;get byte
	call	force_zfn_adj	;make a letter
	ld	(de),a	;save it
	and	a	;check for zero
	ret	z	;and return if so
	inc	hl	;update pointers
	inc	de
	cp	'.'	;check for extension
	jr	z,force_zfn_ext
	djnz	force_zfn_nloop	;loop till max file length
	ld	a,'.'	;put in a period
	ld	(de),a
	inc	de
force_zfn_ext
	ld	b,03h	;set count
force_zfn_eloop
	ld	a,(hl)	;get byte
	call	force_zfn_adj	;make a letter
	ld	(de),a	;save it
	and	a	;check for zero
	ret	z	;and return if so
	inc	hl	;update pointers
	inc	de
	cp	'.'	;check for extension
	jr	z,force_zfn_done
	djnz	force_zfn_eloop	;loop till max ext length
force_zfn_done
	xor	a	;zero out file name
	ld	(de),a
	ret		;and return
;
force_zfn_adj			;adjust a
	and	a	;ignore NULL
	ret	z
	call	qatolower	;make lower case
	cp	'.'	;check for .
	ret	z	;and return if so
	cp	'0'	;adjust if non_numeric
	jr	c,force_zfn_adj	;allways an x
	cp	'9'+1	;check if a number
	ret	c	;and return if so
	cp	'a'	;if < 'a' fix
	jr	c,force_zfn_adj_1
	cp	'z'+1	;if > 'z' fix
	ret	c
force_zfn_adj_1
	ld	a,'x'	;make an x
	ret		;and return
;
	endif
;
clean_file_name			;clean up file name
	push	hl	;buffer
	ld	d,h	;and save in de
	ld	e,l
clean_file_name_1
	ld	a,(hl)	;get byte
	inc	hl	;next byte
	and	a	;set condition flags
	jr	z,clean_file_name_2	;end of string
	cp	'/'	;look for slash
	jr	nz,clean_file_name_1	;loop
	ld	d,h	;save this address
	ld	e,l
	jr	clean_file_name_1	;and loop for more
clean_file_name_2
	pop	hl	;get buffer address back
	ex	de,hl	;exchange
	jp	qstrcpy	;copy (HL) to (DE)	
;
	ifdef	link_display_hex_line
;
display_hex_line
	push	hl	;save buffer address
	ld	a,d	;check for prefix
	and	a
	jr	z,:+1
	ld	a,e
	and	a
	jr	z,:+1
	ex	de,hl	;output prefix text
	sys_call	gn_sop
:
	pop	hl	;get address back
	push	hl	;and save again
	ld	a,10h	;set byte counter
:
	push	af	;save counter
	push	hl	;save address
	ld	a,(hl)	;get byte
	call	qatohexdis	;convert to hex and display it
	ld	a,20h	;and a space
	sys_call	os_out
	pop	hl	;get address and increment
	inc	hl
	pop	af	;get counter
	cp	09h	;check if need extra space
	jr	nz,:+1
	push	af	;save a
	ld	a,20h	;output a space
	sys_call	os_out
	pop	af
:
	dec	a
	jr	nz,:-2	;and loop till done
;
	ld	a,20h	;put out an extra space
	sys_call	os_out
	pop	hl	;get start address back again
	ld	a,10h	;and set counter
:
	push	af	;save counter
	push	hl	;save address
	ld	a,(hl)	;get byte
	cp	20h	;is control
	jr	nc,:+1
	ld	a,'.'	;set to a dot
:
	sys_call	os_out	;and print it
	pop	hl	;get address and increment
	inc	hl
	pop	af	;get counter
	cp	09h	;check if need extra space
	jr	nz,:+1
	push	af	;save a
	ld	a,20h	;output a space
	sys_call	os_out
	pop	af
:
	dec	a
	jr	nz,:-3	;and loop till done
;
	ret		;return
;
	endif
;
	ifdef	link_eprom
ep_verify			;verify EPROM erase
	ld	b,0e0h	;starting bank
	ld	c,02	;bank to map to
:
	push	bc	;save registers
	sys_call	os_mpb	;map bank in
	push	bc	;save mapping
	ld	hl,08000h	;set start address
:
	ld	a,(hl)	;get byte
	cp	0ffh	;check for valid
	jr	nz,:+1	;no good
	inc	hl	;next address
	ld	a,h	;get high byte
	cp	0c0h	;and check for 'overflow'
	jr	nz,:-1
	pop	bc	;get mapping and remap
	sys_call	os_mpb
	pop	bc	;get bank map
	inc	b	;next bank
	ld	a,b
	and	a	;all banks done
	jr	nz,:-2
	ret		;it's zero
:
	pop	bc	;get mapping and remap
	sys_call	os_mpb
	pop	bc	;clean stack
	ld	a,0ffh	;set bad EPROM
	ret		;and return
;
	endif	link_eprom
;
print_freemem			;print free memory (HL) is buf
	push	hl	;save buffer
	call	get_freemem	;get in buffer
	pop	hl	;get buffer address
	sys_call	gn_sop	;and output string
;
get_freemem			;get free memory (string) in (HL)
	ld	a,02h	;get file size
	ld	ix,0ffffh	;for file -1
	ld	de,0	;zero DE
	sys_call	os_frm
	push	hl	;save buffer address
	push	bc	;save lower
	exx		;and put back in hl'
	pop	hl
	ld	de,0010h	;set format string
	exx
	ex	de,hl	;set upper byte
	xor	a	;zero exponent
	ld	c,a
	pop	de	;get buffer address back
	sys_callfp	fp_str	;convert to string
	ret		;return
;
q$float	.set	0ffh
print_long_int			;print long int at (HL) (DE) buf
	push	de	;save buffer
	call	qfpiloadhl	;load into HLhlC
	exx		;get to alt set
	ld	de,0010h	;set format
	exx		;back to main
	sys_callfp	fp_str	;convert to string
	pop	hl	;get buffer address
	sys_call	gn_sop	;and output
	ret		;return
;
dis_userchar			;display user defined character
	push	hl	;save hl
	ld	hl,dis_userchar_data	;set up prefix
	sys_call	gn_sop
	pop	hl	;get hl back
	sys_call	os_out	;then output character
	ret		;and return
dis_userchar_data			;user character prefix
	db	1,'2?',0
;
print_date_time			;print date/time at (HL)
	push	hl	;save date info
	push	de	;and buffer address
	ld	a,27h	;set format
	sys_call	gn_ptm	;convert time
	ex	de,hl	;set hl to end of data
	ld	(hl),0	;set trailing zero
	pop	hl	;get buffer address back
	push	hl
	sys_call	gn_sop	;and output to screen
	pop	de	;get buffer back
	pop	hl	;and data address
	push	de	;save buffer address
	inc	hl	;move up to date
	inc	hl
	inc	hl
	ld	a,0aeh	;set format
	ld	b,0
	ld	c,'/'	;delimiter
	sys_call	gn_pdt	;convert
	ex	de,hl	;set hl to end of data
	ld	(hl),0	;set trailing zero
	pop	hl	;get buffer address back
	sys_call	gn_sop	;and output to screen
	ret
;
set_date_time			;put date/time at (DE)
	push	de	;save buffer
	sys_call	gn_gmt	;get date
	pop	de	;get buffer back
	inc	de	;up 3 byte
	inc	de
	inc	de
	sys_call	gn_gmd	;get time
	ret		;and return
;
; ==========================================================================
;
;	Bar display routines
;
;	These routine set up and display a bar on the screen by using
;	user defined characters.
;
;	bar_display_init - Initialize and put out a blank bar
;		BC - Screen start of bar (will actually start at
;			column -2 due to leading arrow, and
;			extend to column+length+2 due to trailing
;			arrow.
;		A - Length of bar
;
;	bar_display_init_uc - Set up user defined characters. This routine
;		is called by bar_display_init, but may need to be
;		recalled if the task has been suspended causing
;		redefinition of the user defined character set.
;
;	bar_display - Put out a fractional bar graph. The total bar length
;			and current bar length are 4 byte integers
;			as used by the Z88 FP package, and must be
;			less than approx. 2x10^11 (to allow for 
;			multiplys internal to the routine.
;		HLhl - current bar length
;		DEde - total bar length
;		BC - start of bar graph
;		A - Length of bar graph
;
; ==========================================================================
;
	ifdef	link_bar_display
;
bar_display			;put out bar display
	push	bc	;save cursor position
	push	de	;save full length
	ld	de,0	;and set for len * 256
	ld	b,d	;make sure BC is zero
	ld	c,e
	exx
	push	de
	ld	d,a	;length (1 byte shift = *256)
	ld	e,0
	exx
	sys_callfp	fp_mul	;multiply
	exx		;get full length back
	pop	de
	exx
	pop	de
	ld	b,0	;make sure using integer
	sys_callfp	fp_fdiv	;floating divide
	sys_callfp	fp_fix	;convert to an integer
	exx		;now get data
	push	hl	;h=full,l=remainder
	exx
	pop	hl	;now set cursor
	pop	bc
	push	hl
	call	gotoxy
	pop	bc	;get length back from stack
	push	bc
	ld	a,b	;check for zero full bytes
	and	a
	jr	z,bar_display_1a
	ld	a,45h	;full block
bar_display_1
	call	dis_userchar	;display user defined block
	djnz	bar_display_1
bar_display_1a
	pop	hl	;get count back
	ld	h,0	;toss upper byte
	ld	de,0006h	;and multiply by 6
	sys_call	gn_m16
	ld	a,h	;get high byte
	and	a	;check for zero
	ret	z	;and do nothing if so
	add	a,03fh	;make offset
	call	dis_userchar	;output user character
	ret		;and return
;
bar_display_init			;initialize for bargraph
	push	af	;save length
	dec	b	;adjust column position
	dec	b
	call	gotoxy	;position cursor
	ld	a,1	;output left arrow
	sys_call	os_out
	ld	a,241
	sys_call	os_out
	pop	bc	;get length back in b
	ld	a,' '	;set space
bar_display_init_1
	sys_call	os_out	;output space
	djnz	bar_display_init_1	;for full width
	ld	a,1	;output right arrow
	sys_call	os_out
	ld	a,240
	sys_call	os_out
;
bar_display_init_uc		;define characters for bar display
	ld	hl,bar_display_init_data
	sys_call	gn_sop	;output data
	ret		;and return
bar_display_init_data
	db	1,138,'=',40h,0a0h,0a0h,0a0h,0a0h,0a0h,0a0h,0a0h,0a0h
	db	1,138,'=',41h,0b0h,0b0h,0b0h,0b0h,0b0h,0b0h,0b0h,0b0h
	db	1,138,'=',42h,0b8h,0b8h,0b8h,0b8h,0b8h,0b8h,0b8h,0b8h
	db	1,138,'=',43h,0bch,0bch,0bch,0bch,0bch,0bch,0bch,0bch
	db	1,138,'=',44h,0beh,0beh,0beh,0beh,0beh,0beh,0beh,0beh
	db	1,138,'=',45h,0bfh,0bfh,0bfh,0bfh,0bfh,0bfh,0bfh,0bfh
	db	0
;
	endif
;
; ==========================================================================
;
	ifdef	q$float
qfpiloadhl
	push	de	;save de
	ld	e,(hl)	;get low word (HL) in DE
	inc	hl
	ld	d,(hl)
	inc	hl
	push	de	;save low word
	exx		;then put in hl
	pop	hl
	exx
	ld	e,(hl)	;get high word from (HL) in DE
	inc	hl
	ld	d,(hl)
	inc	hl
	ex	de,hl	;swap back to HL
	pop	de	;and get original DE back
	ld	c,0	;then zero C
	ret

qfpiloadde
	push	bc	;save c
	ex	de,hl	;put de into hl (both sets)
	exx
	ex	de,hl
	exx
	call	qfpiloadhl	;load hl
	call	qfpiexchg	;and exchange both sets
	pop	bc	;get c back
	ld	b,0	;zero for int
	ret
;
qfpistore
	push	hl
	ex	de,hl
	exx
	push	hl
	exx
	pop	de
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	pop	de
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ex	de,hl
	ret
;
qfpimove
	push	bc
	ld	bc,4
	ldir
	pop	bc
	ret
;
qfpiexchg
	push	af
	ld	a,b
	ld	b,c
	ld	c,a
	pop	af
	ex	de,hl
	exx
	ex	de,hl
	exx
	ret
;
qfpiaddi			;add DE to (HL)
	push	de
	exx
	pop	de
	exx
	ld	de,0
	ld	b,0
	rst	018h
	db	fp_add
	ret
;
qfpisubi			;subtract DE from (HL)
	push	de
	exx
	pop	de
	exx
	ld	de,0
	ld	b,0
	rst	018h
	db	fp_sub
	ret
	endif
;
; ==========================================================================
;
;	File name processing routines
;
;	Call:	wildcard_match
;	Input:
;		BC - Pointer to scratch buffer (at least 32 bytes)
;		DE - Pointer to wildcard
;		HL - Pointer to file name
;	Output:
;		Fc=1 - Error in either file name or wildcard spec
;		Fz=1 - Wildcard match
;		Fz=0 - No match
;
; ==========================================================================
;
;	The following are support routines for the wildcard processor
;
;	wildcard_match_cln:
;
;		Strips a trailing '.' from the passed data if any
;		is present.
;
;	wildcard_match_wsm
;
;		Call to gn_wsm, except checks for NULL strings and
;		will return a match if both are null.
;
; ==========================================================================
;
	ifdef	link_wildcard
;
wildcard_match			;match filename to wildcard
	push	bc	;save scratch
	push	de	;and wildcard
	push	hl	;and file name
	call	wildcard_match_cln	;clean up file names (strip
	ex	de,hl	;trailing '.' if any) for
	call	wildcard_match_cln	;both name and wildcard
	pop	hl	;get data back
	push	hl
	ld	d,b	;put scratch in de
	ld	e,c
	xor	a	;get file name
	ld	b,0ffh	;last segment
	sys_call	gn_esa
	pop	bc	;get all data back
	pop	de	;(wildcard in (de), buf in (hl)
	pop	hl
	push	hl	;and save again
	push	de
	push	bc
	ld	bc,0010h	;offset
	push	hl	;save first buffer
	add	hl,bc	;in buffer
	push	hl	;save buffer
	ex	de,hl	;put buf in (de) and name in (hl)
	ld	b,0ffh	;last segment
	sys_call	gn_esa
	jr	c,wildcard_match_err5	;error exit
	pop	de	;get wildcard pointer
	pop	hl	;get file name pointer
	call	wildcard_match_wsm
	jr	nz,wildcard_match_err3	;no match, exit
;
;	File name did match, so check for extension
;
;	Stack now has:
;			scratch buffer
;			wildcard
;			file name
;
	pop	hl	;get file name
	pop	bc	;wildcard
	pop	de	;and buf
	push	de	;and save them all back
	push	bc
	push	hl
	call	wildcard_match_getext	;get extension
	jr	c,wildcard_match_err3	;exit on error
	pop	de	;get file name
	pop	de	;and wildcard
	pop	hl	;and buffer
	push	hl	;save buffer
	ld	bc,0010h	;offset
	add	hl,bc	;adjust buffer
	push	hl	;save offset
	ex	de,hl	;get wildcard in (HL)
	call	wildcard_match_getext	;get extension
	jr	c,wildcard_match_err2	;exit on error
	pop	de	;get wildcard pointer
	pop	hl	;get file name pointer
	call	wildcard_match_wsm	;compare them
	ret		;return with flags set
;
wildcard_match_err5		;5 stack exit
	pop	hl
	pop	hl
wildcard_match_err3		;three stack item exit
	pop	hl
wildcard_match_err2		;two stack item exit
	pop	hl
	pop	hl
	ret		;and return
;
;	This routine gets the extension for the file, if none is used
;	DOES set up a null pointer
;
;	DE - buffer
;	HL - filename
;
wildcard_match_getext		;get extension
	push	de	;save buffers
	push	hl
	ld	b,0	;make local
	sys_call	gn_prs
	jr	c,wildcard_match_err2	;problem with syntax
	and	01h	;check for extension
	jr	nz,wildcard_match_getext_1
	pop	hl		;get buffers back
	pop	hl
	xor	a	;zero out (NULL string)
	ld	(hl),a
	ret		;and return
wildcard_match_getext_1		;get the extension
	pop	hl	;get file name
	pop	de	;and buffer
	ld	a,01h	;set to read extension
	ld	b,0ffh	;last segment
	sys_call	gn_esa	;do it
	ret		;and return
;
wildcard_match_wsm		;wildcard match
	ld	a,(de)	;get first string
	and	a	;check for zero
	jr	nz,wildcard_match_wsm_1
	cp	(hl)	;compare to (HL)
	ret	z	;return if zero set
wildcard_match_wsm_1
	ex	de,hl	;put wildcard pointer in hl
	sys_call	gn_wsm	;compare them
	ret		;and return

wildcard_match_cln		;strip trailing '.' from (HL)
	ld	a,(hl)	;get byte
	inc	hl	;up pointer
	and	a	;set flags
	jr	nz,wildcard_match_cln	;loop if not trailing NULL
	dec	hl	;point back to last char
	dec	hl
	ld	a,(hl)	;get byte
	and	a
	cp	'.'	;check for '.'
	ret	nz	;return if not
	xor	a	;zero a and store
	ld	(hl),a
	ret		;then return
;
	endif
;
; ==========================================================================
;
;	Open file for output (with directory create if need be). The
;	fully qualified file name is returned in the original buffer
;	at (HL), which must have a length >=80h.
;
;	open_file
;
;	On entry:
;		HL - Pointer to file name to open
;		DE - Scratch buffer (length >= 80h)
;	On exit:
;		Fc set if error, otherwise handle is returned
;		in IX.
;
; ==========================================================================
;
	ifdef	link_open_file
;
open_file			;open file for output
	push	hl	;save buffers
	push	de
	ld	b,0	;set for local
	ld	c,80h	;and length
	sys_call	gn_fex	;expand file name
	jr	c,open_file_ret2	;return on syntax error
	pop	hl	;get buffer pointer
	pop	de	;back and copy (note de<>hl)
	push	de
	push	hl
	call	qstrcpy	;fully qualified name now in HL
	pop	de
	pop	hl
	push	hl	;and save back
	push	de
open_file_1
	inc	hl	;next byte
	ld	a,(hl)	;find '/'
	cp	'/'
	jr	nz,open_file_1
	ld	b,h	;save in bc, then get de/hl
	ld	c,l
	pop	de	;buffer
	pop	hl	;file name
	push	hl	;push them all back
	push	de
	push	bc	;pointer to separator
open_file_loop
	ld	b,0	;make local
	ld	c,80h	;max buffer length
	ld	a,02	;open for output
	sys_call	gn_opf	;try to open
	jr	nc,open_file_ret3	;opened OK, return
	cp	12h	;check for not found error
	jr	nz,open_file_ret3	;any other errors fatal
open_file_scan_loop
	pop	hl	;get separator pointer
	ld	a,'/'	;and put / back
	ld	(hl),a
open_file_scan
	inc	hl	;next byte
	ld	a,(hl)	;get character
	cp	'/'	;check for delimiter
	jr	z,open_file_nextpath
	cp	0	;check for end of name
	jr	nz,open_file_scan	;next byte
	pop	de	;get buffer
	pop	hl	;and file name
	ld	b,0	;make local
	ld	c,80h
	ld	a,02h	;set for open
	sys_call	gn_opf	;try and open
	ret		;and return
;
open_file_ret3			;3 stack item return
	pop	hl
open_file_ret2			;2 stack item return
	pop	hl
	pop	hl
	ret
;
open_file_nextpath		;open as path
	xor	a	;terminate string
	ld	(hl),a
	push	hl	;and put in bc
	pop	bc	;get pointers back
	pop	de	;buffer
	pop	hl	;full name
	push	hl	;and save back
	push	de
	push	bc
	ld	b,0	;make local string
	ld	c,80h	;and set length
	ld	a,06	;try and get dor
	sys_call	gn_opf
	jr	c,open_file_baddor	;bad dor
	ld	a,05	;free dor
	sys_call	os_dor
	jr	open_file_scan_loop	;and go to next dir
open_file_baddor			;try and create directory
	pop	bc	;separator
	pop	de	;buf
	pop	hl	;filename
	push	hl	;and put back on stack
	push	de
	push	bc
	ld	b,0	;set up for directory create
	ld	c,80h
	ld	a,05	;(create dir)
	sys_call	gn_opf
	jr	c,open_file_ret3	;exit on error
	ld	a,05	;then free dor
	sys_call	os_dor
	jr	open_file_scan_loop	;try till out of data
;
	endif
;
; ----------------------------------------------------------------------
;
;	Routine to prompt for and read in a file name.
;
;	Mail support is provided to
;	allow the File utility to be used.
;
;	Window 5 is used for the prompt session.
;
; ----------------------------------------------------------------------
;
;	get_file_name
;
;	Get a file name and return it in buffer
;
;	Entry:
;		HL - pointer to prompt text
;		DE - pointer to buffer
;		C - buffer length (must be at least 20 bytes)
;	Exit:
;		HL - pointer to start of file name
;
; ----------------------------------------------------------------------
;
	ifdef	link_get_file_name
;
gfn_init	equ	022H	;Initial gn_sip mode
;
get_file_name
	push3		;save stuff
;
;	Zero out any existing mail
;
	ld	a,sr_wpd
	ld	de,0	;set to 0 to null data
	sys_call	os_sr	;null it
;
	xor	a	;make sure the buffer is empty
	ld	(de),a	;for redraw purposes
;
;	Put out a window and read in file name information
;
	ld	hl,:+6	;output window
	call	:+4
;
	ld	a,gfn_init	;set to scroll if long line
:
	pop3		;get data back
	push3
	ld	b,c	;put buffer length in b
	ld	l,50	;set max visible length
	sys_call	gn_sip	;get input line
	jp	nc,:+2	;exit if all done
	cp	rc_susp	;check is suspended or needs redraw
	jr	z,:+1	;and redraw window if so.
	cp	rc_draw
	jr	z,:+1
	jp	:+2	;other error
:
	ld	hl,:+5	;init window
	call	:+2
;
;	Check for mail
;
	pop3		;get data back
	push3
	ex	de,hl	;put address into hl
	ld	de,:+3	;set for type of mail
	ld	a,sr_rpd	;read mail
	sys_call	os_sr	;do the call
;
	ld	a,gfn_init|1	;existing text
	jr	:-2
;
:
	sys_call	os_pur	;purge keyboard buffer
	pop3
	ret
;
;	Support routines for get_file_name
;
;	Print out (HL) followed by text at (HL) on stack, followed
;	by prompt
;
:
	sys_call	gn_sop
	pop	de	;get return address
	pop	hl	;get data address
	push	hl
	push	de	;put return address back
	sys_call	gn_sop	;and output text
	ld	hl,:+4
	sys_call	gn_sop	;and finish prompt
	ret		;and return
;
;	Data storage
;
:
	db	'NAME',0
:
	vid_win	5,10,0,60,8,1
:
	vid_win_sel_ic	5
	vid_just_center
	vid_bold_toggle
	vid_cursor_toggle
	db	0
:
	vid_bold_toggle
	db	13,10,10
	vid_just_normal
	db	'  File Name ',0
;
	endif
;
; ==========================================================================
;
	ifdef	q$fclose
qfclose
	push	hl	;save address
	ld	e,(hl)	;put handle into de
	inc	hl
	ld	d,(hl)
	ld	a,e	;check for zero
	and	a
	jr	nz,:+1
	ld	a,d
	and	a
	jr	nz,:+1
	jr	:+2
:
	xor	a	;zero a
	ld	(hl),a	;and zero out address
	dec	hl
	ld	(hl),a
	push	de	;put handle in IX
	pop	ix
	sys_call	gn_cl	;close file
:
	pop	hl	;get address back
	ret		;return
	endif
;
; ==========================================================================
;
	ifdef	q$wclose
qwclose
	push	hl	;save address
	ld	e,(hl)	;put handle into de
	inc	hl
	ld	d,(hl)
	ld	a,e	;check for zero
	and	a
	jr	nz,:+1
	ld	a,d
	and	a
	jr	z,:+2
:
	xor	a	;zero a
	ld	(hl),a	;and zero out address
	dec	hl
	ld	(hl),a
	push	de	;put handle in IX
	pop	ix
	sys_call	gn_wcl	;close wildcard
:
	pop	hl	;get address back
	ret		;return
	endif
;
; ==========================================================================
;
;	String routines
;
; ==========================================================================
;
	ifdef	q$hltohexdis
q$atohexdis	.set	0ffh
qhltohexdis
	push	hl	;save hl
	ld	a,h	;first byte
	call	qatohexdis	;convert
	push	hl
	pop	bc
	pop	hl
	ld	a,l	;second byte
	call	qatohexdis	;convert
	ex	de,hl	;and put in DE
	ret
	endif
;
; ==========================================================================
;
	ifdef	q$atohexdis
q$atohex	.set	0ffh
qatohexdis			;_atohex and display
	call	qatohex	;convert
	ld	a,h	;and display
	sys_call	os_out
	ld	a,l
	sys_call	os_out
	ret		;return
	endif
;
; ==========================================================================
;
	ifdef	q$hexmemtoa
q$hextoa	.set	0ffh
qhexmemtoa			;convert hex at (HL) to A
	push	de	;save a copy of DE
	ld	a,(hl)	;get first byte
	ld	d,a	;save in d
	inc	hl	;next byte
	ld	a,(hl)	;get next byte
	ld	e,a	;and save as e
	inc	hl	;point to next byte
	push	hl	;save address
	ex	de,hl	;put in hl
	call	qhextoa	;convert hex
	pop	hl	;get hl and de back
	pop	de	ret			;and return
;
; ==========================================================================
;
	ifdef	q$hextohl
q$hextoa	.set	0ffh
qhextohl			;convert BC,DE in hex to binary in HL
	push	bc	;get high byte
	pop	hl
	call	qhextoa	;convert
	push	af	;save
	push	de	;get second set
	pop	hl
	call	qhextoa	;convert
	ld	l,a	;and save
	pop	af
	ld	h,a
	ret		;return
	endif
;
; ==========================================================================
;
	ifdef	link_increment_file_name
;
q$strtolower	.set	0ffh
increment_file_name		;adjust file name up one
	push	de	;save buffer
	push	hl	;save file address
	call	qstrtolower	;convert file name to lower
	ld	b,0	;get number of segments
	sys_call	gn_prs	;returns segments in B
	pop	hl	;get file name and buffer
	pop	de
	push	hl	;and save back again
	push	de
	xor	a	;set for file name read
	dec	b	;file name segment (zero offset)
	sys_call	gn_esa	;read file name
	jr	c,increment_file_name_err1	;error
	pop	hl	;get buffer address
	push	bc	;save segment number
	push	hl	;and buffer address
	call	qstrlen	;get string length
	ld	c,a	;save in c
	dec	hl	;get last character
increment_file_name_1
	ld	a,(hl)	;get last byte
	cp	'z'	;check for z
	jr	c,increment_file_name_2	;can increment
	ld	a,c	;get length
	cp	0ch	;check length
	jr	nc,increment_file_name_err1	;error
	inc	hl	;next byte
	ld	(hl),'a'-1	;set a
	inc	c	;up length
	jr	increment_file_name_1	;loop
increment_file_name_2		;just count up one char
	inc	(hl)	;up hl
	pop	hl	;get buffer address
	pop	bc	;segment number
	pop	de	;and file name
	ld	a,80h	;set for write
	ld	c,060h	;max length to write
	sys_call	gn_esa	;and write file name
	ret		;and return
;
increment_file_name_err			;error exit
	pop	hl	;get stack back
increment_file_name_err1		;error exit with 2 on stack
	pop	hl
	pop	hl
	scf		;set carry
	ret		;and return
;
	endif
;
; ==========================================================================
;
;	Lower level routines, these routines MAY be called by other
;	routines in stdlib.asm, but MAY NOT call other routines in
;	stdlib.asm.
;
; ==========================================================================
;
	ifdef	q$atolower
qatolower
	cp	'A'	;check if too low
	ret	c
	cp	'Z'+1	;check if too high
	jr	c,:+1
	ret
:
	or	32	;set bit
	ret
	endif
;
; ==========================================================================
;
	ifdef	q$aprint
qaprint
	cp	20h	;check for control
	jr	nc,:+1
	ld	a,'.'	;make a period
:
	sys_call	os_out	;and output it
	ret
	endif
;
; ==========================================================================
;
	ifdef	q$atohex
qatohex
	push	af	;save a copy
	call	qatohex_1	;get low 4 bits
	ld	l,a	;and save in l
	pop	af	;get a back
	rrca		;rotate 4 bits down
	rrca
	rrca
	rrca
	call	qatohex_1	;convert to ASCII
	ld	h,a	;and put in h
	ret		;all done
qatohex_1
	and	0fh	;clear high bits
	cp	0ah	;if greater than 9 increment
	jr	c,:+1
	add	a,07h	;add 7
:
	add	a,030h	;make ASCII
	ret		;and return
	endif
;
; ==========================================================================
;
	ifdef	q$hltohex
qhltohex
	push	hl	;save hl
	ld	a,h	;first byte
	call	qatohex	;convert
	push	hl
	pop	bc
	pop	hl
	ld	a,l	;second byte
	call	qatohex	;convert
	ex	de,hl	;and put in DE
	ret
	endif
;
; ==========================================================================
;
	ifdef	q$hextoa
qhextoa
	ld	a,h	;get high nibble
	call	qhextoa_1	;and convert
	and	a	;clear carry
	rla		;and shift over
	rla
	rla
	rla
	ld	h,a	;save back in h
	ld	a,l	;get next nibble
	call	qhextoa_1	;and convert
	add	a,h	;then add high 4 bits
	ret		;and return
qhextoa_1
	cp	60h	;check for lower case
	jr	c,:+1	;jump if upper
	sub	020h	;make lower
:
	sub	030h	;Convert from ASCII
	cp	0ah	;check if > 9
	jr	c,:+1	;jump if < 10
	sub	07h	;decrement by 7
:
	ret		;return
	endif
;
; ==========================================================================
;
	ifdef	q$strcpy
qstrcpy			;copy string
	ld	a,(hl)	;get a
	and	a	;check for zero
	ld	(de),a	;save at (DE) (includes NULL)
	ret	z	;return on zero
	inc	de	;and increment
	inc	hl
	jr	qstrcpy	;and loop
	endif
;
; ==========================================================================
;
	ifdef	q$strcmp
qstrcmp			;compare strings
	ld	a,(de)	;get first byte
	cp	(hl)	;compare with (hl)
	ret	nz	;return if no match
	inc	de	;next byte
	inc	hl
	djnz	qstrcmp	;and loop
	xor	a	;set flags
	ret		;and return
	endif
;
	ifdef	q$strcmpn
qstrcmpn			;compare strings (null term)
	ld	a,(de)	;get first byte
	and	a	;check for null
	ret	z	;and return if so
	cp	(hl)	;compare with (hl)
	ret	nz	;return if no match
	ld	a,(hl)	;check it for zero
	and	a
	ret	z	;and return if so
	inc	de	;next byte
	inc	hl
	jr	qstrcmpn	;and loop
	endif
;
; ==========================================================================
;
	ifdef	q$strtolower
qstrtolower			;convert string to lower
	push	hl	;save hl
qstrtolower_1
	ld	a,(hl)	;get byte
	cp	' '	;check for control
	jr	c,qstrtolower_99
	cp	'A'	;check if too low
	jr	c,qstrtolower_2
	cp	'Z'+1	;check if too high
	jr	nc,qstrtolower_2
	or	32	;set bit
	ld	(hl),a	;save it back
qstrtolower_2
	inc	hl	;next byte
	jr	qstrtolower_1	;and loop
qstrtolower_99
	pop	hl	;restore hl
	ret		;and return
	endif
;
; ==========================================================================
;
	ifdef	q$strlen
qstrlen			;return A as length of string (HL)
	xor	a	;zero a and store in B
	ld	b,a
qstrlen1
	ld	a,(hl)	;get byte
	inc	b	;count it
	inc	hl	;next byte
	and	a	;test
	jr	nz,qstrlen1
	dec	b	;adjust count
	ld	a,b	;save in a
	dec	hl	;adjust hl
	ret		;and return
	endif
;
; ==========================================================================
;
