;
; ==========================================================================
;
;	Library for ZBrowse
;
; ==========================================================================
;
; ==========================================================================
;
;	Output character routine, the character is handled as follows:
;
;	1 - Video control, output as a literal
;	2 - repeat space count
;	3 - Video control, a 1 is output, and the next character is output
;		without translation or token substitution.
;	128-255 - Tokenized text
;
; ==========================================================================
;
zb_outchar				;output character
	push	af		;save character
	ld	a,(zb_expflag)	;get expansion flag and check
	and	a		;to see if working on expansion
	jr	z,zboc_1a		;and skip if so
	cp	1		;check if one (repeat flag)
	jr	nz,zboc_0
	pop	af		;get repeat count back
	ld	b,a
	ld	a,' '
zboc_1
	sys_call	os_out
	djnz	zboc_1
	xor	a		;zero out expansion flag
	ld	(zb_expflag),a
	ret			;and return
zboc_0				;literal character output
	xor	a		;zero out expansion flag
	ld	(zb_expflag),a
	pop	af		;get character back
	jp	zboc_norm		;and output normal
;
zboc_1a	pop	af		;get character back
	cp	1		;check for video prefix
	jp	z,zboc_norm		;output and return
	cp	3		;check for literal video output
	jr	nz,zboc_1b
	ld	a,2		;set flag
	ld	(zb_expflag),a
	ld	a,1		;and output a video control (1)
	jp	zboc_norm
zboc_1b
;
	cp	2		;repeated space character
	jr	nz,zboc_2
	ld	a,01h		;set expansion flag for the
	ld	(zb_expflag),a	;next call to zb_outchar
	ret			;and return
zboc_2
	cp	7		;check for end of line
	jr	nz,zboc_3
	sys_call	gn_nln
	ret
zboc_3
	cp	16		;check for video format
	jr	nc,zboc_4
	push	af		;save byte
	ld	a,1		;output command byte
	sys_call	os_out
	pop	af
	sub	8		;shift down
	ld	c,a
	ld	b,0
	ld	hl,zb_vfmat_table	;get code table
	add	hl,bc		;compute offset
	ld	a,(hl)		;get byte
	sys_call	os_out		;and output
	ret			;and return
zboc_4
	cp	32		;check for lower line draw
	jr	nc,zboc_5
	push	af		;save it
	ld	hl,zb_ldraw_prefix	;output prefix
	sys_call	gn_sop
	ld	hl,zb_ldraw_table	;get table
	pop	af		;get byte back
	sub	16		;fix offset
	ld	c,a		;and put in bc
	ld	b,0
	add	hl,bc		;compute address
	ld	a,(hl)		;get byte
	sys_call	os_out		;and output
	ret			;then return
zboc_5
	cp	128		;check for token
	jr	c,zboc_norm		;and skip token output if not
	and	a		;clear carry flag
	rl	a		;multiply A times 2
	ld	c,a		;and put in bc
	ld	b,0
	ld	hl,(zb_toffset)	;get offset to token table
	add	hl,bc		;start offset
	push	hl		;save it
	call	zb_loadhltoff
	ex	de,hl		;put in de
	pop	hl		;get table address back
	inc	hl		;next offset
	inc	hl
	call	zb_loadhltoff
	ex	de,hl		;swap start into hl, end into de
zboc_tok
	ld	a,l		;check for de=hl
	cp	e
	jr	nz,zboc_tok1
	ld	a,h
	cp	d
	jr	nz,zboc_tok1
	ret			;end of tokens
zboc_tok1
	ld	a,(hl)		;get byte
	push	hl		;save regs
	push	de
	sys_call	os_out		;normal output
;	call	zboc_outchar	;mapped output
	pop	de		;get regs back
	pop	hl
	inc	hl		;next byte
	jr	zboc_tok		;loop
;
zboc_norm
	sys_call	os_out		;normal character output
	ret			;return
;
zb_loadhltoff			;load hl with (hl)+token offset
	push	de		;save de
	ld	e,(hl)		;and get (hl) in de
	inc	hl
	ld	d,(hl)
	ld	hl,(zb_offset)	;get token offset
	add	hl,de		;and add in offset
	pop	de		;get de back
	ret			;and return
;
; ==========================================================================
;
;	Save and restore the fields screens.
;
; ==========================================================================
;
zb_screensave
	ld	a,01h		;save user screen
	sys_call	os_sr		;do it
	ld	(zb_screenhandle),ix	;save handle
	ret			;and return
;
zb_screenrestore
	ld	a,02h		;restore user screen
	ld	ix,(zb_screenhandle)	;get handle
	sys_call	os		;do it
	ret			;and return
;
; ==========================================================================
;
;	Output string at (HL) with token expansion
;
; ==========================================================================
;
zb_sop				;output a string
	ld	a,(hl)		;get byte
	and	a		;set flags
	ret	z		;return on zero
	push	hl		;save hl
	call	zb_outchar	;print it
	pop	hl		;get hl back
	inc	hl		;next byte
	jr	zb_sop
;