;
; ==========================================================================
;
;	Z88 Data Browse Program (ZBrowse)
;
; ==========================================================================
;
zb_offset	equ	zb_buf
zb_toffset	equ	zb_buf+2
zb_hoffset	equ	zb_buf+4
zb_coffset	equ	zb_buf+6
zb_curtopic	equ	zb_buf+8
zb_curhelp	equ	zb_buf+10
zb_numref	equ	zb_buf+12
zb_curref	equ	zb_buf+13
zb_expflag	equ	zb_buf+14
zb_lasttopic	equ	zb_buf+16
zb_reftable	equ	zb_buf+18
;
;
zb_main				;main entry point
	ld	hl,(zb_pname)	;set program name
	sys_call	dc_nam
;
; ==========================================================================
;
;	Set up the screen, and then check to make sure there is an
;	EPROM in the third slot, if not, put out an error message
;	and exit the program.
;
; ==========================================================================
;
	ld	hl,zb_mainwin	;put out main window
	sys_call	gn_sop
	mapmem	03,
;
;	Put out the main window, and 
;
zbrowse				;main entry point
;
;	Get all of the address offsets
;
	ld	hl,zb_text		;get main offset
	ld	(zb_offset),hl	;and save it
	push	hl		;save a couple copies
	push	hl
	ld	e,(hl)		;offset to token table
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)		;offset to help
	inc	hl
	ld	b,(hl)
	inc	hl
	ld	(zb_coffset),hl	;offset to custom char table
	pop	hl		;get offset start
	add	hl,de		;compute token offset
	ld	(zb_toffset),hl	;and save
	pop	hl		;get start offset again
	add	hl,bc		;compute help offset
	ld	(zb_hoffset),hl	;and save
	ld	(zb_curtopic),hl	;set starting topic
	ld	(zb_lasttopic),hl	;and last topic (same)
	xor	a		;zero out expansion flag
	ld	(zb_expflag),a
	call	zb_mainwin	;init windows
;
zb_main				;main loop point
	ld	hl,(zb_curtopic)	;set current topic
	call	zb_drawtopic	;and output
;
zb_input			;input scan location
	sys_call	os_in		;get a key
	jr	c,zb_input	;loop on carry
	call	zb_a_to_lower	;make lower case
	and	a		;check for zero and if so
	jr	z,zb_input1	;get another byte
	cp	13		;look for enter
	jr	nz,zb_noret
zb_select
	ld	hl,(zb_curtopic)	;save current topic as last
	ld	(zb_lasttopic),hl
	ld	a,(zb_curref)	;get current reference number
	ld	hl,zb_reftable	;get table address
	rlc	a		;multiply by two
	ld	c,a		;and put in bc
	ld	b,0
	add	hl,bc		;adjust address
	ld	e,(hl)		;put address in DE
	inc	hl
	ld	d,(hl)
	ex	de,hl		;and get in hl
	ld	(zb_curtopic),hl	;make specified reference current
	jp	zb_main		;and make new
zb_noret
	cp	'c'		;check for table of contents
	jr	nz,zb_notoc
	ld	hl,(zb_hoffset)	;get start of text
	ld	(zb_curtopic),hl	;and make it the current
	jp	zb_main
zb_notoc
	cp	'q'		;check for quit
	ret	z
	cp	'b'		;check for back
	jr	nz,zb_input	;not a valid character
;
zb_selback			;go back to last selection
	ld	hl,(zb_lasttopic)	;make current topic last topic
	ld	(zb_curtopic),hl	;and update
	jr	zb_main
zb_input1
	sys_call	os_in		;get byte
	jp	c,zb_input	;back to start if preempt
;
	cp	252		;left arrow, go back
	jr	z,zb_selback
;
	cp	253		;right arrow, select
	jr	z,zb_select
;
	cp	251		;shift up, first ref
	jr	nz,zb_nosup
	xor	a		;first ref
	jr	zb_changeref_single
;
zb_nosup
	cp	250		;shift down, last ref
	jr	nz,zb_nosdown
	ld	a,(zb_numref)	;last ref
	dec	a
	jr	zb_changeref_single
;
zb_nosdown
	cp	254		;down arrow
	jr	nz,zb_nodown
	ld	a,(zb_numref)	;get number of references
	ld	c,a		;and put in c
	ld	a,(zb_curref)	;get current reference
	inc	a		;up to match offset
	cp	c		;check for already at end
	jr	z,zb_iadone
	dec	a		;adjust offset
	ld	b,a		;save current in b
	inc	a		;up index
	jr	zb_changeref	;and output new
zb_nodown
	cp	255		;up arrow
	jr	nz,zb_noup
	ld	a,(zb_curref)	;get current reference
	ld	b,a		;save in b
	and	a		;count down
	jr	z,zb_iadone	;at zero, change
	dec	a		;move down
	jr	zb_changeref	;update reference
zb_noup
zb_iadone
	jp	zb_input	;back to read input routine
;
zb_changeref_single			;change to ref (A)
	push	af		;save af
	ld	a,(zb_curref)	;get current reference
	ld	b,a		;and put it in b
	pop	af		;then get a back
zb_changeref			;change from ref (B) to (A)
	push	af		;save new
	ld	a,b		;get old
	call	zb_drawref_n	;put out normal
	pop	af		;get new
	ld	(zb_curref),a	;and save as new ref number
	call	zb_drawref_r	;put out reverse
	jr	zb_iadone	;and return
	
	
	ret			;and return
