;
; ==============================================================
;
;	Z88 communications program
;
; ==============================================================
;
zcp_max_err	equ	0bh		;retry count
;
zcp_video_cls	macro		;clear video screen def
	db	13,13,13
	endm
;
outcom_t	macro			;put character out the com port
	ld	ix,(zcp_com_handle)	;get handle
	rst	20h		;and do call
	db	03CH		;(os_pb)
	endm
;
; ==============================================================
;
;	Start of program
;
; ==============================================================
;
zcp_main				;main entry point
	ld	hl,0		;reset all data
	add	hl,sp		;get stack location
	ld	(zcp_stack),hl	;and save it
	ld	hl,zcp_pname	;set program name
	sys_call	dc_nam
	ld	hl,0		;zero out all handles
	ld	(zcp_com_handle),hl
	ld	(zcp_file_handle),hl
	ld	(zcp_wild_handle),hl
	ld	a,00h		;starting value for macbinary
	ld	(zcp_macbinary_flag),a	;and set macbinary flags
	xor	a		;zero a
	ld	(zcp_macbinary_auto),a
	ld	a,':'		;set prompt character
	ld	(zcp_prompt_char),a
;
	call	zcp_open_com	;open com port for output
;
	ld	hl,zcp_mainwin	;output initial screen
	sys_call	gn_sop
	ld	hl,zcp_shareware_msg	;and shareware message
	sys_call	gn_sop
	jr	zcp_main_menu_init_1
;
zcp_main_menu_init
	ld	hl,zcp_main_screen_msg	;main screen message
	sys_call	gn_sop
zcp_main_menu_init_1
	ld	hl,zcp_main_menu_msg	;output main menu text
	call	zcp_output_menu
zcp_main_menu			;main menu character get
	sys_call	os_in		;get character from input
	jr	c,zcp_main_menu	;loop on error
	call	a_to_lower		;make lower case
	cp	'a'		;ascii file send
	jp	z,zcp_send_text
	cp	'g'		;go to terminal
	jp	z,zcp_goto_terminal
	cp	'o'		;options
	jp	z,zcp_options_menu
	cp	'r'		;receive file
	jp	z,zcp_receive_file
	cp	's'		;send file
	jp	z,zcp_send_file
;
	ifdef	zcp_remote_asmflag
	cp	't'		;remote mode
	jp	z,zcp_remote_mode_start
	endif
;
	cp	'x'		;exit program
	jp	z,zcp_alldone
	jr	zcp_main_menu
;
	jp	zcp_main_menu	;then go back to main menu
;
; ==============================================================
;
;	Send ascii file
;
; ==============================================================
;
zcp_send_text			;send ascii file
	ld	hl,zcp_sndtxt_msg0	;output message
	call	zcp_gnsop
	call	zcp_local_getline	;get filename
	jp	c,zcp_main_menu_init
;
;	Now open file
;
	ld	a,01h		;open for input
	ld	bc,0080h		;local and length
	ld	hl,zcp_buf		;file name
	ld	de,zcp_file_name	;expanded name
	sys_call	gn_opf		;open
	jr	nc,zcp_sndtxt_1	;no error
	call	zcp_local_fileerr	;put out error message
	jp	zcp_main_menu_init	;and return
zcp_sndtxt_1
	ld	(zcp_file_handle),ix	;save handle
	call	zcp_tran_msg	;transfer screen
	call	zcp_tran_line_init	;intialize for line count
zcp_sndtxt_loop
	ld	ix,(zcp_file_handle)	;get handle
	sys_call	os_gb		;get a byte
	jr	nc,zcp_sndtxt_l1	;no error
	cp	rc_eof		;end of file
	jr	z,zcp_sndtxt_done
	jr	zcp_sndtxt_loop	;otherwise loop
zcp_sndtxt_l1
	push	af		;save byte
	outcom_t			;and output character
	pop	af		;get it back
	cp	13		;and check for cr
	jr	nz,zcp_sndtxt_loop	;not eof, go for more
	call	zcp_tran_num_out	;count this line
	ld	hl,zcp_prompt_char	;address of prompt
zcp_wait_loop			;wait for prompt character
	push	hl		;save prompt address
	call	zcp_getcomchar	;get a character
	pop	hl		;get hl back
	cp	(hl)		;compare with a
	jr	nz,zcp_wait_loop	;loop till it does
	jr	zcp_sndtxt_loop	;loop for more
zcp_sndtxt_done			;done with file tran
	ld	hl,zcp_file_handle	;close out the file
	call	close_file
	jp	zcp_main_menu_init
;
; ==============================================================
;
;	Options menu
;
; ==============================================================
;
zcp_options_menu			;options menu
	ld	hl,zcp_options_menu_msg	;output menu
	call	zcp_output_menu
	ld	hl,zcp_options_macbin_msg0	;macbinary flag
	sys_call	gn_sop
	ld	a,(zcp_macbinary_flag)	;get flag
	and	a		;set condition flags
	jr	nz,zcp_options_menu_1
	ld	hl,zcp_options_macbin_msg1
	jr	zcp_options_menu_2
zcp_options_menu_1
	ld	hl,zcp_options_macbin_msg2
zcp_options_menu_2
	sys_call	gn_sop		;output mode flag
	ld	bc,00e02h		;pos 2,14
	call	gotoxy
	ld	a,(zcp_prompt_char)	;and output the prompt char
	sys_call	os_out
	ld	hl,zcp_options_macbin_msg3	;final switch
	sys_call	gn_sop
;
zcp_options_main			;main character loop
	sys_call	os_in		;get a character
	call	a_to_lower		;convert to lower case
	jr	c,zcp_options_main	;loop on character
	cp	'm'		;macbinary option
	jr	z,zcp_options_macbinary
	cp	'p'		;prompt text
	jr	z,zcp_options_prompt
	cp	'x'		;exit
	jp	z,zcp_main_menu_init	;back to main menu
	jr	zcp_options_main	;loop till done
;
zcp_options_macbinary			;toggle macbinary flag
	ld	a,(zcp_macbinary_flag)	;get flag address
	cpl	a
	ld	(zcp_macbinary_flag),a	;and save back
	jr	zcp_options_menu	;loop back to menu
;
zcp_options_prompt			;set prompt
	ld	hl,zcp_options_prompt_msg	;output prompt
	sys_call	gn_sop
zcp_options_prompt_1
	sys_call	os_in		;get a character
	jr	c,zcp_options_prompt_1	;loop on error
	ld	(zcp_prompt_char),a	;save prompt character
	ld	hl,zcp_options_prompt_msg1	;clear and reset
	sys_call	gn_sop		;the cursor
	jr	zcp_options_menu	;loop back to menu
;
; ==============================================================
;
;	Receive file
;
; ==============================================================
;
zcp_receive_file			;receive file
	ld	hl,zcp_rcvfile_msg0	;output message
	call	zcp_gnsop
	ld	hl,zcp_buf		;set address
	call	zcp_local_getline	;get filename
	jp	c,zcp_main_menu_init
	call	zcp_no_xon		;turn off xon/xoff
	call	zcp_init_xmodem	;init here (needed for Macbin support)
	call	zcp_receive_xfile	;receive XModem file
	call	zcp_restore_xon
	jp	zcp_main_menu_init	;loop back to main menu
;
; ==============================================================
;
;	Receive XModem file routine
;
; ==============================================================
;
zcp_receive_xfile
;
;	Open file (name in zcp_buf)
;
	xor	a		;zero a
	ld	(zcp_fblock_flag),a	;and set first block flag
	ld	a,(zcp_macbinary_auto)	;check for auto macbinary
	and	a
	jr	nz,zcp_rcvfile_noopen
	ld	a,02h		;open for output
	ld	bc,0080h		;local and length
	ld	de,zcp_file_name	;expanded name
	ld	hl,zcp_buf		;file name
	sys_call	gn_opf		;open
	ret	c		;return if error
	ld	(zcp_file_handle),ix	;save handle
	jr	zcp_rcvfile_noopen_1	;jump to screen output
zcp_rcvfile_noopen
	ld	de,zcp_file_name	;expanded name
	xor	a		;null
	ld	(de),a		;zero file name
zcp_rcvfile_noopen_1
	call	zcp_tran_msg	;transfer screen
	call	zcp_tran_block_init	;intialize screen
zcp_rcvfile_start
	call	zcp_com_flush	;flush com buffer
	ld	a,nak		;send starting <nak>
	outcom_t		
zcp_rcvfile_next
	ld	hl,zcp_block_number	;increment block number
	inc	(hl)
	call	zcp_tran_num_out	;output block number
	call	zcp_get_xblock	;get a block
	jp	c,zcp_rcvfile_err	;return if an error
	jr	z,zcp_rcvfile_done	;return if end of data
	ld	a,(zcp_macbinary_flag)	;check for macbinary mode
	and	a
	jr	z,zcp_not_first_block	;skip it
;
;	Now have an xmodem block in hand, check to see if it is the
;	first block, and if so, evaluate for MacBinary protocol. If a
;	valid block, get file name, close current file if open, and open
;	corrected output file
;
	ld	a,(zcp_fblock_flag)	;get flag
	and	a		;set condition flags
	jr	nz,zcp_not_first_block
	ld	a,0ffh		;set first block flag
	ld	(zcp_fblock_flag),a
	ld	a,(zcp_buf)		;check byte 0
	and	a
	jr	nz,zcp_rcvfile_err	;not a macbinary block
	ld	a,(zcp_buf+74)	;check byte 74
	and	a
	jr	nz,zcp_rcvfile_err	;not a macbinary block
	ld	a,(zcp_buf+82)	;check byte 82
	and	a
	jr	nz,zcp_rcvfile_err	;not a macbinary block
	ld	a,(zcp_buf+1)	;get name length
	ld	c,a		;and put in bc
	ld	b,0
	ld	hl,zcp_buf+2	;copy file name into zcp_buf
	ld	de,zcp_buf1
	push	de		;save this buf addr
	ldir
	xor	a		;make sure trailing zero byte
	ld	(de),a
	pop	hl		;get source addr back
	ld	de,zcp_buf		;and adjust to valid name
	call	force_z88fn
;
	ld	hl,zcp_file_handle	;close first file if open
	call	close_file
	ld	de,zcp_file_name	;get expanded name location
	ld	a,02h		;open for output
	ld	bc,0080h		;local and length
	ld	hl,zcp_buf		;file name
	sys_call	gn_opf		;open
	ret	c		;return if error
	ld	(zcp_file_handle),ix	;save handle
	call	zcp_tran_msg	;redraw transfer screen
	call	zcp_tran_block_init	;and block message
	jr	zcp_rcvfile_nextblock	;and get the next block
;
zcp_not_first_block
	ld	ix,(zcp_file_handle)	;get file handle
	ld	bc,0080h		;transfer 128 bytes
	ld	de,0		;buffer
	ld	hl,zcp_buf		;from buffer to file
	sys_call	os_mv		;do it
;
zcp_rcvfile_nextblock
	ld	a,ack		;ack and get another
	outcom_t			;block
	jp	zcp_rcvfile_next	;loop for more
;
zcp_rcvfile_done			;done with file tran
	ld	a,ack		;send out final <ack>
	outcom_t		
	ld	hl,zcp_file_handle	;close out the file
	call	close_file
	ret			;and return
;
zcp_rcvfile_err
	ld	hl,zcp_file_handle	;close file if open
	call	close_file
	scf			;indicate an error
	ret			;and return
;
; ==============================================================
;
;	Send file
;
; ==============================================================
;
zcp_send_file			;send file
	ld	hl,zcp_sndfile_msg0	;output message
	call	zcp_gnsop
	call	zcp_local_getline	;get filename
	jp	c,zcp_main_menu_init
	call	zcp_send_xfile	;send XModem file
	jp	zcp_main_menu_init
;
; ==============================================================
;
;	Send XModem file routine
;
; ==============================================================
;
zcp_send_xfile
;
;	Open file (name in zcp_buf)
;
	ld	a,01h		;open for input
	ld	bc,0080h		;local and length
	ld	hl,zcp_buf		;file name
	ld	de,zcp_file_name	;expanded name
	sys_call	gn_opf		;open
	ret	c		;return if error
	ld	(zcp_file_handle),ix	;save handle
	call	zcp_init_xmodem	;init for xmodem
	call	zcp_tran_msg	;transfer screen
	call	zcp_tran_block_init	;intialize screen
	ld	a,(zcp_macbinary_flag)	;test for macbinary
	and	a		;set condition flags
	jr	z,zcp_sndfile_start	;no macbinary
	ld	a,esc		;send out send request
	outcom_t			;to the other computers
	ld	a,'b'		;macterminal or compatible
	outcom_t			;program
zcp_sndfile_start
	call	zcp_getcomchar_xto60	;get a character
	jr	nc,zcp_sndfile_start_1	;no timeout
	ret			;return
zcp_sndfile_start_1
	cp	nak		;wait for starting nak
	jr	nz,zcp_sndfile_start	;loop till <nak> received
	ld	a,(zcp_macbinary_flag)	;test for macbinary
	and	a		;set condition flags
	jr	z,zcp_sndfile_next	;nope, ignore
;
;	Set up header for MacBinary
;
	ld	hl,zcp_file_name	;copy file name to scratch
	ld	de,zcp_buf1
	push	de		;save two copies of name addr
	push	de
	call	copy_string
	pop	hl		;get buffer address back
	call	clean_file_name	;and clean up file name
	ld	bc,0080h		;copy dummy xmodem block over
	ld	de,zcp_buf		;set buffer address
	ld	hl,zcp_macbinary_default_block
	ldir			;copy dummy macbinary header
	pop	hl		;get name addr
	push	hl		;and save a sec
	call	get_string_length	;get length
	ld	(zcp_buf+1),a	;save length
	pop	hl		;get name addr
	ld	de,zcp_buf+2	;and location
	call	copy_string		;and copy
	ld	ix,(zcp_file_handle)	;get file handle
	ld	a,02h		;and read extent
	ld	de,0
	sys_call	os_frm
	ld	hl,zcp_buf+83	;start of data fork length
	ld	(hl),d		;save length (MSB first)
	inc	hl
	ld	(hl),e
	inc	hl
	ld	(hl),b
	inc	hl
	ld	(hl),c
	ld	hl,zcp_block_number	;increment block number
	inc	(hl)
	call	zcp_send_xblock	;send block
	jp	c,zcp_sndfile_done	;close and return if an error
	call	zcp_tran_num_out	;output block number
zcp_sndfile_next
	ld	hl,zcp_block_number	;increment block number
	inc	(hl)
	call	zcp_tran_num_out	;output block number
	ld	ix,(zcp_file_handle)	;get file handle
	ld	bc,0080h		;get 128 bytes
	ld	de,zcp_buf		;buffer
	ld	hl,0		;from file to buffer
	sys_call	os_mv		;do it
	jr	nc,zcp_sndfile_noteof	;no error
	cp	rc_eof		;check for end of file
	jr	z,zcp_sndfile_eof	;jump if ok
	call	zcp_sndfile_done	;close out file
	scf			;error
	ret
zcp_sndfile_eof			;end of file, fill out buffer
	ld	a,c		;check no bytes sent
	cp	080h
	jr	z,zcp_sndfile_eot	;end of transmission
	and	a		;check if padding needed
	jr	nz,zcp_sndfile_padit	;and do it if need be
	xor	a		;zero a
	jr	zcp_sndfile_block	;and output the block
zcp_sndfile_padit
	xor	a		;zero a
zcp_sndfile_eof_pad
	ld	(de),a		;pad out field
	inc	de
	djnz	zcp_sndfile_eof_pad	;loop till buffer full
	jr	zcp_sndfile_block	;handle this block
zcp_sndfile_noteof
	ld	a,0ffh		;set a
zcp_sndfile_block			;send out the block
	push	af		;save eof flag
	call	zcp_send_xblock	;send block
	jr	nc,zcp_sndfile_block_1	;no error
	pop	af		;clean stack
	scf			;set carry
	ret			;and return
zcp_sndfile_block_1
	pop	af		;get eof flag
	and	a		;set condition flags
	jr	nz,zcp_sndfile_next	;next block
zcp_sndfile_eot
	call	zcp_com_flush	;flush input buffer
	ld	a,eot		;all done, send out eot
	outcom_t			;output <eot>
zcp_sndfile_eot_ok
	ld	bc,00e10h		;1 minute timeout
	call	zcp_getcomchar_to	;get a character
	jr	nc,zcp_sndfile_eot_2	;no timeout
	call	zcp_sndfile_done	;close out file
	scf			;set error flag
	ret			;and return
zcp_sndfile_eot_2
	cp	ack		;check for <ack>
	jr	nz,zcp_sndfile_eot	;send another <eot>
zcp_sndfile_done			;done with file tran
	ld	hl,zcp_file_handle	;close out the file
	call	close_file
	ret			;and return
;
; ==============================================================
;
;	Go to terminal via CLI command
;
; ==============================================================
;
zcp_goto_terminal			;go to terminal via CLI
	call	zcp_close_com	;close out com port
	ld	hl,zcp_terminal_cli	;copy CLI string to local
	ld	de,zcp_buf
	push	de		;save a copy for a few microsec
	call	copy_string
	pop	hl
	ld	bc,0003h		;length
	sys_call	dc_icl
;
;	Now make a call to a system input routine to start .CLI
;	execution.
;
	ld	bc,1		;set for zero wait
	sys_call	os_tin		;call get standard input
	call	zcp_open_com	;open com port back up
	jp	zcp_main_menu_init	;and return
;
; ==============================================================
;
;	Remote mode
;
; ==============================================================
;
	ifdef	zcp_remote_asmflag
;
zcp_remote_mode_start
	ld	hl,zcp_remote_init_msg	;initial message
	call	zcp_outstring
zcp_remote_mode
	ld	hl,zcp_remote_winmsg	;set up Z88 screen
	sys_call	gn_sop
zcp_remote_main
	ld	hl,zcp_remote_prompt	;output com port prompt
	call	zcp_outstring	;output to com port
zcp_remote_menugc
	call	zcp_getcomchar	;get a character
	call	a_to_lower		;convert to lower
	cp	'c'		;change dir
	jp	z,zcp_remote_changedir
	cp	'e'		;erase a file
	jp	z,zcp_remote_erase
	cp	'f'		;files listing
	jp	z,zcp_remote_files
	cp	'i'		;information screen
	jp	z,zcp_remote_info
	cp	'o'		;options
	jp	z,zcp_remote_options
	cp	'r'		;receive file
	jp	z,zcp_remote_receive
	cp	's'		;send file
	jp	z,zcp_remote_send
	cp	't'		;type file
	jp	z,zcp_remote_type
	cp	'?'		;help
	jp	z,zcp_remote_help
	cp	'x'		;exit
	jp	z,zcp_remote_exit
;
;	If running Macbinary, check for <esc>b to initiate a transfer
;
	cp	esc
	jr	nz,zcp_remote_mode	;no escape
	ld	a,(zcp_macbinary_flag)	;check if macbinary enabled
	and	a		;set condition flags
	jr	z,zcp_remote_mode
	call	zcp_getcomchar	;get another character
	cp	'b'		;check for 'b'
	jr	nz,zcp_remote_menugc
;
	ld	a,0ffh		;set auto macbinary flag
	ld	(zcp_macbinary_auto),a
	call	zcp_init_xmodem	;init here (needed for Macbin support)
	call	zcp_no_xon		;turn flow control off
	call	zcp_receive_xfile	;get a file
	call	zcp_restore_xon
	xor	a		;zero auto macbinary flag
	ld	(zcp_macbinary_auto),a
	jp	zcp_remote_mode
;
; ==============================================================
;
;	REMOTE - Change current directory
;
;	This function changes both the current directory and the
;	current name.
;
;	Since no checking of the information is done by the system the
;	following procedure is used:
;
;	A user specified path is read in
;	That path/filename is fully expanded
;	the expanded name is used to open a wildcard handle
;	the first wildcard match is read in which will result in one of
;		two conditions, either it is a root directory, in which
;		case it will return a DOR handle of 081h, or it is the
;		name of a directory, in which case it will return a 
;		handle of 12. The cases are handled appropriately from
;		that point.
;
; ==============================================================
;
zcp_remote_changedir
	ld	hl,zcp_rmc_dirmsg0	;put out starting message
	call	zcp_outstring
	call	zcp_print_currentdev	;print current device and
	call	zcp_print_currentdir	;print current directory
	ld	hl,zcp_rmc_dirmsg1	;and next message
	call	zcp_outstring
;
	ld	hl,zcp_buf		;set buffer address
	call	zcp_getline		;get input line
	jp	c,zcp_remote_mode	;loop on no entry
;
;	now that we have a file name, fully expand it
;
	ld	bc,0080h		;set local and length
	ld	hl,zcp_buf		;get buffer
	ld	de,zcp_buf1		;full expanded location
	sys_call	gn_fex		;expand name
	jp	c,zcp_rmc_error	;error in name
	sys_call	gn_nln
	ld	hl,zcp_buf		;print name
	sys_call	gn_sop
;
;	Now take that name and open a wildcard handle with it
;
	xor	a		;set mode
	ld	b,0		;local string
	ld	hl,zcp_buf1		;and file name
	sys_call	gn_opw		;open wildcard handle
	jr	c,zcp_rmc_error
	ld	de,zcp_buf		;expansion space
	ld	c,80h		;expansion length
	sys_call	gn_wfn		;fetch next match
	jr	c,zcp_rmc_wdone
	cp	081h		;check for :RAM. DOR
	jr	z,zcp_rmc_device
	cp	012h		;check for directory
	jr	z,zcp_rmc_dir
zcp_rmc_wdone			;error while reading wildcard
	sys_call	gn_wcl		;clear handle for wildcard
	jr	zcp_rmc_error	;and error exit
;
zcp_rmc_error
	ld	hl,zcp_rmc_emsg_0	;put out error message
	call	zcp_outstring
	jp	zcp_remote_mode	;and loop back to menu
;
zcp_rmc_device			;just a device
	ld	hl,zcp_buf		;set for buffer
	push	hl		;save for a moment
	ld	bc,sp_dev		;set for device
	sys_call	os_sp		;do it
	xor	a		;zero path out
	pop	hl		;get address
	ld	(hl),a		;and store NULL
	ld	bc,sp_dir		;dir mode
	sys_call	os_sp		;do it
	jp	zcp_remote_mode	;and return to menu
;
zcp_rmc_dir
	ld	hl,zcp_buf		;set buffer position
zcp_rmc_dir_1
	ld	a,(hl)		;get character
	inc	hl		;next byte
	cp	'/'		;find first /
	jr	nz,zcp_rmc_dir_1	;loop till do find it
	dec	hl		;go back to /
	push	hl		;save this spot
	ld	bc,sp_dir		;set current directory
	sys_call	os_sp
	pop	hl		;get buffer back
	xor	a		;and zero it
	ld	(hl),a
	ld	hl,zcp_buf		;get buffer
	ld	bc,sp_dev		;and reset device
	sys_call	os_sp
	jp	zcp_remote_mode	;back to menu
;
;
; ==============================================================
;
;	REMOTE - Erase a file
;
; ==============================================================
;
zcp_remote_erase
	ld	hl,zcp_rme_msg0	;put out initial message
	call	zcp_outstring
	call	zcp_get_filename	;get name of the file
	jp	z,zcp_remote_mode	;back to menu if no entry
	ld	hl,zcp_rme_msg1
	call	zcp_outstring
	ld	hl,zcp_buf
	call	zcp_outstring
	ld	b,0		;set to local address
	ld	hl,zcp_buf		;for file name
	sys_call	gn_del		;try and delete
	call	c,zcp_print_fileerr	;print an error message
	jp	zcp_remote_mode	;back to menu
;
; ==============================================================
;
;	REMOTE - Files listing
;
; ==============================================================
;
zcp_remote_files
	ld	hl,zcp_rmf_msg0	;output initial msg
	call	zcp_outstring
	call	zcp_print_currentdev	;print device
	call	zcp_print_currentdir	;and directory
	ld	hl,zcp_rmf_msg1	;and next part of msg
	call	zcp_outstring
	ld	bc,0080h		;local string and scratch length
	ld	hl,zcp_wildcard_spec	;wildcard spec
	ld	de,zcp_buf		;scratch
	push	bc		;save for a moment
	push	de
	sys_call	gn_fex		;expand file name
	pop	hl		;get back for wildcard open
	pop	bc
	ld	a,00h		;set mode
	sys_call	gn_opw
	jp	c,zcp_fatal_exit
	ld	(zcp_wild_handle),ix	;save handle
zcp_rmf_loop
	ld	ix,(zcp_wild_handle)	;get handle
	ld	c,80h		;scratch size
	ld	de,zcp_buf		;scratch
	sys_call	gn_wfn		;get next wildcard
	jr	nc,zcp_rmf_pwild
	cp	rc_eof		;check for finished
	jr	z,zcp_rmf_done
	jp	zcp_fatal_exit
zcp_rmf_pwild			;print out file name
	push	af		;save dor type (A)
	ld	hl,zcp_buf		;set name buffer
	ld	d,h		;and put in de
	ld	e,l
zcp_rmf_pwild_1			;find last / to print just
	ld	a,(hl)		;the file name
	inc	hl		;next byte
	and	a
	jr	z,zcp_rmf_pwild_2	;end of string
	cp	'/'		;look for /
	jr	nz,zcp_rmf_pwild_1	;loop
	ld	d,h		;save address in de
	ld	e,l
	jr	zcp_rmf_pwild_1
zcp_rmf_pwild_2
	pop	af		;get dor type back
	cp	11h		;is it a file
	jr	z,zcp_rmf_pwild_file
	cp	12h		;it is a directory
	jr	z,zcp_rmf_pwild_dir
	jp	zcp_rmf_loop	;loop for more
;
zcp_rmf_pwild_dir			;print out directory
	call	zcp_rmf_get_dor	;print file and get dor
	ld	a,dr_fre		;free dor handle
	sys_call	os_dor
	ld	hl,zcp_rmf_msg_dir	;and directory text
	call	zcp_outstring
	jr	zcp_rmf_loop	;and loop
;
zcp_rmf_pwild_file			;print out file name
	call	zcp_rmf_get_dor	;print file and get dor
	ld	b,'X'		;get extent
	ld	c,4		;data length
	ld	de,zcp_buf1		;place for data
	push	de		;save buffer address
	ld	a,dr_rd		;read dor
	sys_call	os_dor		;do it
	pop	hl		;get buffer address back
	call	fpi_loadhl		;and get as an int
	ld	de,zcp_buf		;expand in zcp_buf
	push	de		;save zcp_buf address
	exx			;set up format control
	ld	de,0000h		;general format
	exx
	sys_callfp	fp_str		;convert to string
	pop	hl		;get buffer address back
	push	ix		;save dor handle
	call	zcp_outstring	;and print it
	ld	a,13		;put out cr
	call	zcp_outchar
	pop	ix		;get dor handle back
	ld	a,dr_fre		;free dor handle
	sys_call	os_dor
	jp	zcp_rmf_loop	;and loop
;
zcp_rmf_done			;exit files listing
	ld	hl,zcp_wild_handle	;close handle
	call	close_wildcard
	jp	zcp_remote_mode	;back to menu
;
zcp_rmf_dorerr			;error opening dor
	pop	hl		;clean stack
	jp	zcp_rmf_loop
;
zcp_rmf_get_dor
	ex	de,hl		;put address in hl
	ld	b,12h		;padded length
	call	zcp_outstring_pad	;write it out
;
	ld	a,06h		;open DOR
	ld	bc,0080h		;local and length
	ld	de,zcp_buf1		;scratch
	ld	hl,zcp_buf		;file name
	sys_call	gn_opf		;do it
	jr	c,zcp_rmf_dorerr	;return on error
	ld	b,'C'		;output creation date/time
	call	zcp_rmf_print_dor
	ld	b,'U'		;output update date/time
	call	zcp_rmf_print_dor
	ret			;return
;
zcp_rmf_print_dor			;get dor and print date/time
	ld	de,zcp_buf1		;put at buf1
	ld	c,06h		;length of time/date info
	ld	a,dr_rd
	sys_call	os_dor
	ret	c		;return on error (no info)
	push	ix		;save ix
	ld	hl,zcp_buf1+3	;date address
	ld	de,zcp_buf1+40h	;offset for output
	push	de		;save a copy
	ld	a,28h		;format bytes
	ld	b,0
	ld	c,'/'		;delimiter
	sys_call	gn_pdt
	ex	de,hl		;trailing zero
	ld	(hl),0
	pop	hl		;get buffer address back
	ld	b,0ah		;padded length
	call	zcp_outstring_pad	;write it out
;
	ld	hl,zcp_buf1+40h	;expanded text
	ld	hl,zcp_buf1		;time address
	ld	de,zcp_buf1+40h	;offset for output
	push	de		;save a copy
	ld	a,21h		;time format byte
	sys_call	gn_ptm		;convert time
	ex	de,hl		;trailing zero
	ld	(hl),0
	pop	hl		;get buffer address back
	ld	b,0ah		;padded length
	call	zcp_outstring_pad	;write it out
	pop	ix		;get handle back
	ret
;
; ==============================================================
;
;	REMOTE - Information screen
;
; ==============================================================
;
zcp_remote_info
	ld	hl,zcp_rmi_msg_0	;put out initial message
	call	zcp_outstring
	call	zcp_print_currentdev	;output current device
	call	zcp_print_currentdir
	ld	hl,zcp_rmi_msg_2	;and free memory
	call	zcp_outstring
	ld	hl,zcp_buf		;get free memory
	push	hl		;save a copy
	call	get_freemem
	pop	hl		;get buf address back
	call	zcp_outstring	;and print it
	ld	hl,zcp_rmi_msg_99	;final text block
	call	zcp_outstring
	jp	zcp_remote_mode	;back to menu
;
; ==============================================================
;
;	REMOTE - Options (not used in version 1.00)
;
; ==============================================================
;
zcp_remote_options
	jp	zcp_remote_mode	;back to menu
;
; ==============================================================
;
;	REMOTE - Receive binary
;
; ==============================================================
;
zcp_remote_receive
	ld	hl,zcp_rmtrcv_msg0	;output message
	call	zcp_outstring	;to remote
	ld	hl,zcp_gfn_msg	;and file prompt
	call	zcp_outstring
	ld	hl,zcp_buf		;set buffer address
	call	zcp_getline		;get filename
	jp	c,zcp_remote_mode
	ld	hl,zcp_rmtrcv_msg1
	call	zcp_outstring
	call	zcp_send_xfile	;send XModem file
	jr	c,zcp_rmtrcv_2	;handle error return
zcp_rmtrcv_1
	call	zcp_getcomchar	;get a com character
	cp	13		;wait for cr
	jr	nz,zcp_rmtrcv_1
	jp	zcp_remote_mode
zcp_rmtrcv_2
	ld	hl,zcp_rmtrcv_errmsg	;output error message
	call	zcp_outstring
	jp	zcp_remote_mode
;
; ==============================================================
;
;	REMOTE - Send binary
;
; ==============================================================
;
zcp_remote_send
	ld	hl,zcp_rmtsnd_msg0	;output message
	call	zcp_outstring	;to remote
	ld	hl,zcp_gfn_msg	;and file prompt
	call	zcp_outstring
	ld	hl,zcp_buf		;set buffer address
	call	zcp_getline		;get filename
	jp	c,zcp_remote_mode
	ld	hl,zcp_rmtsnd_msg1
	call	zcp_outstring
	call	zcp_no_xon		;turn off xon/xoff
	call	zcp_init_xmodem	;initialize for xmodem
	call	zcp_receive_xfile	;send XModem file
	push	af		;save flags
	call	zcp_restore_xon	;restore flow control
	pop	af		;get flags back
	jr	c,zcp_rmtsnd_2	;handle error return
zcp_rmtsnd_1
	call	zcp_getcomchar	;get a com character
	cp	13		;wait for cr
	jr	nz,zcp_rmtsnd_1
	jr	zcp_rmtsnd_3
zcp_rmtsnd_2
	ld	hl,zcp_rmtrcv_errmsg	;output error message
	call	zcp_outstring
zcp_rmtsnd_3
	jp	zcp_remote_mode	;back to menu
;
; ==============================================================
;
;	REMOTE - Type a file
;
; ==============================================================
;
zcp_remote_type
	ld	hl,zcp_rmt_msg0	;put out initial message
	call	zcp_outstring
	call	zcp_get_filename	;get file name
	jp	c,zcp_remote_mode	;return on carry
	ld	hl,zcp_rmt_msg1	;put out start leader
	call	zcp_outstring
	ld	bc,0080h		;set local and scratch length
	ld	hl,zcp_buf		;file name
	ld	de,zcp_buf1		;scratch
	ld	a,01h		;open for input
	sys_call	gn_opf
	jr	c,zcp_rmt_error	;unable to open file
	ld	(zcp_file_handle),ix	;save handle
zcp_rmt_loop
	ld	ix,(zcp_file_handle)	;get handle
	sys_call	os_gb		;get byte
	jr	c,zcp_rmt_rerr	;read error
	call	zcp_outchar		;output character
	jr	zcp_rmt_loop
zcp_rmt_rerr
	cp	rc_eof		;check if eof error
	jr	z,zcp_rmt_eof	;print error message
	call	zcp_print_fileerr
zcp_rmt_eof
	ld	hl,zcp_file_handle	;close out file
	call	close_file
	jp	zcp_remote_mode
zcp_rmt_error
	call	zcp_print_fileerr	;print error message
	jp	zcp_remote_mode	;back to menu
;
; ==============================================================
;
;	REMOTE - help screen
;
; ==============================================================
;
zcp_remote_help
	ld	hl,zcp_remote_help_msg	;send message text to remote
	call	zcp_outstring
	jp	zcp_remote_main
;
; ==============================================================
;
;	REMOTE - exit remote
;
; ==============================================================
;
zcp_remote_exit
	ld	hl,zcp_remote_exit_msg	;put out closing message
	call	zcp_outstring
	jp	zcp_main_menu_init
	endif
;
; ==============================================================
;
;	Support routines
;
;	zcp_cleanup - close all open stuff
;	zcp_getcomchar - get a com port character
;	zcp_getcomchar_to - get a com character with timeout
;		on input, BC is timeout in ticks
;	zcp_getcomchar_xto1 - get a com char with 1 second timeout
;	zcp_getcomchar_xto10 - get a com char with 10 second timeout
;	zcp_getline - get CR terminated input line in (HL)
;	zcp_local_getline - get line local (Cf set on no input)
;	zcp_gnsop - output local text as heading
;	zcp_outstring - output (HL) to com port (text mode)
;	zcp_outstring_pad - output (HL) to com port (text mode)
;		string is padded to length in B
;	zcp_outchar - output A to com port (text mode)
;	zcp_outchar_t - output A to com port (no filter - transparent)
;	zcp_output_menu - output menu in (HL)
;	zcp_print_currentdev - print current device (remote)
;	zcp_print_currentdir - print current dir (remote)
;	zcp_reset_comport - close/open com port
;	zcp_print_fileerr - print out a file error
;	zcp_get_filename - get a file name with a prompt
;		returns the fully qualified filename in zcp_buf, uses
;		both zcp_buf and zcp_buf1
;	zcp_tran_block_init - init for block transfers
;	zcp_tran_line_init - init for line transfers
;	zcp_tran_num_out - output number (line/block)
;	zcp_get_xblock - get xmodem block, Fc on error, Fz=0 on eot
;	zcp_send_xblock - send xmodem block
;	zcp_init_xmodem - initialize for xmodem
;	zcp_init_xblock - initialize for a new xmodem block
;	zcp_com_flush - flush out com port
;	zcp_block_msg - output block transfer screen last message
;
; ==============================================================
;
zcp_get_xblock			;get xmodem block
	call	zcp_init_xblock	;initialize for a block transfer
zcp_gxblck_1
	ld	hl,zcp_block_errcnt	;count error
	dec	(hl)
	jr	nz,zcp_gxblck_2
	ld	hl,zcp_emsg_errcnt	;last message
	call	zcp_block_msg	;and output
	call	zcp_tran_errnum_out	;output error number
	scf			;error getting packet
	ret
zcp_gxblck_2
	call	zcp_getcomchar_xto10	;get with timeout
	jr	c,zcp_gxblck_terror	;timeout error
	cp	soh		;check for start of header
	jr	z,zcp_gxblck_4	;jump if ok
	cp	eot		;check for end of transmission
	ret	z
	ld	hl,zcp_emsg_badheader	;set last message
	jr	zcp_gxblck_error	;and go to error branch
zcp_gxblck_4
	call	zcp_getcomchar_xto1	;get char
	jr	c,zcp_gxblck_terror	;timeout error
	ld	(zcp_block_num),a	;save as block number
	call	zcp_getcomchar_xto1	;get next char
	jr	c,zcp_gxblck_terror	;timeout error
	cpl			;complement
	ld	b,a		;and put in b
	ld	hl,(zcp_block_num)	;and compare with the block
	cp	b		;number
	jr	z,zcp_gxblck_5	;no problem
	ld	hl,zcp_emsg_badblknum
	jr	zcp_gxblck_error
zcp_gxblck_5
	ld	b,080h		;set block count
	ld	de,zcp_buf		;and buffer address
	ld	hl,zcp_block_check	;and checksum address
zcp_gxblck_6
	push	bc		;save count
	push	de		;and buffer address
	push	hl		;and checksum address
	call	zcp_getcomchar_xto1	;get a character
	jr	nc,zcp_gxblck_7	;no timeout
	pop	hl		;clear stack
	pop	de
	pop	bc
	jr	zcp_gxblck_terror	;timeout error
zcp_gxblck_7
	pop	hl		;get chksum, buff add and
	pop	de		;counter back
	pop	bc
	ld	(de),a		;save byte
	inc	de		;count it
	add	a,(hl)		;update checksum
	ld	(hl),a
	djnz	zcp_gxblck_6	;count and loop
	push	hl		;save checksum addr
	call	zcp_getcomchar_xto1	;get checksum
	pop	hl		;get our chk addr back
	jr	c,zcp_gxblck_terror	;timeout error
	cp	(hl)		;compare with memory
	jr	nz,zcp_gxblck_chkerr	;checksum error
	ld	a,0ffh		;set non-zero return
	and	a
	ret			;then return
zcp_gxblck_chkerr
	ld	hl,zcp_emsg_badchk	;bad checksum
	jr	zcp_gxblck_error
;
;	Error handling for receive block, call a purge routine, send
;	back a <nak>, and then try to get the block again.
;
zcp_gxblck_terror
	ld	hl,zcp_emsg_timeout	;timeout error
zcp_gxblck_error
	call	zcp_block_msg
	call	zcp_tran_errnum_out	;output error number
zcp_gxblck_err1
	call	zcp_getcomchar_xto1	;get byte with timeout
	jr	nc,zcp_gxblck_err1	;loop till timeout
	ld	a,nak		;output a nak
	outcom_t		
	jp	zcp_gxblck_1
;
zcp_block_msg			;put out error message
	push	hl		;save addr
	ld	hl,zcp_emsg_blank	;blank out error message
	call	zcp_block_msg_1	;output blank
	pop	hl		;get msg back and fall through
zcp_block_msg_1
	push	hl		;save hl
	ld	bc,02b04h		;set cursor
	call	gotoxy
	pop	hl		;get message back
	sys_call	gn_sop		;and output
	ret
;
zcp_send_xblock			;send xmodem block
	call	zcp_init_xblock	;init for block transfer
	xor	a		;zero a
	ld	hl,zcp_buf		;buffer address
	ld	b,080h		;set count
zcp_send_xbs_csum
	add	a,(hl)		;add in count
	inc	hl		;increment
	djnz	zcp_send_xbs_csum	;loop till done
	ld	(zcp_block_check),a	;save it
	ld	(zcp_buf_chksum),a	;and a copy in the block
zcp_send_xbs_1
	ld	hl,zcp_block_errcnt	;get error count address
	dec	(hl)		;count down
	jr	nz,zcp_send_xbs_2	;not out of tries yet
	scf			;error
	ret
zcp_send_xbs_2
	ld	a,soh		;start of header
	ld	(zcp_buf_blkstrt),a	;and put in position
	ld	a,(zcp_block_number)	;get block number
	ld	(zcp_buf_blkstrt+1),a	;block number
	cpl			;1s complement
	ld	(zcp_buf_blkstrt+2),a	;complement of block number
;
;
	ifdef	hardware_io		;direct hardware manipulation
;
;	Init for I/O packet
;
	ld	a,(04b1h)		;get soft copy of INT
	and	0efh		;clear transmit int bit
	out	(0b1h),a		;and set port
	ld	a,(04e5h)		;and set trans bit in UMK
	or	10h
	out	(0e5h),a
;
;	Send out the data
;
	ld	b,084h		;count
	ld	hl,zcp_buf_blkstrt	;and address
zcp_send_xbs_loop
	ld	a,(hl)		;get byte
	out	(0e3h),a		;output serial port
zcp_send_xbs_waitio
	in	a,(0e5h)		;get status
	and	010h		;check bit 4
	jr	nz,zcp_send_xbs_waitio	;loop till next one
	djnz	zcp_send_xbs_loop	;process all characters
;
;	Reset port
;
	ld	a,(04b1h)		;get soft copy of INT
	out	(0b1h),a		;and output it
	ld	a,(04e5h)		;and soft copy of UMK
	out	(0e5h),a		;and also update it
;
	else
;
;	Use normal I/O channels
;
	ld	ix,(zcp_com_handle)	;get handle
	ld	de,0		;set to move from buffer
	ld	hl,zcp_buf_blkstrt	;and address
	ld	bc,0084h		;set count
	sys_call	os_mv		;output block
;
	endif
;
zcp_send_xbs_3
	call	zcp_getcomchar_xto60	;60 second delay wait
	ret	c
	cp	ack		;check for <ack>
	ret	z
	cp	nak		;check for <nak>
	jr	z,zcp_send_xbs_2	;resend block
	jr	zcp_send_xbs_3	;any other character
	ret
;
zcp_init_xmodem			;initialize xmodem transfer
	ld	a,00h		;set to 00
	ld	(zcp_block_number),a	;init block count
	ret			;return
;
zcp_init_xblock
	xor	a		;zero a
	ld	(zcp_block_check),a	;zero out block checksum
	ld	a,zcp_max_err	;set error counter
	ld	(zcp_block_errcnt),a
	ret
;
zcp_com_flush			;flush out the com port
	ld	bc,0060h		;timeout in ticks
	call	zcp_getcomchar_to	;and call timed get
	jr	nc,zcp_com_flush	;loop till timeout
	ret			;and then return
;
zcp_getcomchar			;get a com character
	ld	ix,(zcp_com_handle)	;get com port handle
zcp_getcomchar_1
	sys_call	os_gb		;get byte
	ret	nc		;return if no error
	cp	rc_susp		;check for suspend
	jp	nz,zcp_fatal_exit	;bad error
	call	zcp_reset_comport	;reset com port
	jr	zcp_getcomchar_1	;and loop
;
zcp_getcomchar_xto60			;get com char with 60 sec to
	ld	bc,01770h		;60 second timeout
	jr	zcp_getcomchar_to
zcp_getcomchar_xto10			;get com char with 10 sec to
	ld	bc,03e8h		;10 second timeout
	jr	zcp_getcomchar_to
zcp_getcomchar_xto1			;get com char with 1 sec to
	ld	bc,0064h		;1 second
zcp_getcomchar_to			;get a com character with timeout
	ld	ix,(zcp_com_handle)	;get com port handle
zcp_gccto_1
	sys_call	os_gbt		;get byte
	ret	nc		;return if no error
	cp	rc_tim
	jr	nz,zcp_gccto_2
	scf			;set carry flag
	ret			;and return
zcp_gccto_2
	cp	rc_susp		;check for suspend
	jp	nz,zcp_fatal_exit	;bad error
	call	zcp_reset_comport	;reset com port
	jr	zcp_gccto_1		;and loop
;
zcp_getline
	ld	b,0		;character counter
zcp_getline_next
	call	zcp_getcomchar	;get a character
	cp	8		;is it a backspace
	jr	z,zcp_getline_bksp
	cp	127		;or <del>
	jr	z,zcp_getline_bksp
	cp	13		;is it a cr
	jr	z,zcp_getline_cr
	cp	27		;it is escape
	jr	z,zcp_getline_esc
	ld	(hl),a		;save byte
	call	zcp_outchar
	inc	hl		;next loc
	inc	b		;count it
	jr	zcp_getline_next
zcp_getline_bksp
	dec	b		;count back
	jp	p,zcp_getline_bksp_ok	;if positive didn't try to back
	inc	b		;over buffer so put B back and
	jr	zcp_getline_next
zcp_getline_bksp_ok
	dec	hl		;adjust buffer
	ld	a,8		;and clean screen up
	call	zcp_outchar
	ld	a,32
	call	zcp_outchar
	ld	a,8
	call	zcp_outchar
	jr	zcp_getline_next
zcp_getline_cr
	xor	a		;put out a terminating
	ld	(hl),a		;null
	ld	a,b		;get b
	and	a		;set conditions flag
	ret	nz		;data was input
	scf			;consider no data an error
	ret			;return
zcp_getline_esc
	scf			;aborted input
	ret			;return
;
zcp_outstring			;output string
	ld	a,(hl)		;get character
	and	a		;test for zero
	ret	z		;and return if so
	call	zcp_outchar		;output the character
	inc	hl		;next byte
	jr	zcp_outstring	;and loop
;
zcp_outstring_pad			;output string (padded to B)
	ld	a,(hl)		;get character
	and	a		;test for zero
	jr	z,zcp_outstring_pad_1	;and jump to padding if so
	call	zcp_outchar		;output the character
	inc	hl		;next byte
	dec	b		;count
	jr	zcp_outstring_pad	;and loop
zcp_outstring_pad_1			;pad out string
	ld	a,32		;output space
	call	zcp_outchar
	djnz	zcp_outstring_pad_1	;and loop
;
zcp_outchar				;output character
	push	af		;save character
	outcom_t			;output byte
	pop	af		;get char
	cp	13		;check for CR
	jr	z,zcp_outchar_cr	;handle CR
	cp	9
	jr	z,zcp_outchar_tab	;handle tab
	ret			;return on normal character
zcp_outchar_cr			;cr processing
	ld	a,10		;otherwise put out a LF
	sys_call	os_pb
	ret			;and return
zcp_outchar_tab			;handle a tab
	push	bc		;save bc
	ld	b,08h		;expand to 8 spaces
zcp_outchar_tab_1
	ld	a,32		;output a space
	sys_call	os_pb
	djnz	zcp_outchar_tab_1	;and loop
	pop	bc		;get bc back
	ret			;then return
;
zcp_outchar_t			;output character (transpar)
	ld	ix,(zcp_com_handle)	;get handle
	sys_call	os_pb		;output byte
	ret			;and return
;
zcp_print_fileerr
	ld	hl,zcp_fileerr_msg	;set message
	jp	zcp_outstring	;and print
;
zcp_get_filename
	ld	hl,zcp_gfn_msg	;output prompt
	call	zcp_outstring
	ld	hl,zcp_buf1		;set buffer
	call	zcp_getline		;and get a line
	ret	c		;return on some sort of error
	ld	bc,0080h		;set local and length
	ld	hl,zcp_buf1
	ld	de,zcp_buf		;expanded name
	sys_call	gn_fex
	ret			;and return
;
zcp_output_menu			;output menu text in (HL)
	push	hl		;save pointer
	ld	hl,zcp_menu_text_pre	;output prefix
	sys_call	gn_sop
	pop	hl		;get text
	sys_call	gn_sop
	ld	hl,zcp_menu_text_suf	;and output suffix
	sys_call	gn_sop
	ret			;return
;
zcp_cleanup				;close all open stuff
	ld	hl,zcp_com_handle	;com handle
	call	close_file
	ld	hl,zcp_file_handle	;file
	call	close_file
	ld	hl,zcp_wild_handle	;wildcard
	call	close_wildcard
	ret
;
zcp_reset_comport			;close/open com port
	call	zcp_close_com	;close
	jp	zcp_open_com	;jump to open
;
zcp_open_com			;open com file
	ld	a,03h
	ld	b,0		;local text
	ld	hl,zcp_com_name
	ld	de,zcp_buf		;scratch
	ld	c,80h		;scratch length
	sys_call	gn_opf
	jp	c,zcp_fatal_exit	;if problem abort it all
	ld	(zcp_com_handle),ix	;save handle
	ret			;and return
;
zcp_close_com			;close out com file
	ld	hl,zcp_com_handle	;set handle address
	call	close_file		;and close
	ret
;
zcp_fatal_exit			;fatal error exit
	push	af		;save error number
	call	zcp_cleanup		;close files if open
	pop	af		;get file back
	sys_call	gn_err		;put out error message
	jp	exit_all		;return to basic
;
zcp_alldone				;normal exit routine
	call	zcp_cleanup		;close out all files
	ld	hl,(zcp_stack)	;get old stack
	ld	sp,hl		;and make current
	ret			;and return
;
zcp_print_currentdev			;print current device
	ld	bc,nq_dev		;get address of current
	sys_call	os_nq
	jr	zcp_print_extadd	;and print
;
zcp_print_currentdir			;print current directory
	ld	bc,nq_dir		;get dir address
	sys_call	os_nq
	sys_call	gn_rbe		;check for NULL and if not
	and	a		;set condition flags
	jr	nz,zcp_print_extadd	;continue as normal
	ld	a,'/'		;otherwise print out as /
	call	zcp_outchar		;and then
	ret			;exit back
;
zcp_print_extadd			;print extended address
	sys_call	gn_rbe		;read byte
	and	a		;set flags
	ret	z		;and return if done
	call	zcp_outchar		;output character
	inc	hl		;next character
	jr	zcp_print_extadd	;and loop
;
zcp_gnsop				;print local window text
	push	hl		;save hl
	ld	hl,zcp_gnsop_msg0	;prefix
	sys_call	gn_sop
	pop	hl		;get the text back
	sys_call	gn_sop
	ld	hl,zcp_gnsop_msg1	;and suffix
	sys_call	gn_sop
	ret
;
zcp_local_getline			;get a local filename in zcp_buf
	ld	hl,zcp_local_gfnmsg0	;put out prompt
	sys_call	gn_sop
	ld	a,32		;get string
	ld	b,80h		;buffer length
	ld	l,30h		;line length for input
	ld	de,zcp_buf		;buffer
	sys_call	gn_sip
	jr	c,zcp_local_getline	;loop on preempt
	ld	a,(zcp_buf)		;check for null
	and	a		;set condition flags
	ret	nz		;normal return
	scf			;set carry flag
	ret			;and return
;
zcp_local_fileerr			;put out local file error msg
	ld	hl,zcp_local_fileerr_msg
	sys_call	gn_sop
	call	pause		;wait for user input
	clear_screen		;clear the screen
	ret			;and return
;
zcp_tran_msg			;put out file transfer message
	ld	hl,zcp_tran_msg_msg
	sys_call	gn_sop
	ld	bc,0602h		;set cursor and output filename
	call	gotoxy
	ld	hl,zcp_file_name
	sys_call	gn_sop
	ret			;return
;
zcp_tran_block_init			;init for block transfers
	ld	hl,zcp_tran_block_msg	;screen info
	jr	zcp_tran_line_init_1
	ret			;and return
zcp_tran_line_init			;init for line transfers
	ld	hl,zcp_tran_line_msg	;screen info
zcp_tran_line_init_1
	sys_call	gn_sop
	ld	hl,0		;zero counter
	ld	(zcp_counter),hl
	ret			;and return
zcp_tran_num_out			;increment number and output
	ld	hl,(zcp_counter)	;get counter
	inc	hl		;and inc
	ld	(zcp_counter),hl	;save back
	ld	b,h		;then put in bc
	ld	c,l
	ld	hl,0002h		;set for local convert
	ld	de,zcp_scratch	;scratch space
	push	de		;save buffer
	ld	a,00h		;set format
	sys_call	gn_pdn		;convert number
	xor	a		;zero trailing byte
	ld	(de),a
	ld	bc,1604h		;set cursor
	call	gotoxy
	pop	hl		;get buffer
	sys_call	gn_sop		;and output
	ret			;then return
;
zcp_tran_errnum_out			;output error number
	ld	a,(zcp_block_errcnt)	;get error count
	ld	b,a		;and save in b
	ld	a,zcp_max_err	;get maximum error
	sub	b		;subtract to get count
	ld	c,a		;and put in bc
	ld	b,0
	ld	hl,0002h		;set for local convert
	ld	de,zcp_scratch	;scratch space
	push	de		;save buffer
	ld	a,00h		;set format
	sys_call	gn_pdn		;convert number
	xor	a		;zero trailing byte
	ld	(de),a
	ld	bc,2005h		;set cursor
	call	gotoxy
	pop	hl		;get buffer
	sys_call	gn_sop		;and output
	ret			;then return
;
zcp_no_xon
	ld	a,1		;byte count
	ld	de,zcp_xonxoff_status	;save address
	ld	bc,pa_xon		;and value
	sys_call	os_nq		;get it
	ld	hl,zcp_xonxoff_no	;not on
	jr	zcp_set_xon		;and set it
;
zcp_restore_xon
	ld	hl,zcp_xonxoff_status	;saved value
zcp_set_xon
	ld	a,1		;set count
	ld	bc,pa_xon		;set type of setting
	sys_call	os_sp		;and set
	xor	a		;and then do it
	ld	bc,pa_gfi
	sys_call	os_sp
	ret			;and return
;
; ============================================================================
; ============================================================================
;
;	Script language routines
;
; ============================================================================
;
;	This following set of code contains all of the script processing
;	routines and the associated static data.
;
; ============================================================================
;
;	Routines:
;
;	zcps_exec - execute a script, prompts the user for the
;		name of the script to execute.
;
;	zcps_exit - exit routine for the script processor, restores
;		the stack state and returns from a call to
;		zcp_exec_script.
;
;	zcps_gpline - gets a line of text from the script file, and
;		parses it for command.
;
;	zcps_jump - go to label (HL) in script
;
;	All of the commands are handled by routines which take data
;	from the data table and process it accordingly. The address for
;	these routines are stored in the table 'zcps_cmnd_addr'
;
; ============================================================================
;
;	Static structures used:
;
;	zcps_cmnd_addr - jump addresses for zcp commands
;	zcps_cmnd_index - index into zcps_cmnd_table
;	zcps_nprompt - Script file open prompt
;
; ============================================================================
;
;	Allowable commands:
;
;	cls - clear screen
;	exit - exit script
;	goto label - branch to a label
;	readkey - read a key and jump to label @A-@Z
;	label label - define a label
;	print "string" - print a string to the screen
;	reply "string" - send out string, trailing ';' for no CR
;	timeout duration - wait for specified time
;	wait "string" - wait for specified string
;
; ============================================================================
;
;	Dynamic data storage:
;
;	zcps_stack - stack pointer
;	zcps_fhandle - script file handle
;
; ============================================================================
; ============================================================================
;
	ifdef	zcp_script_asmflag
;
zcps_exec				;execute a script
	ld	hl,0		;save stack pointer
	add	hl,sp
	ld	(zcps_stack),hl
	ret
zcps_error				;error, restore stack and exit
	ld	hl,zcps_fhandle	;close file if open
	call	close_file
	ld	hl,(zcps_stack)	;restore stack
	ld	sp,hl
	ret			;and return
;
; ============================================================================
;
;	Individual command execution code
;
; ============================================================================
;
zcps_exec_cls
	jp	zcps_exec_done
;
zcps_exec_exit
	jp	zcps_exec_done
;
zcps_exec_goto
	jp	zcps_exec_done
;
zcps_exec_readkey
	jp	zcps_exec_done
;
zcps_exec_label
	jp	zcps_exec_done
;
zcps_exec_print
	jp	zcps_exec_done
;
zcps_exec_reply
	jp	zcps_exec_done
;
zcps_exec_timeout
	jp	zcps_exec_done
;
zcps_exec_wait
	jp	zcps_exec_done
;
zcps_exec_done
	ret		;return
;
; ============================================================================
;
zcps_cmnd_index			;command index
	dw	zcps_cmndt_cls
	dw	zcps_cmndt_exit
	dw	zcps_cmndt_goto
	dw	zcps_cmndt_readkey
	dw	zcps_cmndt_label
	dw	zcps_cmndt_print
	dw	zcps_cmndt_reply
	dw	zcps_cmndt_timeout
	dw	zcps_cmndt_wait
	dw	0
;
zcps_cmndt_cls	db	'cls',0
zcps_cmndt_exit	db	'exi',0
zcps_cmndt_goto	db	'got',0
zcps_cmndt_readkey	db	'rea',0
zcps_cmndt_label	db	'lab',0
zcps_cmndt_print	db	'pri',0
zcps_cmndt_reply	db	'rep',0
zcps_cmndt_timeout	db	'tim',0
zcps_cmndt_wait	db	'wai',0
;
zcps_cmnd_addr			;command address table
	dw	zcps_exec_cls
	dw	zcps_exec_exit
	dw	zcps_exec_goto
	dw	zcps_exec_readkey
	dw	zcps_exec_label
	dw	zcps_exec_print
	dw	zcps_exec_reply
	dw	zcps_exec_timeout
	dw	zcps_exec_wait
	dw	0
;
	endif
;
; ==============================================================
;
;	Static storage area
;
; ==============================================================
;
zcp_main_menu_msg			;main menu
	db	' ',1,'B','A',1,'B','SCII Send',13,10
	db	' ',1,'B','G',1,'B','oto Terminal',13,10
	db	' ',1,'B','O',1,'B','ptions',13,10
	db	' ',1,'B','R',1,'B','eceive binary',13,10
	db	' ',1,'B','S',1,'B','end binary',13,10
	db	' remo',1,'B','T',1,'B','e',13,10
	db	' e',1,'B','X',1,'B','it'
	db	0
zcp_main_screen_msg			;main screen message
	db	12
	vid_cursor_off
	vid_just_center
	db	10,'Local Mode',10
	vid_just_normal
	db	0
;
zcp_sndtxt_msg0
	db	'Send ASCII or text file',0
zcp_sndfile_msg0
	db	'XModem file send',0
zcp_rcvfile_msg0
	db	'XModem file receive',0
zcp_local_gfnmsg0
	vid_gotoxy	8,4
	vid_cursor_on
	db	'File name : ',0
;
zcp_gnsop_msg0			;standard window prefix
	db	12
	vid_just_center
	db	10,0
zcp_gnsop_msg1			;standard window suffix
	db	10
	vid_just_normal
	db	0
zcp_local_fileerr_msg			;local filename error message
	db	12,10,10
	vid_just_center
	db	'Error accessing file',10
	vid_just_normal
	db	0
;
zcp_tran_msg_msg			;transfering files screen
	db	12
	vid_cursor_off
	vid_just_center
	db	'File Transfer in Progress',10
	vid_just_normal
	db	13,10
	db	'File:'
	vid_gotoxy	30,4
	db	'Last message'
	db	0
zcp_tran_block_msg
	vid_gotoxy	10,4
	db	'Block'
	vid_gotoxy	10,5
	db	'Error number'
	db	0
zcp_tran_line_msg
	vid_gotoxy	10,4
	db	'Line',0
;
zcp_options_menu_msg			;options menu
	db	10,' ',1,'B','M',1,'B','acbinary',13,10
	db	' ',1,'B','P',1,'B','rompt char ( )',13,10
	db	10,' e',1,'B','X',1,'B','it',10
	db	0
zcp_options_prompt_msg			;prompt character get
	db	12
	vid_gotoxy	20,3
	db	'Prompt character to use : '
	vid_cursor_on
	db	0
zcp_options_prompt_msg1			;terminal prompt message
	vid_cursor_off
	db	12,0
zcp_options_macbin_msg0
	vid_win_sel	3		;select window 3
	vid_gotoxy	0,1		;position cursor
	db	0
zcp_options_macbin_msg1
	db	' ',0		;blank space
zcp_options_macbin_msg2
	db	1,245,0		;bullet mark
zcp_options_macbin_msg3
	vid_win_sel	1		;set back to main window
	db	0
;
;	Remote mode static data
;
	ifdef	zcp_remote_asmflag
;
zcp_remote_winmsg			;message for z88 window
	db	12
	vid_just_center
	db	10,'Remote Mode',10
	vid_just_normal
	db	0
zcp_remote_prompt			;remote prompt
	db	13,10
	db	'ZCP Remote - C/E/F/I/R/S/T/X/? : '
	db	0
zcp_remote_init_msg			;startup message
	zcp_video_cls
	db	'ZCP Remote - Copyright 1990 Richard Haw',13
	db	0
zcp_remote_exit_msg			;remote exit message
	zcp_video_cls
	db	'Closing down Z88 Communications Program Remote System'
	db	13,13
	db	0
;
zcp_rmf_msg0			;remote files init message
	zcp_video_cls
	db	'Listing of files in ',0
zcp_rmf_msg1
	db	13,13
	db	'File name          '
	db	'Creation Date         '
	db	'Modification Date     '
	db	'File Size',13,0
zcp_rmf_msg_dir
	db	'< Directory >',13,0
;
zcp_rmi_msg_0			;remote information screen
	zcp_video_cls
	db	'Z88 Information',13
	db	13,9,'Current path      ',0
zcp_rmi_msg_2
	db	13,9,'Available memory  ',0
zcp_rmi_msg_99
	db	13,13,0
;
zcp_rmc_dirmsg0
	db	13,13,'Current path is ',0
zcp_rmc_dirmsg1
	db	13,'Enter new path  ',0
zcp_rmc_emsg_0
	db	13,13,'Error - Bad syntax or invalid path',13,0
zcp_rme_msg0
	db	13,13,'Erase Z88 file',0
zcp_rme_msg1
	db	13,13,'Erasing file ',0
;
zcp_rmt_msg0
	db	13,13,'Type out a Z88 file',0
zcp_rmt_msg1
	db	13,13,0
;
zcp_rmtrcv_msg0
	db	13,13,'Receive a file from the Z88',0
zcp_rmtrcv_msg1
	db	13,13,'Set up to receive file, press <RETURN> when '
	db	'transfer is complete',13,0
zcp_rmtsnd_msg0
	db	13,13,'Send a file to the Z88',0
zcp_rmtsnd_msg1
	db	13,13,'Start file send now, press <RETURN> when '
	db	'transfer is complete',13,0
zcp_rmtrcv_errmsg
	db	13,'Error in file transfer',0
;
zcp_remote_help_msg			;remote help screen
	zcp_video_cls
	db	'ZCP Remote Commands',13,13
	db	9,'(C)hange working directory',13
	db	9,'(E)rase a Z88 file',13
	db	9,'(F)ile listing',13
	db	9,'(I)nformation',13
	db	9,'(R)eceive binary file from the Z88',13
	db	9,'(S)end binary file to the Z88',13
	db	9,'(T)ype a text file',13
	db	9,'e(X)it remote and shut down Z88',13
	db	9,'(?) This help screen',13
	db	13,0
;
	endif
;
zcp_gfn_msg
	db	13,13,'Enter file name : ',0
;
zcp_fileerr_msg
	db	13,'Error accessing the specified file',13,0
;
zcp_mainwin
	vid_win_titled	3,74,0,16,8,'MENU'
	vid_win_titled	1,0,0,72,8,'ZCP - Z88 COMMUNICATIONS PROGRAM V1.03'
	db	0
zcp_main_win_init			;reset main window
	vid_just_normal		;normal justify
	vid_cursor_off		;cursor off
	db	12		;and clear screen
	db	0
;
zcp_menu_text_pre			;prefix for menu output
	vid_win_sel	3		;select window 3
	db	12		;and clear it
	db	0
zcp_menu_text_suf			;suffix for menu output
	vid_win_sel	1
	db	0
;
zcp_pname	db	"ZCP v1.03"
	db	0
;
zcp_shareware_msg			;shareware information screen
	db	12
	vid_just_center
	vid_cursor_off
	db	10,'ZCP - Z88 Communications Program',10,10,10
	db	'Copyright 1989,1990 Richard Haw, All Rights Reserved'
	vid_just_normal
	db	0
;
zcp_com_name			;com port file name
	db	':COM.0',0
zcp_wildcard_spec			;file listing spec
	db	'*',0
zcp_terminal_cli			;CLI command to get to Terminal
	db	'#V',0
;
; ==============================================================
;
;	Xmodem transfer static storage
;
; ==============================================================
;
;
;	Error messages
;
zcp_xonxoff_no			;off status for setup panel
	db	'N'
zcp_emsg_errcnt
	db	'Error count exceeded',0
zcp_emsg_timeout
	db	'Timeout',0
zcp_emsg_badheader
	db	'Bad block header',0
zcp_emsg_badblknum
	db	'Bad block number',0
zcp_emsg_badchk
	db	'Checksum error',0
zcp_emsg_blank
	db	'                ',0
;
;	Data block for MacBinary transfer protocol
;
zcp_macbinary_default_block
	bsz	65		;blank fill
	db	'TEXT'		;type
	db	'Z88 '		;creator
	bsz	55		;blank fill
