#include "zdc.h"

/*
	File - proc.c

	Routines for doing the actual pass by pass processing of the
	data.

	int proc_file(void)
		Process the file, called from the main routine.

	int write_var_table(void)
		Writes out the variable table
*/

int proc_file(void)
{
	int isel;
	char *c1;

	rewind(source_file);		/* rewind the source file */
	while (read_line() == 0)		/* read entire file */
	{
		card_ptr = card;		/* set to start of buffer */
		if (com_flag == 1)
			fprintf(asm_file,"; %s\n",card);	/* output source */
		isel = parse_command(card);
		switch (isel)
		{
		case ZPRINT :
			proc_print();
			break;
		case ZAT :
			skip_param(); 		/* get coordinates */
			goto_pos(card_ptr);	/* set cursor position */
			skip_param();		/* get subcommand */
			switch(parse_command(card_ptr))
			{
			case ZSAY :
				proc_atsay();
				break;
			case ZGET :
				proc_atget();
				break;
			default :
				syntax_error();
				break;
			}
            break;
		case ZDELETE :
		case ZDO :
		case ZCLS :
			fprintf(asm_file,"\tcall\tcls\n");
			break;
		case ZELSE :
			proc_else();
            break;
		case ZENDDO :
			proc_enddo();
			break;
		case ZENDIF :
			proc_endif();
			break;
		case ZFIND :
		case ZGO :
			proc_go();
			break;
		case ZIF :
			proc_if();
			break;
		case ZLET :
		case ZSELECT :
		case ZUSE :
		case ZRESTORE :
		case ZRETURN :
			proc_return();
			break;
		case ZDECLARE :
			proc_declare();
			break;
		default :
			printf("Unknown command:\n%s\n",card);
			gerr++;
			break;
		}
	}
	if (com_flag != 0)
	{
		fprintf(asm_file,";\n");
		fprintf(asm_file,";\t========================================================\n");
		fprintf(asm_file,";\n");
        fprintf(asm_file,";\tDynamic variable area\n");
		fprintf(asm_file,";\n");
		fprintf(asm_file,";\t========================================================\n");
		fprintf(asm_file,";\n");
	}

	write_var_table(var);		/* write out variable table */

	return(0);
}

int proc_go(void)
{
	skip_param();
	fprintf(asm_file,"\tld\thl,%s\n",card_ptr);
	fprintf(asm_file,"\tcall\tzdc_go\n");
	return(0);
}

int proc_return(void)
{
	fprintf(asm_file,"\tret\n");
	return(0);
}

int proc_declare(void)
{
	char *c, *c1;
	int i, len;
	struct varstruct **v, *v1;

	skip_param();		/* skip to next word */
	c = card_ptr;		/* and save location */
	skip_param();		/* skip to type */
	switch (parse_command(card_ptr))
	{
	case ZSTRING :
		skip_param();	/* get length */
        len = atoi(card_ptr);
		break;
	case ZNUMERIC :
    	len = 5;
		break;
	default :
		printf("Declaration syntax error:\n%s\n",card);
		gerr++;
		return(-1);
	}

	v = &var;
	while (*v != NULL)	/* find end of chain */
		v = &((*v)->next);

	*v = malloc(sizeof(struct varstruct));
	if (*v == NULL)
	{
		printf("Out of memory\n");
		exit(0);
	}
	(*v)->next = NULL;

	i = 0;
	c1 = (*v)->name;
	while ((isspace(*c) == 0) && (i < 10))
	{
		*c1 = tolower(*c);
		c++;
		c1++;
		i++;
	}
	*c1 = 0;
	(*v)->length = len;

	return(0);
}


int proc_if(void)
{
	char lab[9];

	labindex++;		/* next label */
	iflabel[iflevel] = labindex;	/* save label */
	iflevel++;		/* next if level */
	gen_label(lab,labindex);
	fprintf(asm_file,"\tjp\tnz,%s\n",lab);
	return(0);
}

int proc_endif(void)
{
	char lab[9];

	iflevel--;
	if (iflevel < 0)
	{
		printf("Nesting level error:\n%s\n",card);
		gerr++;
		return(0);
	}
	gen_label(lab,iflabel[iflevel]);
	fprintf(asm_file,"%s\n",lab);
	return(0);
}

int proc_else(void)
{
	char lab[9];

	labindex++;		/* new lable */
	gen_label(lab,labindex);
	fprintf(asm_file,"\tjmp\t%s\n",lab);	/* jump to endif */
	gen_label(lab,iflabel[iflevel-1]);	/* from if statement */
	fprintf(asm_file,"%s\n",lab);
	iflabel[iflevel-1] = labindex;		/* save new index */
	return(0);
}

int proc_enddo(void)
{
	char lab[9];

	whilelevel--;
	if (whilelevel < 0)
	{
		printf("Nesting level error:\n%s\n",card);
		gerr++;
		return(0);
	}
	gen_label(lab,whilelabel[whilelevel]);
	fprintf(asm_file,"%s\n",lab);
	return(0);
}

int proc_atsay(void)
{
	skip_param();
	load_hl(card_ptr);
	fprintf(asm_file,"\tsyscall\tgn_sop\n");
	return(0);
}

int proc_atget(void)
{
	skip_param();
	return(0);
}

int proc_print(void)
{
	return(0);
}

int goto_pos(char *c)		/* go to cursor location *c */
{
	int i1, i2;

	i1 = atoi(c);
	while (isdigit(*c) != 0)
		c++;
	c++;
	i2 = atoi(c);
	i1 = i2 * 256 + i1;
	fprintf(asm_file,"\tld\tbc,0%xh\n\tcall\tgotoxy\n",i1);
	return(0);
}

int load_hl(char *c)		/* load HL with string expression *c */
{
	char lab[9];

	c++;
	*strchr(c,'"') = 0;
	labindex++;
	gen_label(lab,labindex);
	fprintf(asm_file,"\tld\thl,%s\n",lab);
	labindex++;
	gen_label(lab,labindex);
	fprintf(asm_file,"\tjr\t%s\n",lab);
	labindex--;
	output_string(labindex,c);
	labindex++;
	gen_label(lab,labindex);
	fprintf(asm_file,"%s\n",lab);

	return(0);
}

void write_var_table(struct varstruct *v)
{
	if (v == NULL) return;		/* nothing here */

	if (v->length == 0)	/* numeric */
		fprintf(asm_file,"zdc_%s\tbsz\t%d\n",v->name,5);
	else
		fprintf(asm_file,"zdc_%s\tbsz\t%d\n",v->name,v->length);
	write_var_table(v->next);
	free(v);
	return;
}