#include "zdc.h"

/*
	File - Utils.c

	int gen_label(char *label,int index)
		Generates a unique label from the passed information

	int get_files(void)
		Asks for the file name for the source files and opens them

	int read_line(void)
		Reads a line from the input file, and puts that line in
		card. Comments are ignored and leading spaces are removed.
		Returns 0 for OK, 1 for EOF.

	int output_string(int index, char *string)
		Outputs defines for the passed string.

	int parse_command(char *cmnd)
		returns command number for *cmnd.

	int check_keyword(char *cmnd, char *card)
		returns 1 if *card starts with *cmnd

	int skip_param(void)
		Move card_ptr to the next parameter in the field. If no more
		data card_ptr is set to NULL.

	int syntax_error(void)
		Puts out a syntax error message along with the offending line.

*/

int gen_label(char *label, int index)
{
	sprintf(label,"zdc_%0X",index);
	return(0);
}


int get_files(void)
{
	printf("\nFile to process (.prg/.asm) : ");
	gets(card);
	if (strchr(card,'.') != NULL)		/* if an existing extension strip */
	{
		*strchr(card,'.') = 0;
	}
	strcat(card,".prg");
	source_file = fopen(card,"rt");
	if (source_file == NULL)
	{
		printf("\nUnable to open source file %s\n",card);
		exit(0);
	}
	*strchr(card,'.') = 0;
	strcat(card,".asm");
	asm_file = fopen(card,"wt");
	if (asm_file == NULL)
	{
		printf("\nUnable to open asm file %s\n",card);
		exit(0);
	}
	return(0);
}

int read_line(void)
{
	char *c;

	while (1)
	{
		if (feof(source_file))
			return(1);

		fgets(card,256,source_file);
		if (*card != 0)			/* ignore null lines */
		{
			c = strchr(card,'\n');		/* strip trailing \n */
			if (c == NULL)
			{
				if (strlen(card) > 245)
				{
					printf("Line too long:\n%s\n",card);
					gerr++;
				}
			}
			else
				*c = 0;
		}
		if (*card != 0)
		{
			c = card;
			if (*c == '*')
			{
				c++;
				if ((*c != ' ') && (*c != 0))
					return(0);
			}
			else
			{
				while (*c == ' ') c++;
				strcpy(card,c);
				return(0);
			}
		}
	}
}

int output_string(int index,char *string)
{
	char *c;
	char label[9];
	int ifirst, ic;

	c = string;
	ic = 24;
	ifirst = 0;
	
	gen_label(label,index);
	fprintf(asm_file,"%s\n",label);
	fprintf(asm_file,"\tdb\t");
	
	while (*c != 0)
	{
		if (ic > 80)			/* start a new line */
		{
			fprintf(asm_file,"\n\tdb\t");
			ifirst = 0;
			ic = 24;
		}
		if (ifirst != 0)		/* add comma if needed */
		{
			fputc(',',asm_file);
			ic++;
		}
		sprintf(label,"%d",*c);
		ic = ic + strlen(label);
		fprintf(asm_file,"%s",label);
		c++;
		ifirst = 1;
	}
	fprintf(asm_file,"\n\tdb\t0\n");
	return(0);
}


int parse_command(char *cmnd)
{
	int i;

	for (i=0;i<NUM_COMMANDS;i++)
		if (check_keyword(command_names[i],cmnd) != 0)
			return(i);
	return(-1);
}

int check_keyword(char *cmnd, char *c)
{
	while (*cmnd != 0)
	{
		if (*cmnd != tolower(*c))
			return(0);
		cmnd++;
		c++;
	}
	return(1);
}

int skip_param(void)
{
	while ((*card_ptr != 0) && (isspace(*card_ptr) == 0))
		card_ptr++;
	if (*card_ptr == 0)
	{
		card_ptr = NULL;
		return(0);
	}
	while ((*card_ptr != 0) && (isspace(*card_ptr) != 0))
		card_ptr++;
	if (*card_ptr == 0)
	{
		card_ptr = NULL;
		return(0);
	}
	return(1);
}

int syntax_error(void)
{
	printf("Syntax error:\n%s\n",card);
	gerr++;
	return(0);
}