;
; ==========================================================================
;
;	Z88 File Encryption Program (ZENC)
;
; ==========================================================================
;
zenc_main				;main entry point
	ld	hl,0		;reset all data
	ld	(zenc_fhand),hl
	add	hl,sp		;get stack location
	ld	(zenc_stack),hl	;and save it
;
	ld	hl,zenc_pname	;set program name
	sys_call	dc_nam
;
;	Put out initial screen message
;
	ld	hl,zenc_mainwin	;put out main windows
	sys_call	gn_sop
	ld	hl,zenc_pw_msg	;password prompt
	sys_call	gn_sop
zenc_getpw			;read data into zenc_buf
	ld	bc,0b03h		;set cursor
	call	gotoxy
	ld	de,zenc_buf		;set address
	ld	a,00h		;set mode
	ld	b,32		;buffer length
	ld	c,0
	ld	l,74		;locked line length
	sys_call	gn_sip		;call routine
	jr	nc,zenc_getpw1	;no error occured
	ld	bc,0b03h		;reset cursor
	call	gotoxy
	jr	zenc_getpw
zenc_getpw1
;
;	Now make up password
;
	ld	de,zenc_buf		;set buffer address
	ld	hl,0		;and seed
zenc_makepw
	ld	a,(de)		;get byte
	cp	a		;and check for zero
	jr	z,zenc_makepw_done
	ld	b,a		;save byte in b
	inc	de		;next byte
	jr	zenc_makepw
zenc_makepw_done
	ld	(zenc_seed),hl	;save seed value
;
;	Main file processing loop
;
zenc_main_loop
	clear_screen		;erase screen
	ld	hl,zenc_file_msg	;prompt
	ld	de,zenc_buf		;buffer
	ld	c,040h		;buffer length
	call	get_file_name
	ld	hl,zenc_buf		;set buffer address
	ld	a,(hl)		;check for no data
	and	a
	jp	z,zenc_done		;and exit if none
;
;	Process file
;
	ld	hl,zenc_mainwin	;redraw window
	sys_call	gn_sop
	ld	hl,zenc_proc_msg	;working message
	sys_call	gn_sop
	ld	hl,zenc_buf		;and file name
	sys_call	gn_sop
	call	pause
;
	jp	zenc_main_loop
;
; ==========================================================================
;
;	Utility and support routines
;
; ==========================================================================
;
zenc_cleanup			;cleanup open files/wildcards
	ld	hl,zenc_fhand	;close file if open
	call	close_file
	ret
;
zenc_done				;exit point
	ld	hl,(zenc_stack)	;get old stack
	ld	sp,hl		;and make current
	call	zenc_cleanup	;cleanup all files
	ret			;and return
;
; ==========================================================================
;
;	Static storage area
;
; ==========================================================================
;
zenc_mainwin
	vid_win_titled	1,0,0,84,8,'Z88 FILE ENCRYPTION PROGRAM V1.00'
	db	0
;
zenc_pw_msg				;password prompt
	db	12,10
	vid_just_center
	db	'Z88 File Encryption Program '
	db	'- Copyright 1990 Richard Haw'
	db	10
	vid_just_normal
	db	13,10
	db	'  Password:',0
;
zenc_pname	db	"ZENC v1.00"
	db	0
;
zenc_file_msg			;file prompt
	db	'File to Encode/Decode',0
;
zenc_proc_msg			;processing file message
	db	12,10,10
	db	'  Processing file',13,10
	db	'    : ',0
;
;
; ==========================================================================
;
;	Dynamic storage area
;
; ==========================================================================
;
	IFDEF	BASIC
zenc_dstart	equ	$
	bsz	060h
	ENDIF
;
	IFDEF	ROM
zenc_dstart	equ	02100h
	ENDIF
;
zenc_stack	equ	zenc_dstart		;stack save pointer
zenc_fhand	equ	zenc_stack+2	;file handle
zenc_fptr	equ	zenc_fhand+2	;file pointer
zenc_seed	equ	zenc_fptr+4		;random seed
zenc_ran1	equ	zenc_seed+2		;random count 1
zenc_ran2	equ	zenc_ran1+2		;random count 2
zenc_buf	equ	zenc_ran2+2		;buffer
zenc_eod	equ	zenc_buf+40h	;end of data
;
