;
; ==============================================================
;
;	Z88 file directory program
;
; ==============================================================
;
;	Start of program
;
; ==============================================================
;
zfd_main				;main entry point
	ld	hl,0		;reset all data
	add	hl,sp		;get stack location
	ld	(zfd_stack),hl	;and save it
	ld	hl,zfd_pname	;set program name
	sys_call	dc_nam
	ld	hl,0		;zero out all handles
	ld	(zfd_in_handle),hl
	ld	(zfd_out_handle),hl
	ld	(zfd_wild_handle),hl
	ld	(zfd_file_count),hl	;and file counters
	ld	(zfd_pdfile_count),hl
;
	ld	hl,zfd_mainwin	;put out main window
	sys_call	gn_sop
;
;	Open wildcard handle and proceed
;
	ld	b,0		;local string
	ld	hl,zfd_wildstring	;wildcard name
	ld	a,1		;set mode (return full path)
	sys_call	gn_opw		;open wildcard handle
	jp	c,zfd_exit		;exit on an error
	ld	(zfd_wild_handle),ix	;save handle
;
;	Open output file
;
	ld	b,0		;local file name
	ld	hl,zfd_rfilename	;get file name
	ld	de,zfd_scratch	;expanded file name
	ld	c,080h		;buffer size
	ld	a,02h		;open for output
	sys_call	gn_opf		;open file
	jp	c,zfd_exit		;just exit on error
	ld	(zfd_out_handle),ix	;save output file handle
;
zfd_mainloop
	ld	ix,(zfd_wild_handle)	;get wildcard handle
	ld	c,80h		;buffer size
	ld	de,zfd_file_name	;and buffer
	sys_call	gn_wfn		;get next match
	jp	c,zfd_done		;all done if error
	cp	011h		;check for file type (DOR 0x11)
	jr	nz,zfd_mainloop	;loop on all others
;
	ld	hl,zfd_msg3		;put out file name
	sys_call	gn_sop
	ld	hl,zfd_file_name
	sys_call	gn_sop
	sys_call	gn_nln
;
	ld	hl,zfd_msg1		;count prefix
	sys_call	gn_sop
	ld	hl,(zfd_file_count)	;count the file
	inc	hl
	ld	(zfd_file_count),hl
	call	zfd_printnum
;
;	Now open file and check to see if it is a pipedream file
;
	ld	b,0		;local file name
	ld	hl,zfd_file_name	;get file name
	ld	de,zfd_scratch	;expanded file name
	ld	c,080h		;buffer size
	ld	a,01h		;open for input
	sys_call	gn_opf		;open file
	jp	c,zfd_mainloop	;if an error just ignore it
	ld	(zfd_in_handle),ix	;save handle
;
;	Read in a block of stuff (4 bytes), if this is a pipedream file
;	it will be something like %CO:, or %OP%. If %CO: then no title,
;	otherwise search for the title.
;
zfd_pdcheck
	ld	bc,06h		;get six bytes
	ld	ix,(zfd_in_handle)	;get handle
	ld	hl,0		;move to memory
	ld	de,zfd_scratch	;position to move to
	sys_call	os_mv		;get the data
	jp	c,zfd_eof		;not enough data, close it
	ld	hl,zfd_scratch	;check for column entry
	ld	de,zfd_cmp_column
	ld	b,4
	call	compare_string
	jr	z,zfd_pdnotitle	;pd but no title
	ld	hl,zfd_scratch	;check for options
	ld	de,zfd_cmp_title
	ld	b,4
	call	compare_string
	jr	nz,zfd_eof		;not title or options, no pd file
	ld	hl,zfd_scratch	;check for title
	ld	de,zfd_cmp_title
	ld	b,6
	call	compare_string
	jr	z,zfd_pdtitle	;print with title
	ld	ix,(zfd_in_handle)	;get infile handle
zfd_pdskip
	sys_call	os_gb		;get a byte
	jr	c,zfd_eof		;if eof skip file
	cp	13		;check for cr
	jr	nz,zfd_pdskip	;and skip if not
	jr	zfd_pdcheck
zfd_pdnotitle
	ld	a,09h		;and then a tab
	call	zfd_putbyte
	call	zfd_copy_fname	;output file name
	ld	a,13		;put out end of line
	call	zfd_putbyte
	jr	zfd_end_pdfile
zfd_pdtitle
	call	zfd_getbyte		;get a byte from the file
	jr	c,zfd_pdnotitle	;error reading, take it from here
	cp	13
	jr	z,zfd_pdnotitle	;end of title
	call	zfd_putbyte		;put byte
	jr	zfd_pdtitle
;
zfd_eof
	ld	hl,zfd_in_handle	;close the file back out
	call	close_file
	jp	zfd_mainloop
;
zfd_end_pdfile			;end of this pd file
	ld	hl,zfd_msg2		;count prefix
	sys_call	gn_sop
	ld	hl,(zfd_pdfile_count)	;count the file
	inc	hl
	ld	(zfd_pdfile_count),hl
	call	zfd_printnum
	jr	zfd_eof		;and finish with this file
;
zfd_done
	pause			;pause
	jp	zfd_exit		;and exit with full close
;
; ==============================================================
;
;	Support routines
;
; ==============================================================
;
;	zfd_cleanup - close all open files and wildcard handles
;	zfd_copy_fname - copy file name to output file
;	zfd_exit - exit program with cleanup
;	zfd_getbyte - read byte from input file
;	zfd_printnum - print number in hl
;	zfd_putbyte - put byte to report file
;
; ==============================================================
;
zfd_cleanup
	ld	hl,zfd_in_handle	;input file
	call	close_file
	ld	hl,zfd_out_handle	;output file
	call	close_file
	ld	hl,zfd_wild_handle	;wildcard handle
	call	close_wildcard
	ret			;and return
;
zfd_copy_fname
	ld	ix,(zfd_out_handle)	;get output handle
	ld	hl,zfd_file_name	;and filename address
zfd_cfn_1
	ld	a,(hl)		;get byte
	and	a		;set flags
	ret	z		;return if done
	sys_call	os_pb		;put it to the file
	jp	c,zfd_exit		;exit program on error
	inc	hl		;next byte
	jr	zfd_cfn_1		;and loop
;
zfd_exit
	call	zfd_cleanup		;close all open
	ld	hl,(zfd_stack)	;restore stack
	ld	sp,hl
	ret			;and return
;
zfd_getbyte				;get a byte from the input file
	ld	ix,(zfd_in_handle)	;get handle
	sys_call	os_gb		;byte,
	ret			;and return
;
zfd_printnum
	push	hl		;put count in BC
	pop	bc
	ld	hl,2		;set for local print
	ld	de,zfd_scratch	;print in scratch
	push	de		;save address
	ld	a,3		;no leading or trailing spaces
	sys_call	gn_pdn		;print decimal
	xor	a		;put out trailing zero
	ld	(de),a
	pop	hl		;get address
	sys_call	gn_sop		;and print it
	ret			;then return
;
zfd_putbyte				;put byte to report file
	ld	ix,(zfd_out_handle)	;get handle
	sys_call	os_pb		;output byte
	jr	c,zfd_exit		;exit program on error
	ret
;
; ==============================================================
;
;	Static storage area
;
; ==============================================================
;
zfd_pname	db	'ZFD v1.01',0
;
zfd_mainwin
	vid_win_titled	1,0,0,90,2,'ZFD V1.01     Z88 File Directory       COPYRIGHT 1989,1990 RICHARD HAW'
	db	'  Files:  Total         Pipedream'
	vid_win_titled	2,0,2,90,6,'SCANNED FILES'
	db	0
;
zfd_msg1			;file count message
	vid_win_sel	1
	vid_gotoxy	16,0
	db	0
zfd_msg2			;pd file count message
	vid_gotoxy	35,0
	db	0
zfd_msg3			;select win 2
	vid_win_sel	2
	db	0
;
zfd_wildstring		;wildcard search string
	db	':*//*',0
zfd_rfilename		;report file name
	db	'ZFD.rep',0
;
zfd_cmp_column		;column compare
	db	'%CO:'
zfd_cmp_title		;title compare
	db	'%OP%DE'