;
; ==========================================================================
;
;	ZFU Program
;
; ==========================================================================
;
zfu_main				;main entry point
	ld	hl,0		;reset all data
	add	hl,sp		;get stack location
	ld	(zfu_stack),hl	;and save it
;
	ld	hl,zfu_pname	;set program name
	sys_call	dc_nam
;
	ld	hl,0		;initialize handles
	ld	(zfu_arcfile_handle),hl
	ld	(zfu_wildcard_handle),hl
	ld	(zfu_filehandle),hl
;
	ld	a,0ffh
	ld	(cmpflg),a		;set default to compress
	ld	(zfu_newfile_pause),a	;and to pause
	ld	hl,000ffh		;set initial size limit
	ld	(zfu_arcfile_limit+2),hl
	ld	hl,zfu_mainwin	;output main window
	sys_call	gn_sop
	call	zfu_get_archive_file	;get archive file name
	ld	hl,zfu_init_data	;initialize date information
	ld	de,zfu_credate_buf
	ld	bc,000ch
	ldir
;
;	Put out initial shareware message
;
	ld	hl,zfu_share_text
	sys_call	gn_sop
	jr	zfu_main_mnu
;
zfu_main_menu_0
	ld	hl,zfu_main_win_init	;reset main window
	sys_call	gn_sop
;
; ==========================================================================
;
;	Main menu
;
; ==========================================================================
;
zfu_main_mnu
	ld	hl,zfu_main_mnum
	call	zfu_output_menu
;
zfu_mainopt
	sys_call	os_in
	jr	c,zfu_mainopt
	call	a_to_lower
	cp	'b'		;backup menu
	jp	z,zfu_backupmenu
	cp	'f'		;file menu
	jp	z,zfu_file_menu
	cp	'i'		;information
	jp	z,info
	cp	'o'		;options
	jp	z,zfu_opt_mnu
	cp	'x'		;exit
	jp	z,zfu_done
	jp	zfu_mainopt
;
; ==========================================================================
;
;	File menu
;
; ==========================================================================
;
zfu_file_menu
	ld	hl,zfu_file_mnutext	;output menu text
	call	zfu_output_menu
zfu_file_menu_1
	sys_call	os_in		;get keystroke
	jr	c,zfu_file_menu_1	;loop on carry
	call	a_to_lower
	cp	'a'		;addfile
	jp	z,zfu_addfile
	cp	'e'		;extract file
	jp	z,zfu_extract_file
	cp	'l'		;list archive
	jp	z,zfu_listfile
	cp	'x'		;exit menu
	jp	z,zfu_main_mnu
	jr	zfu_file_menu_1
;
; ==========================================================================
;
;	Backups menu and processing
;
; ==========================================================================
;
zfu_backupmenu
	call	zfu_backup_getdates	;get backup dates
	ld	hl,zfu_backup_msg1	;put out data screen
	sys_call	gn_sop
	ld	bc,2803h		;set cursor
	call	gotoxy
	ld	de,zfu_buffer	;set scratch buffer
	push	de		;and save
	ld	hl,zfu_backup_full_date	;and output full backup date
	call	print_date_time
	ld	bc,2804h		;set cursor
	call	gotoxy
	ld	hl,zfu_backup_part_date	;and output part backup date
	pop	de		;get scratch buffer address
	call	print_date_time
	ld	hl,zfu_backup_mnutxt	;output menu text
	call	zfu_output_menu
;
zfu_backup_menu0
	sys_call	os_in
	call	a_to_lower
	cp	'x'
	jp	z,zfu_main_menu_0
	cp	'f'		;full backup
	jp	z,zfu_backup_full
	cp	'd'		;differential backup
	jp	z,zfu_backup_diff
	cp	'i'		;incremental
	jp	z,zfu_backup_inc
	jr	zfu_backup_menu0
;
zfu_backup_full			;full backup
	ld	hl,zfu_init_data	;reset dates to origin
	ld	de,zfu_credate_buf	;active buffer
	ld	bc,000ch		;change both dates
	ldir
	call	zfu_backup_all_files	;backup all matching files
	ld	de,zfu_backup_full_date	;update dates
	call	set_date_time
;
zfu_backup_full_99
	ld	de,zfu_backup_part_date	;also update partial date
	call	set_date_time
	call	zfu_backup_update_file	;update file
	jp	zfu_main_menu_0	;go back to main menu
;
zfu_backup_diff			;differential backup
	ld	hl,zfu_backup_full_date	;set date
	jr	zfu_backup_inc_1	;jump to entry
;
zfu_backup_inc			;incremental backup
	ld	hl,zfu_backup_part_date	;reset date to last backup
zfu_backup_inc_1
	push	hl		;save source
	ld	de,zfu_credate_buf	;active buffer
	ld	bc,0006h		;move date
	push	bc		;save copy
	ldir
	pop	bc		;get count back
	pop	hl		;and source
	ldir			;and move second block
	call	zfu_backup_all_files	;backup all matching files
	jp	zfu_backup_full_99	;update dates and return
;
zfu_backup_all_files			;backup all files with match
	ld	hl,zfu_backup_wildstring	;set up file wildcard
	ld	de,zfu_wildcard_name
	call	copy_string		;and copy
	call	zfu_addfile_internal	;and add all matching files
	ret
;
; ==========================================================================
;
;	Backup support routines
;
; ==========================================================================
;
zfu_backup_getdates			;get dates from backup file
	ld	hl,zfu_backup_file_name	;open file
	ld	b,0
	ld	a,01h		;open for input
	ld	de,zfu_buffer	;scratch space
	ld	c,40h		;scratch length
	sys_call	gn_opf		;open file
	jr	nc,zfu_backup_getdates1	;no problem opening file
	ld	hl,zfu_init_data	;move dummy data to dates
	ld	de,zfu_backup_full_date
	ld	bc,000ch
	ldir
	call	zfu_backup_update_file	;update file
	jr	zfu_backup_getdates	;and try opening again
zfu_backup_getdates1
	ld	bc,000ch		;set length to get
	ld	hl,0		;move to memory
	ld	de,zfu_backup_full_date
	sys_call	os_mv		;get bytes
	jp	c,zfu_main_error
	sys_call	gn_cl		;close file
	jp	c,zfu_main_error
	ret			;return
;
zfu_backup_update_file			;output .cfg file with dates
	ld	hl,zfu_backup_file_name	;open file for output
	ld	b,0
	ld	a,02h		;open for output
	ld	de,zfu_buffer	;scratch space
	ld	c,40h		;scratch length
	sys_call	gn_opf		;open file
	jp	c,zfu_main_error	;error opening file
	ld	bc,000ch		;put data out to file
	ld	de,0		;set to put data to file
	ld	hl,zfu_backup_full_date	;use initial dates
	sys_call	os_mv		;move to file
	jp	c,zfu_main_error
	sys_call	gn_cl		;close file
	jp	c,zfu_main_error
	ret			;return
;
; ==========================================================================
;
;	Information screen
;
; ==========================================================================
;
info
	call	zfu_close_archive	;close archive file
	call	zfu_open_archive	;and open to update info
	ld	hl,zfu_info_text	;put out information screen
	call	zfu_gnsop
	ld	hl,zfu_buffer
	call	print_freemem	;print available memory
	ld	bc,0801h		;position cursor
	call	gotoxy
	ld	hl,zfu_afilename	;archive file name
	push	hl
	sys_call	gn_sop
	pop	hl		;get file name back
	call	zfu_get_file_info_0	;load info block with data
	ld	bc,0802h		;set cursor
	call	gotoxy
	ld	hl,zfu_file_credate	;output creation date
	ld	de,zfu_buffer	;set up some scratch space
	push	de		;and save a couple copies
	push	de
	push	de
	call	print_date_time	;and print date/time
	ld	bc,2004h		;set cursor
	call	gotoxy
	pop	de
	ld	hl,zfu_credate_buf	;output creation date setting
	call	print_date_time
	ld	bc,2005h		;set cursor
	call	gotoxy
	pop	de
	ld	hl,zfu_moddate_buf	;output creation date setting
	call	print_date_time
	ld	bc,2402h		;set cursor
	call	gotoxy
	pop	de		;get scratch back
	ld	hl,zfu_file_ext
	call	print_long_int	;print it
	call	pause
	clear_screen
	jp	zfu_main_mnu
;
; ==========================================================================
;
;	Add file(s) section
;
; ==========================================================================
;
zfu_addfile
	ld	hl,zfu_addfile_msg	;output Files question
	call	zfu_gnsop
	ld	a,32		;get file names
	ld	b,80
	ld	l,32
	ld	de,zfu_wildcard_name
	sys_call	gn_sip
	jr	c,zfu_addfile	;loop if preempted
	ld	a,(zfu_wildcard_name)	;check for NULL string
	and	a
	jp	z,zfu_main_menu_0	;jump to main menu if no input
	clear_screen		;clear the screen
	call	zfu_addfile_internal	;call actual add file routine
	jr	c,zfu_addfile	;loop on error
	jp	zfu_main_menu_0
;
;	Internal entry point, the file string to be added to the archive
;	should be stored at (zfu_wildcard_name)
;
zfu_addfile_internal			;internal entry point
;
	ld	a,(zfu_wildcard_name)	;check first character
	cp	'\'		;for path or device info
	jr	z,zfu_addfile_path	;and set flag (zfu_savepath_flag) if
	cp	'/'		;if the full path is to
	jr	z,zfu_addfile_path	;be saved, otherwise
	cp	':'		;(zfu_savepath_flag) is 0.
	jr	z,zfu_addfile_path
	xor	a
zfu_addfile_path
	ld	(zfu_savepath_flag),a
;
;	expand file name
;
	ld	b,0		;expand file name
	ld	c,80
	ld	hl,zfu_wildcard_name
	ld	de,zfu_wildcard_str
	sys_call	gn_fex
	ret	c	;invalid filename
;
;	set up wild card handle for file
;
	ld	hl,zfu_wildcard_str	;set expanded name address
	xor	a		;zero a
	ld	b,a
	sys_call	gn_opw		;open wildcard
	ret	c		;invalid string or no memory
	ld	(zfu_wildcard_handle),ix	;save wild handle
	ld	a,0
	call	zfu_scan_file	;set to end of file
;
zfu_addfile_nextwild
	ld	ix,(zfu_wildcard_handle)	;get wild handle
	ld	de,zfu_wildcard_name	;set address for string
	ld	c,80		;and length
	sys_call	gn_wfn		;get next file name
	jp	c,zfu_addfile_done	;exit on any error
	cp	011h		;skip if non-file (DOR 11h)
	jr	nz,zfu_addfile_nextwild
;
;	check for match with archive name and ignore if so
;
;	Since there may be multiple copies saved out with the fixed
;	size option, loop through all sizes created by this archive.
;
	ld	b,0
	ld	de,zfu_afilename_orig	;expanded original name
	ld	hl,zfu_wildcard_name	;expanded file name
	sys_call	gn_cme		;system compare routine
	jr	z,zfu_addfile_nextwild	;don't add, its a match
	ld	a,(zfu_afile_index)	;get index
	and	a		;check for zero
	jr	z,zfu_addfile_addfile	;no versions to check
	ld	b,a		;put count in b
	ld	hl,zfu_afilename_orig	;put original name in scratch
	ld	de,zfu_buffer
	call	copy_string
zfu_addfile_verchk
	push	bc		;save counter
	ld	hl,zfu_buffer	;increment name
	push	hl		;save address
	ld	de,zfu_buffer+80h	;scratch space
	pop	de		;get name
	ld	hl,zfu_wildcard_name	;compare with name
	sys_call	gn_cme
	jr	z,zfu_addfile_novers	;if match skip
	ld	hl,zfu_buffer	;increment file name in
	call	increment_file_name	;scratch buffer
	pop	bc		;get counter
	djnz	zfu_addfile_verchk	;loop till count is done
	jr	zfu_addfile_addfile	;no match, add file
;
zfu_addfile_novers
	pop	bc		;clear stack
	jr	zfu_addfile_nextwild	;and loop for more names
;
zfu_addfile_addfile			;try and add this file
	call	zfu_get_file_info	;get file information
	jr	c,zfu_addfile_nextwild	;skip if some sort of error
;
;	Check file dates to see if we want to add it to the archive
;
	ld	hl,zfu_credate_buf+3	;check creation date
	ld	de,zfu_file_credate+3
	call	zfu_compare_date
	cp	040h		;0c0 if de<hl
	jr	nz,zfu_addfile_dateok	;add file
	ld	hl,zfu_moddate_buf+3	;check modification date
	ld	de,zfu_file_moddate+3
	call	zfu_compare_date
	cp	040h
	jp	z,zfu_addfile_nextwild		;don't add
zfu_addfile_dateok
;
; ==========================================================================
;
;	File is to be added to the archive, the first thing to
;	do is to check the current size of the archive to see if
;	adding the file will put the archive file over the limit.
;
; ==========================================================================
;
	ld	hl,zfu_arcfile_ptr	;get file ptr (size)
	call	fpi_loadhl		;and load as long int
	ld	de,zfu_file_ext	;get file length
	call	fpi_loadde		;and put in DEde
	sys_callfp	fp_add		;and add
	exx			;get to alt set
	ld	de,0100h		;add 256 for safety
	exx
	ld	de,0000h		;clean main
	ld	b,d		;and bc
	ld	c,e
	sys_callfp	fp_add		;and add
	ld	de,zfu_arcfile_limit	;get file size limit
	call	fpi_loadde		;and put in DEde
	sys_callfp	fp_comp		;and compare
	cp	0c0h		;check for negative
	jr	z,zfu_addfile_sizeok	;size is OK
;
;	File size is over the limit, check to make sure we are not at
;	the start of the file (trying to save a file larger than the
;	specified limit), and if so add it anyway
;
	ld	hl,zfu_arcfile_ptr	;get ptr
	call	fpi_loadhl		;and load
	ld	de,zfu_arcfile_origin	;set starting pointer
	call	fpi_loadde
	sys_callfp	fp_comp		;compare
	and	a		;check for zero
	jr	z,zfu_addfile_sizeok	;add anyway if match
;
;	Need to open another archive file
;
	call	zfu_close_archive	;close current
	ld	hl,zfu_afilename	;increment name
	ld	de,zfu_buffer
	call	increment_file_name	;next file name
	call	zfu_open_archive	;open the new archive file
	ld	hl,zfu_afile_index	;and count the new version
	inc	(hl)
	ld	a,0
	call	zfu_scan_file	;set to end of this file
;
;	Now that file is created, pause if need be
;
	ld	a,(zfu_newfile_pause)	;get pause flag
	and	a		;and check for zero
	jr	z,zfu_addfile_sizeok
	ld	hl,zfu_newarcfile_msg	;put out message
	sys_call	gn_sop
	call	pause		;and pause
zfu_addfile_sizeok
;
;	Output adding files screen
;
	ld	hl,zfu_addfile_msg1	;initial screen
	call	zfu_gnsop
	ld	hl,zfu_wildcard_name	;expanded file name
	sys_call	gn_sop
	ld	hl,zfu_addfile_msg2	;position for next file
	sys_call	gn_sop
	call	zfu_sel_fname
	sys_call	gn_sop
	ld	hl,zfu_addfile_msg3	;file size prefix
	sys_call	gn_sop
	ld	hl,zfu_file_ext	;set to file extent
	call	zfu_print_int	;print out as an int
	ld	hl,zfu_addfile_msg4	;free memory prefix
	sys_call	gn_sop
	ld	hl,zfu_buffer	;scratch space
	call	print_freemem	;print free memory
	ld	hl,zfu_addfile_msg5	;set for status
	sys_call	gn_sop
	ld	a,40		;set up for bar graph
	ld	bc,01206h
	call	bar_display_init
	ld	hl,0		;zero output byte counter
	ld	(zfu_out_byte),hl
	ld	(zfu_out_byte+2),hl
	xor	a		;zero out busy counter
	ld	(zfu_busy_cnt),a
	ld	hl,zfu_statmsg_adding	;put out status message
	call	zfu_status_display
;
;	open file, output header info
;
	ld	a,1
	ld	b,0
	ld	c,80
	ld	hl,zfu_wildcard_name
	ld	de,zfu_buffer
	sys_call	gn_opf
	ld	(zfu_filehandle),ix
;
;	put out first header
;
	call	zfu_sel_fname
	ld	(addtemp),hl
	call	get_string_length
	ld	(flnlen1),a
	xor	a
	ld	(flnlen),a
	call	zfu_arcfile_buf_write
	ld	a,(flnlen1)
	ld	(flnlen),a
;
;	put out file
;
	ld	hl,0
	ld	(zfu_checksum),hl	;init counters
	ld	de,tptr
	ld	hl,zfu_file_ext
	call	fpi_move	;put length in tptr
	ld	a,(cmpflg)
	and	a
	jp	z,zfu_addfilenc	;compress or not?
;
;	write	out file in compressed form
;
adfc0
	call	zfu_init_lbuf
	call	zfu_out_chi
	xor	a
	ld	(sbufl),a	;init
	ld	hl,zfu_cmpfile_ext
	xor	a
	ld	(hl),a
	inc	hl
	ld	(hl),a
	inc	hl
	ld	(hl),a
	inc	hl
	ld	(hl),a
adfc2
	ld	ix,(zfu_filehandle)
	sys_call	os_gb
	jr	nc,adfc3
	ld	hl,aermsg
	sys_call	gn_sop
	jp	zfu_addfile_nextwild
adfc3
	push	af
	call	chkupd
	call	zfu_busy_display
	pop	af
adfc4
	push	af
	ld	hl,sbufl
	ld	a,(hl)
	inc	(hl)
	ld	d,a
	ld	c,a
	ld	b,0
	ld	hl,sbuf
	add	hl,bc
	pop	af
	ld	(hl),a
	ld	a,d
	and	a
	jr	z,adfc11	;get another if 0 count
	ld	a,(hl)
	push	af
	call	zfu_scan_buffer
	jr	z,adfc10	;found
	pop	af
	ld	(charsave),a
	ld	hl,sbufl
	dec	(hl)
	call	zfu_out_data
	xor	a
	ld	(sbufl),a
	ld	a,(charsave)
	jp	adfc4
adfc10
	pop	af
	ld	a,(sbufl)
	cp	34
	jp	m,adfc11
	call	zfu_out_data	;buffer full, output
adfc11
	ld	hl,tptr
	call	fpi_dec
	and	a
	jp	nz,adfc2
	call	zfu_out_data
	ld	a,002h
	ld	(zfu_arcfile_attr),a
	call	zfu_out_chf
	jp	addf5

zfu_init_lbuf
	ld	hl,zfu_buffer
	ld	bc,00830h
zfu_init_lbuf1
	ld	a,32
	ld	(hl),a
	inc	hl
	dec	bc
	ld	a,b
	and	a
	jr	nz,zfu_init_lbuf1
	ld	a,c
	and	a
	jr	nz,zfu_init_lbuf1
	ld	hl,zfu_buffer
	ld	(lbptr),hl
	ret
;
;	uncompressed write
;
zfu_addfilenc
	ld	ix,(zfu_filehandle)
	sys_call	os_gb
	jr	nc,zfu_addfilenca
	ld	hl,aermsg
	sys_call	gn_sop
	jp	zfu_addfile_nextwild	;if error, msg and skip file
aermsg
	db	13
	db	7
	db	"error reading file.."
	db	13
	db	10
	db	0
;
zfu_addfilenca
	push	af		;update checksum and busy display
	call	chkupd
	call	zfu_busy_display
	pop	af
	ld	ix,(zfu_arcfile_handle)
	sys_calle	os_pb,zfu_main_error
	ld	hl,tptr
	call	fpi_dec
	and	a
	jp	nz,zfu_addfilenc	;loop for more data
	ld	de,zfu_cmpfile_ext
	ld	hl,zfu_file_ext
	call	fpi_move
	xor	a
	ld	(zfu_arcfile_attr),a
;
;	close out file and update pointers
;
addf5
	ld	a,001h
	ld	ix,(zfu_arcfile_handle)
	ld	de,0
	sys_calle	os_frm,zfu_main_error
	ld	hl,tptr
	ld	(hl),c
	inc	hl
	ld	(hl),b
	inc	hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	b,64
	ld	ix,(zfu_arcfile_handle)
addf5a
	xor	a
	push	bc
	sys_calle	os_pb,zfu_main_error
	pop	bc
	djnz	addf5a
	ld	de,zfu_arcfile_ptr
	ld	hl,zfu_arcfile_ptr_save
	call	fpi_move
	call	zfu_arcfile_buf_write
	ld	de,zfu_arcfile_ptr
	ld	hl,tptr
	call	fpi_move
;
;	close out file
;
	ld	ix,(zfu_filehandle)	;set file handle
	sys_call	gn_cl		;close file
	ld	hl,0		;and zero handle
	ld	(zfu_filehandle),hl
	call	zfu_busy_display_1	;force full output of bar display
	jp	zfu_addfile_nextwild
;
zfu_addfile_done
	ld	ix,(zfu_wildcard_handle)	;get wildcard handle
	sys_call	gn_wcl		;and close out
	ld	hl,0		;then zero out mem copy
	ld	(zfu_wildcard_handle),hl
	ret			;and return
;
zfu_out_chi
	ld	hl,zfu_out_cnt
	ld	(hl),8
	inc	hl
	ld	(hl),0
	ld	hl,zfu_out_buf
	ld	(zfu_out_add),hl
	ret

zfu_out_ch
	and	a
	jr	nz,zfu_out_cha
	call	zfu_out_m1
	ld	a,b
	call	zfu_out_ch1
	ret
zfu_out_cha
	call	zfu_out_m0
	ld	a,b
	call	zfu_out_ch1
	call	zfu_out_m0
	ld	a,c
	call	zfu_out_ch1
	ret
zfu_out_ch1
	ld	hl,(zfu_out_add)
	ld	(hl),a
	inc	hl
	ld	(zfu_out_add),hl
	ld	hl,zfu_out_cnt
	dec	(hl)
	ret	nz
	push	bc
	call	zfu_out_chf
	pop	bc
	ret

zfu_out_chf
	ld	a,(zfu_out_cnt)
	cp	8
	ret	z
zfu_out_chf0
	and	a
	jr	z,zfu_out_chf1
	call	zfu_out_m0
	dec	a
	jr	zfu_out_chf0
zfu_out_chf1
	ld	hl,zfu_out_cnt
	ld	a,9
	sub	(hl)
	ld	e,a
	ld	d,0
	ld	hl,zfu_cmpfile_ext
	push	de
	call	fpi_addi
	pop	bc
	ld	ix,(zfu_arcfile_handle)
	ld	de,0
	ld	hl,zfu_out_msk
	sys_calle	os_mv,zfu_main_error
	ld	a,8
	ld	(zfu_out_cnt),a
	ld	hl,zfu_out_buf
	ld	(zfu_out_add),hl
	ret

zfu_out_m1
	ld	hl,zfu_out_msk
	sla	(hl)
	inc	(hl)
	ret
zfu_out_m0
	ld	hl,zfu_out_msk
	sla	(hl)
	ret
;
; ==========================================================================
;
;	Scan Buffer for a string match
;
; ==========================================================================
;
zfu_scan_buffer
	ld	hl,zfu_buffer
	ld	bc,00800h
	ld	de,sbuf
zfu_scanbuf_1
	ld	a,(de)
	cpir
	ret	nz
	push	hl
	push	de
	push	bc
	ld	a,(sbufl)
	dec	a
	ld	b,a
zfu_scanbuf_2
	inc	de
	ld	a,(de)
	cp	(hl)
	jr	nz,zfu_scanbuf_3
	inc	hl
	djnz	zfu_scanbuf_2
	pop	bc
	pop	de
	pop	hl	;save offset
	dec	hl
	ld	de,zfu_buffer
	and	a
	sbc	hl,de
	ld	(lbsptr),hl
	xor	a
	ret
zfu_scanbuf_3
	pop	bc
	pop	de
	pop	hl
	ld	a,b
	or	c
	jr	nz,zfu_scanbuf_1
	ld	a,0ffh
	and	a
	ret

zfu_out_data
	ld	a,(sbufl)
	cp	3
	jr	nc,zfu_out_dat1
;	just	output characters
	ld	a,(sbufl)
	ld	hl,sbuf
zfu_out_dat0
	push	af
	ld	a,(hl)
	inc	hl
	push	hl
	ld	b,a
	xor	a
	push	bc
	call	zfu_out_ch
	pop	bc
	ld	a,b
	call	zfu_out_atbuf
	pop	hl
	pop	af
	dec	a
	jr	nz,zfu_out_dat0
	jr	zfu_out_dat2
zfu_out_dat1
	ld	a,(sbufl)
	ld	c,a
	ld	hl,sbuf	;update large buffer
zfu_out_dat1a
	push	hl
	push	bc
	ld	a,(hl)
	call	zfu_out_atbuf
	pop	bc
	pop	hl
	inc	hl
	dec	c
	jr	nz,zfu_out_dat1a
	ld	hl,(lbsptr)
	ld	a,007h
	and	h
	ld	h,a
	ld	a,(sbufl)
	dec	a
	dec	a
	dec	a
	rlca
	rlca
	rlca
	or	h
	ld	h,a
	ld	c,l
	ld	b,h
	ld	a,0ffh
	call	zfu_out_ch
zfu_out_dat2
	xor	a
	ld	(sbufl),a
	ret

zfu_out_atbuf
	push	af
	ld	hl,(lbptr)
	ld	(hl),a
	ld	de,zfu_buffer
	and	a
	sbc	hl,de
	push	hl
	ld	hl,(lbptr)
	pop	de
	push	de
	ld	a,d
	and	a
	jr	nz,zfu_out_atb2
	ld	a,e
	cp	35
	jr	nc,zfu_out_atb2
	ld	de,00800h
	add	hl,de
	pop	de
	pop	af
	push	de
	ld	(hl),a
	jr	zfu_out_atb1	;save at end of buffer
zfu_out_atb2
	pop	hl
	pop	af
	push	hl
zfu_out_atb1
	ld	hl,(lbptr)
	inc	hl
	ld	(lbptr),hl
	pop	hl
	ld	a,h
	and	0f8h
	ret	z
	ld	hl,zfu_buffer
	ld	(lbptr),hl
	ret
;
; ==========================================================================
;
;	Compare two date/time blocks in (DE) and (HL) and return with
;	the status flags set accordingly.
;
; ==========================================================================
;
zfu_compare_date			;compare two date/time blocks
	push	de		;save de and hl
	push	hl
	call	zfu_compare_date_1
	and	a		;check for same date
	jr	z,zfu_compare_date_0	;if so go check time
	pop	hl		;clean stack
	pop	de
	ret			;and return
zfu_compare_date_0
	pop	hl		;get addresses back
	dec	hl		;and decrement them
	dec	hl
	dec	hl
	pop	de
	dec	de
	dec	de
	dec	de		;and fall on through to compare
;
zfu_compare_date_1			;compare 3 bytes at (de) & (hl)
	push	de		;save first date
	ld	b,0		;zero b (MSB)
	ld	e,(hl)		;get (DE) data in BCDE
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	pop	hl		;get first date back
	push	bc		;save second date
	push	de
	ld	b,0		;zero b (MSB)
	ld	e,(hl)		;get (HL) data in BCDE
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	push	de		;save lower bytes of second
	exx
	pop	de		;and put lower bytes in dehl
	pop	hl
	exx
	pop	hl		;get second date back
	ld	d,b		;copy first date to de
	ld	e,c
	ld	b,0		;zero b,c (for int calc)
	ld	c,b
	sys_callfp	fp_comp		;and do a floating compare
	ret
;
; ==========================================================================
;
;	Scan through the archive file
;
;	If A=00 on entry the file is searched block by block with
;	no display, if A=0ffh then the file information is written
;	out for each block as it occurs.
;
; ==========================================================================
;
zfu_scan_file
	ld	(scaflg),a		;save option flag
	ld	a,001h
	ld	ix,(zfu_arcfile_handle)		;set to start of file
	ld	hl,zfu_arcfile_origin
	sys_call	os_fwm
	ld	hl,zfu_arcfile_origin		;set (zfu_arcfile_ptr) to match file
	ld	de,zfu_arcfile_ptr		;pointer for future calc.
	call	fpi_move
zfu_scan_file1
	call	zfu_arcblock_read	;read a header
	ret	z		;return if EOF
	ld	a,(scaflg)		;get option flag back
	and	a
	jr	z,zfu_scan_file1	;just loop if no output
	ld	hl,zfu_filename
	ld	c,57
	call	dis_pad  		;output file name
	ld	hl,zfu_file_ext
	call	zfu_print_int 	;output file size
	ld	hl,zfu_cmpfile_ext
	call	zfu_print_int	;output compressed size
	ld	hl,(zfu_checksum)	;get checksum address
	call	hl_to_hexdis	;output checksum
	sys_call	gn_nln		;and cr/lf
	jr	zfu_scan_file1	;loop back for next rec.

; ==========================================================================
;
;	Archive file block read
;
;	This routine reads in a header block from the archive file,
;	saving all needed information in the dynamic data storage
;	area.
;
;	The file name is read into (zfu_filename), while the data
;	block for the file is read into a 25 byte block starting at
;	(zfu_arcfile_attr).
;
; ==========================================================================
;
zfu_arcblock_read
	ld	a,001h		;make sure file pointer
	ld	hl,zfu_arcfile_ptr		;is current with our data
	ld	ix,(zfu_arcfile_handle)
	sys_calle	os_fwm,zfu_main_error
	ld	ix,(zfu_arcfile_handle)		;get first byte from header
	sys_calle	os_gb,zfu_main_error
	ld	(flnlen),a		;this is the file length
	and	a
	ret	z		;if 0, return with Z flag set
	call	aptrupd		;update pointer
	ld	c,a
	ld	b,0
	ld	hl,0
	ld	de,zfu_filename
	sys_calle	os_mv,zfu_main_error
	xor	a
	ld	(de),a
	ld	c,25
	ld	b,0
	ld	hl,0
	ld	de,zfu_arcfile_attr
	sys_calle	os_mv,zfu_main_error
	ld	a,(flnlen)
	add	a,26
	ld	d,0
	ld	e,a	;compute block length
	ld	hl,(zfu_arcfile_ptr)
	add	hl,de
	ld	(zfu_arcfile_ptr),hl
	ld	hl,(zfu_arcfile_ptr+2)
	ld	de,0
	adc	hl,de
	ld	(zfu_arcfile_ptr+2),hl
	ld	hl,(zfu_arcfile_ptr)
	ld	de,(zfu_cmpfile_ext)	;add in length of the file
	add	hl,de
	ld	(zfu_arcfile_ptr),hl
	ld	hl,(zfu_arcfile_ptr+2)
	ld	de,(zfu_cmpfile_ext+2)
	adc	hl,de
	ld	(zfu_arcfile_ptr+2),hl
	ld	a,01
	ld	hl,zfu_arcfile_ptr
	ld	ix,(zfu_arcfile_handle)
	sys_calle	os_fwm,zfu_main_error
	ret
;
; ==========================================================================
;
;	Update archive file pointer
;
;	This routine updates the saved file pointer with the 
;	temporary pointer
;
; ==========================================================================
;
aptrupd
	ld	de,zfu_arcfile_ptr_save
	ld	hl,zfu_arcfile_ptr
	jp	fpi_move
;
zfu_arcfile_buf_write
	ld	hl,zfu_arcfile_ptr
	ld	de,zfu_arcfile_ptr_save
	call	fpi_move
	ld	ix,(zfu_arcfile_handle)
	ld	a,001h
	ld	hl,zfu_arcfile_ptr
	sys_calle	os_fwm,zfu_main_error
	ld	a,(flnlen)
	sys_calle	os_pb,zfu_main_error
	ld	a,(flnlen1)
	ld	hl,(addtemp)
	call	zfu_afbw_0
	ld	a,25
	ld	hl,zfu_arcfile_attr
	call	zfu_afbw_0
	ret

zfu_afbw_0
	ld	ix,(zfu_arcfile_handle)
	ld	b,a
zfu_afbw_1
	ld	a,(hl)
	push	hl
	push	bc
	sys_calle	os_pb,zfu_main_error
	pop	bc
	pop	hl
	inc	hl
	djnz	zfu_afbw_1
	ret
;
; ==========================================================================
;
;	Extract files section
;
; ==========================================================================
;
zfu_extract_file
	ld	hl,zfu_extfile_msg	;output Files question
	call	zfu_gnsop
	ld	a,32		;get file names
	ld	b,80
	ld	l,32
	ld	de,zfu_wildcard_name
	sys_call	gn_sip
	jr	c,zfu_extract_file	;loop if preempted
	ld	a,(zfu_wildcard_name)	;check for NULL string
	and	a
	jp	z,zfu_main_menu_0	;jump to main menu if no input
;
	clear_screen		;clear the screen
	ld	hl,zfu_arcfile_origin	;set the start of the scan
	ld	de,zfu_arcfile_ptr	;to the start of the file
	call	fpi_move
;
zfu_extfile_1
	call	zfu_arcblock_read	;read the next block
	jr	nz,zfu_extfile_1i	;if valid block no exit
	ld	hl,zfu_extract_msgd	;put out final message
	sys_call	gn_sop
	call	pause		;and wait for user input
	jp	zfu_main_menu_0	;then go back to main menu
zfu_extfile_1i
;
;	Check the file at this point to see if it is one to be
;	extracted, and if so, extract it.
;
	ld	bc,zfu_buffer	;get scratch buffer
	ld	de,zfu_wildcard_name	;get wildcard
	ld	hl,zfu_filename	;file name
	call	wildcard_match	;call compare routine
	jr	nz,zfu_extfile_1	;go for the next block
	ld	hl,zfu_checksum	;get the checksum
	ld	d,(hl)		;and save in (fcsum)
	inc	hl
	ld	e,(hl)
	ex	de,hl
	ld	(fcsum),hl		;save the files checksum
	ld	a,(zfu_arcfile_attr)	;check for compressed file
	and	002h
	jr	z,zfu_extfile_1a	;and set if file was compressed
	ld	a,0ffh		;set compression flag
zfu_extfile_1a
	ld	(cmpflg),a		;save compression flag
	ld	hl,zfu_filename	;get file name and
	call	zfu_fn_adjust	;clean up if need be
	ld	hl,zfu_filename
	ld	de,zfu_buffer
	call	open_file		;open file
	jr	nc,zfu_extract_1b
	ld	a,020h
	jp	zfu_main_error
zfu_extract_1b
	ld	(zfu_filehandle),ix	;save file handle
;
;	Output extracting files screen
;
zfu_extract_init_screen
	ld	hl,zfu_extfile_msg1	;initial screen
	call	zfu_gnsop
	ld	hl,zfu_filename	;stored file name
	sys_call	gn_sop
	ld	hl,zfu_addfile_msg2	;position for next file
	sys_call	gn_sop
	ld	hl,zfu_buffer	;expand name from gn_opf call
	sys_call	gn_sop
	ld	hl,zfu_addfile_msg3	;file size prefix
	sys_call	gn_sop
	ld	hl,zfu_file_ext	;set to file extent
	call	zfu_print_int	;print out as an int
	ld	hl,zfu_addfile_msg4	;free memory prefix
	sys_call	gn_sop
	ld	hl,zfu_buffer	;scratch space
	call	print_freemem	;print free memory
	ld	hl,zfu_addfile_msg5	;set for status
	sys_call	gn_sop
	ld	a,40		;set up for bar graph
	ld	bc,01206h
	call	bar_display_init
	ld	hl,0		;zero output byte counter
	ld	(zfu_out_byte),hl
	ld	(zfu_out_byte+2),hl
	xor	a		;zero out busy counter
	ld	(zfu_busy_cnt),a
;
	ld	a,(flnlen)		;get file name length
	add	a,26		;add size of header
	ld	d,0		;then put in DE and add
	ld	e,a		;to pointer
	ld	hl,zfu_arcfile_ptr_save
	call	fpi_addi
	ld	a,01		;position archive file to start
	ld	hl,zfu_arcfile_ptr_save	;of data
	ld	ix,(zfu_arcfile_handle)
	sys_calle	os_fwm,zfu_main_error
	ld	de,tptr
	ld	hl,zfu_cmpfile_ext
	call	fpi_move
	ld	hl,tptr
	call	fpi_inc
	ld	a,1
	ld	(zfu_out_cnt),a
	ld	hl,0
	ld	(zfu_checksum),hl
	call	zfu_init_lbuf
;
zfu_extfile_2
	ld	ix,(zfu_arcfile_handle)
	sys_calle	os_gb,zfu_main_error
	push	af
	ld	hl,tptr
	call	fpi_dec
	and	a
	jp	z,zfu_extfile_10
	ld	a,(cmpflg)
	and	a
	jr	nz,zfu_extfilec_0
	pop	af
	call	zfu_extfc_outchar
	jr	zfu_extfile_2
zfu_extfilec_0
	ld	hl,zfu_out_cnt
	dec	(hl)
	jr	nz,zfu_extfilec_1	;get mask byte if need be
	ld	a,9
	ld	(hl),a
	pop	af
	ld	(zfu_out_msk),a
	jr	zfu_extfile_2
zfu_extfilec_1
	ld	hl,zfu_out_msk
	sla	(hl)
	jr	c,zfu_extfilec_5	;normal byte
zfu_extfilec_1a
	ld	ix,(zfu_arcfile_handle)
	sys_calle	os_gb,zfu_main_error
	push	af	;get rest of index
	ld	hl,tptr
	call	fpi_dec
	ld	hl,zfu_out_cnt
	dec	(hl)
	jr	nz,zfu_extfilec_2
	ld	a,9
	ld	(hl),a
	pop	af
	ld	(zfu_out_msk),a
	jr	zfu_extfilec_1a
zfu_extfilec_2
	ld	hl,zfu_out_msk
	sla	(hl)
	pop	af
	ld	c,a
	pop	af
	ld	b,a
	push	bc
	pop	de
	ld	a,d
	and	007h
	ld	d,a
	ld	hl,zfu_buffer
	add	hl,de	;get address
	ld	a,b
	rra
	rra
	rra
	and	01fh
	inc	a
	inc	a
	inc	a
	ld	b,a	;and length
	push	bc
	push	hl
zfu_extfilec_10
	push	bc
	push	hl
	ld	a,(hl)
	call	zfu_extfc_outchar
	pop	hl
	pop	bc
	inc	hl
	djnz	zfu_extfilec_10
	pop	hl
	pop	bc
	push	bc
	ld	c,b
	ld	b,0
	ld	de,sbuf
	push	de
	ldir
	pop	hl
	pop	bc
zfu_extfilec_11
	push	bc
	push	hl
	ld	a,(hl)
	call	zfu_out_atbuf
	pop	hl
	pop	bc
	inc	hl
	djnz	zfu_extfilec_11
	jp	zfu_extfile_2

zfu_extfilec_5
	pop	af
	push	af
	call	zfu_extfc_outchar
	pop	af
	call	zfu_out_atbuf
	jp	zfu_extfile_2

zfu_extfc_outchar
	push	af		;save a
	call	chkupd		;update checksum
	call	zfu_busy_display	;and busy display
	pop	af		;get character back
	ld	ix,(zfu_filehandle)	;and write to the file
	sys_calle	os_pb,zfu_main_error
	ret			;return

zfu_extfile_10
	pop	af
	ld	ix,(zfu_filehandle)
	sys_call	gn_cl
	ld	hl,0
	ld	(zfu_filehandle),hl
	ld	hl,(zfu_checksum)	;get checksum and compare
	ld	de,(fcsum)		;note that checksum in HL
	ld	a,d		;is in reverse from stored
	cp	l		;format used in file
	jr	nz,zfu_extfile_5
	ld	a,e
	cp	h
	jr	nz,zfu_extfile_5
	jr	zfu_extfile_9
zfu_extfile_5			;checksum error extracting file
	ld	hl,zfu_statmsg_chkerr	;get error message
	call	zfu_status_display	;put it out
	call	pause		;and wait till ack.
zfu_extfile_9
	call	zfu_busy_display_1	;force full output of bar disp.
	jp	zfu_extfile_1	;jump back for another file
;
; ==========================================================================
;
;	List out archive contents
;
; ==========================================================================
;
zfu_listfile
	clear_screen
	ld	a,0ffh
	call	zfu_scan_file
	call	pause
	jp	zfu_main_menu_0
;
; ==========================================================================
;
;	Options menu
;
; ==========================================================================
;
zfu_opt_mnu
	ld	hl,zfu_options_menu_msg
	call	zfu_output_menu
zfu_opt_mnu_0
	clear_screen
zfu_opt_mnu1
	ld	hl,odcm0
	sys_call	gn_sop
	ld	a,(cmpflg)
	and	a
	jr	nz,zfu_opt_dc1
	ld	hl,odcm1
	jr	zfu_opt_dc2
zfu_opt_dc1
	ld	hl,odcm2
zfu_opt_dc2
	sys_call	gn_sop
	ld	hl,odcm3		;set back to main window
	sys_call	gn_sop
zfu_opt_opt
	sys_call	os_in
	or	020h
	cp	'x'		;exit back to main menu
	jp	z,zfu_main_mnu
	cp	'c'		;set create date limit
	jp	z,zfu_opt_cre
	cp	'm'		;set mod date limit
	jp	z,zfu_opt_mod
	cp	'o'		;toggle compression
	jp	z,zfu_opt_dc
	cp	's'		;set file limit
	jp	z,zfu_opt_afile_size
	jp	zfu_opt_opt

zfu_opt_dc
	ld	hl,cmpflg
	ld	a,(hl)
	cpl
	ld	(hl),a
	jp	zfu_opt_mnu_0
;
odcm0	vid_win_sel	3		;select window 3
	vid_gotoxy	0,1		;position cursor
	db	0
odcm1	db	' ',0		;blank space
odcm2	db	1,245,0		;bullet mark
odcm3	vid_win_sel	1		;set back to main window
	db	0
;
zfu_opt_cre
	ld	hl,zfu_options_cre_msg
	ld	de,zfu_credate_buf+3	;offset past time
	call	zfu_opt_gd
	jp	zfu_opt_mnu_0

zfu_opt_gd
	push	de
	push	hl
	ld	hl,zfu_opt_date_prmpt
	sys_call	gn_sop
	pop	hl
	push	hl
	sys_call	gn_sop
	ld	a,020h
	ld	b,80
	ld	l,11
	ld	de,zfu_buffer
	sys_call	gn_sip
	ld	a,0
	pop	hl
	pop	de
	push	hl
	push	de
	ld	hl,zfu_buffer
	sys_call	gn_gdt
	pop	de
	pop	hl
	jr	c,zfu_opt_gd
	ret
;
zfu_opt_mod
	ld	hl,zfu_options_mod_msg
	ld	de,zfu_moddate_buf+3	;offset past time
	call	zfu_opt_gd
	jp	zfu_opt_mnu_0
;
zfu_opt_afile_size			;set archive file size
	ld	hl,zfu_opt_afs_msg	;put out message
	sys_call	gn_sop
zfu_opt_afile_size_mnu
	ld	hl,zfu_opt_afs_menu	;and menu
	call	zfu_output_menu
	ld	hl,zfu_opt_afs_msg0
	sys_call	gn_sop
	ld	a,(zfu_newfile_pause)	;get pause flag
	and	a		;and set display
	jr	nz,zfu_opt_afsm_1	;accordingly
	ld	hl,zfu_opt_afs_msg1
	jr	zfu_opt_afsm_2
zfu_opt_afsm_1
	ld	hl,zfu_opt_afs_msg2
zfu_opt_afsm_2
	sys_call	gn_sop
	ld	hl,zfu_opt_afs_msg3		;set back to main window
	sys_call	gn_sop
zfu_opt_afs_1
	sys_call	os_in		;get a character
	jr	c,zfu_opt_afs_1	;handle error
	call	a_to_lower		;make lower case
	cp	'n'		;check for no limit
	jr	nz,zfu_opt_afs_2
	ld	hl,0ffffh		;set limit
	jr	zfu_opt_afs_99
zfu_opt_afs_2
	cp	'3'		;32K
	jr	nz,zfu_opt_afs_3
	ld	hl,0080h
	jr	zfu_opt_afs_99
zfu_opt_afs_3
	cp	'6'		;64K
	jr	nz,zfu_opt_afs_4
	ld	hl,0100h
	jr	zfu_opt_afs_99
zfu_opt_afs_4
	cp	'1'		;128K
	jr	nz,zfu_opt_afs_5
	ld	hl,0200h
	jr	zfu_opt_afs_99
zfu_opt_afs_5
	cp	'p'		;toggle pause flag
	jr	nz,zfu_opt_afs_1		;go back to menu
	ld	hl,zfu_newfile_pause	;toggle pause flag
	ld	a,(hl)
	cpl
	ld	(hl),a
	jp	zfu_opt_afile_size_mnu	;back to menu redraw
;
zfu_opt_afs_99
	ld	(zfu_arcfile_limit+1),hl	;save size
	jp	zfu_opt_mnu
;
; ==========================================================================
;
;	Get archive file name
;
; ==========================================================================
;
zfu_get_archive_file
	xor	a		;zero out index number
	ld	(zfu_afile_index),a
	ld	hl,zfu_getarcfile_msg	;put out prompt message
	sys_call	gn_sop
;
;	get input line
;
	ld	de,zfu_buffer
	ld	a,32
	ld	b,80
	ld	l,32
	sys_call	gn_sip
	jr	c,zfu_get_archive_file	;loop on error (preempt)
	ld	a,(zfu_buffer)	;check for null input
	and	a		;and exit if so
	jp	z,zfu_done
;
;	expand file name, and add default extension of zfu
;
	ld	b,0
	ld	hl,zfu_buffer	;starting copy
	ld	de,zfu_afilename_orig	;scratch
	ld	c,80h		;scratch length
	sys_call	gn_fex		;expand name
	jr	c,zfu_get_archive_file	;loop on bad syntax
;
;	force extension to .zfu
;
	ld	a,081h		;set to write extension
	ld	b,0ffh		;segment number
	ld	c,80h		;scratch length
	ld	de,zfu_afilename_orig	;scratch
	ld	hl,zfu_arcfile_extension	;extension
	sys_call	gn_esa		;write file name segment
	ld	hl,zfu_afilename_orig	;copy original name to current
	ld	de,zfu_afilename	;file name
	call	copy_string
	call	zfu_open_archive	;open archive file
	ret			;return
;
zfu_open_archive			;open archive file
	ld	a,3		;set to open for update
	ld	b,0		;local address
	ld	c,80h		;scratch length
	ld	de,zfu_buffer	;scratch
	ld	hl,zfu_afilename
	sys_call	gn_opf
	jr	nc,zfu_open_archive_ok	;file open ok
	cp	rc_onf 		;if file not found
	jp	nz,zfu_done		;error creating file
;
	ld	a,2		;open for output
	ld	b,0		;local address
	ld	c,80h		;scratch length
	ld	de,zfu_buffer	;scratch
	ld	hl,zfu_afilename	;file name
	sys_call	gn_opf		;open for output
	jp	c,zfu_done		;error, abort zfu
	ld	(zfu_arcfile_handle),ix	;save handle
	ld	a,0ffh		;now write out starting block
	sys_call	os_pb		;output header block for file
	ld	a,0ffh
	sys_call	os_pb
	ld	a,010h
	sys_call	os_pb
	xor	a
	sys_call	os_pb
	ld	ix,(zfu_arcfile_handle)	;then close out the file
	sys_call	gn_cl
	jr	zfu_open_archive	;loop back to start
;
zfu_open_archive_ok			;file open OK
	ld	(zfu_arcfile_handle),ix	;save file handle
	ret			;and return
;
zfu_close_archive			;close archive file
	ld	hl,zfu_arcfile_handle	;get handle pointer
	call	close_file		;close file
	ret			;and return
;
zfu_cleanup				;cleanup open files/wildcards
	ld	hl,zfu_arcfile_handle
	call	close_file
zfu_cleanup_noarc			;cleanup all but archive file
	ld	hl,zfu_filehandle
	call	close_file
	ld	hl,zfu_wildcard_handle
	call	close_wildcard
	ret
;
zfu_done				;exit point
	ld	hl,(zfu_stack)	;get old stack
	ld	sp,hl		;and make current
	call	zfu_cleanup		;cleanup all files
	ret			;and return
;
zfu_main_error			;main error handler
	call	zfu_cleanup_noarc	;close all but archive file
	sys_call	gn_err		;output error box
	ld	sp,(zfu_stack)	;restore the stack
	jp	zfu_main_menu_0
;
zfu_get_file_info
;
;	get just the file name
;
	xor	a
	ld	hl,zfu_wildcard_name
	ld	de,zfu_filename
	ld	b,0ffh
	ld	c,12
	sys_call	gn_esa
	xor	a
	ld	b,a
	ld	c,a
	ld	hl,zfu_filename
	sys_call	gn_skt
	ld	(hl),'.'
	inc	hl
	ex	de,hl
	ld	hl,zfu_wildcard_name
	ld	a,1
	ld	b,0ffh
	sys_call	gn_esa
	ld	hl,zfu_wildcard_name
;
zfu_get_file_info_0			;put file info in storage block
	ld	a,6
	ld	b,0
	ld	c,80
	ld	de,zfu_buffer
	sys_call	gn_opf
	ld	de,zfu_file_credate
	ld	b,'C'
	ld	c,6
	call	zfu_get_file_info_1
	ld	de,zfu_file_moddate
	ld	b,'U'
	ld	c,6
	call	zfu_get_file_info_1
	ld	de,zfu_fileattr
	ld	b,'A'
	ld	c,2
	call	zfu_get_file_info_1
	ld	de,zfu_file_ext
	ld	b,'X'
	ld	c,4
	call	zfu_get_file_info_1
	ld	a,dr_fre
	sys_call	os_dor
	ret
;
zfu_get_file_info_1
	ld	a,dr_rd
	sys_call	os_dor
	ret
;
dis_pad
	ld	a,(hl)
	and	a
	jr	z,dispd1
	sys_call	os_out
	inc	hl
	dec	c
	jr	nz,dis_pad
	ret
dispd1
	ld	a,020h
	sys_call	os_out
	dec	c
	jr	nz,dispd1
	ret

chkupd
	ld	hl,(zfu_checksum)	;put checksum in HL
	call	crc_update		;call crc routine
	ld	(zfu_checksum),hl	;save back
	ret			;return
;
; ==========================================================================
;
;	Utility and support routines
;
; ==========================================================================
;
zfu_print_int			;output int at (HL)
	call	fpi_loadhl		;load int as FP value
	ld	de,zfu_buffer	;convert to string
	exx
	ld	de,00ah
	exx
	sys_callfp	fp_str
	ld	hl,zfu_buffer	;set address
	ld	c,8		;display padded
	call	dis_pad
	ret			;return
;
zfu_gnsop				;reset main win and output text
	push	hl		;save hl
	ld	hl,zfu_gnsop_msg	;reset main window char
	sys_call	gn_sop
	pop	hl		;get hl back
	sys_call	gn_sop		;and output called text
	ret			;then return
;
zfu_output_menu			;output menu text in (HL)
	push	hl		;save pointer
	ld	hl,zfu_menu_text_pre	;output prefix
	sys_call	gn_sop
	pop	hl		;get text
	sys_call	gn_sop
	ld	hl,zfu_menu_text_suf	;and output suffix
	sys_call	gn_sop
	ret			;return
;
zfu_busy_display			;busy display counter
	ld	hl,zfu_busy_cnt	;increment counter
	inc	(hl)
	ret	nz		;return if non-zero (/256)
	ld	hl,zfu_out_byte	;set address for output
	ld	de,0100h		;add 256
	call	fpi_addi		;to the output byte count
	ld	de,zfu_file_ext	;load file extent
	call	fpi_loadde		;and put in DEde
	ld	hl,zfu_out_byte	;get current output
	call	fpi_loadhl		;and put in HLhl
zfu_busy_display_0
	ld	a,40		;set bar length
	ld	bc,1206h		;and bar location
	call	bar_display		;then display bar
	ret			;and return
zfu_busy_display_1			;full bar entry point
	ld	de,zfu_file_ext	;load file extent
	call	fpi_loadde		;and put in DEde
	ld	hl,zfu_file_ext	;load file extent
	call	fpi_loadhl		;and put in HLhl
	jr	zfu_busy_display_0	;and jump back to do it
;
zfu_status_display			;display status msg (HL)
	push	hl		;save hl
	ld	bc,3a04h		;move to 58,4
	call	gotoxy
	pop	hl		;get status message
	sys_call	gn_sop		;and output it
	ret			;then return
;
zfu_fn_adjust
	xor	a
	ld	b,a
	ld	c,80
	cpir	;find	end of string
	dec	hl
	dec	hl
	ld	a,(hl)
	cp	'.'
	ret	nz	;if trailing . remove it
	xor	a
	ld	(hl),a
	ret	;zero	out .
;
;
;	Set HL to point to appropriate file name (either short or long)
;	based on the current value of the (zfu_savepath_flag) flag.
;
zfu_sel_fname
	ld	a,(zfu_savepath_flag)
	and	a
	jr	z,zfu_sel_fname1
	ld	hl,zfu_wildcard_name
	ret
zfu_sel_fname1
	ld	hl,zfu_filename
	ret
;
; ==========================================================================
;
;	Static storage area
;
; ==========================================================================
;
zfu_arcfile_origin			;start of data for .zfu file
	dw	00003h
	dw	00000h
;
zfu_gnsop_msg			;main window reset
	vid_cursor_off
	vid_just_normal
	db	12,0
;
zfu_info_text
	vid_cursor_off
	db	12,10,' Archive ',13,10,' Created'
	vid_gotoxy	31,2
	db	'Size '
	vid_gotoxy	5,4
	db	'Date Limits:'
	vid_gotoxy	18,4
	db	'Creation'
	vid_gotoxy	18,5
	db	'Modification'
	vid_gotoxy	45,2
	db	'Free Memory ',0
;
zfu_opt_date_prmpt			;get date prompte
	db	10,10,0
zfu_options_cre_msg			;creation date prompt
	db	'Creation date ',0
zfu_options_mod_msg			;modification date prompt
	db	'Modification date ',0

zfu_options_menu_msg			;options menu
	db	10,' c',1,'B','O',1,'B','mpress',13,10
	db	' ',1,'B','C',1,'B','re date',13,10
	db	' ',1,'B','M',1,'B','od Date',13,10
	db	' ',1,'B','S',1,'B','ize',13,10
	db	' e',1,'B','X',1,'B','it',10
	db	0
;
zfu_opt_afs_msg			;set bank size message
	db	12,10
	vid_just_center
	db	'Set the maximum size for the archive file',10,10
	db	'When the archive file reaches this size a new file',10
	db	'will be started with a number suffix added',10,0
;
zfu_opt_afs_menu			;archive file size menu
	db	10,' ',1,'B','N',1,'B','o limit',13,10
	db	' ',1,'B','3',1,'B','2 K',13,10
	db	' ',1,'B','6',1,'B','4 K',13,10
	db	' ',1,'B','1',1,'B','28 K',13,10
	db	' ',1,'B','P',1,'B','ause',0
zfu_opt_afs_msg0
	vid_win_sel	3		;select window 3
	vid_gotoxy	0,5		;position cursor
	db	0
zfu_opt_afs_msg1
	db	' ',0		;blank space
zfu_opt_afs_msg2
	db	1,245,0		;bullet mark
zfu_opt_afs_msg3
	vid_win_sel	1		;set back to main window
	db	0
;
zfu_addfile_msg			;adding files message
	db	10,10,' Files to add ',0
zfu_addfile_msg1			;adding file text 1
	db	'Adding:',13,10,'  ',0
zfu_addfile_msg2
	db	13,10,'As:',13,10,'  ',0
zfu_addfile_msg3
	db	13,10,'File size: ',0
zfu_addfile_msg4
	vid_gotoxy	20,4
	db	'Available Memory: ',0
zfu_addfile_msg5
	vid_gotoxy	50,4
	db	'Status: '
	vid_tiny_toggle
	vid_gotoxy	17,5
	db	'0%'
	vid_gotoxy	36,5
	db	'50%'
	vid_gotoxy	56,5
	db	'100%'
	vid_tiny_toggle
	db	0
;
zfu_extract_msgd			;done extracting message
	db	12,10,10,10
	vid_just_center
	db	'Extraction Complete',10
	vid_just_normal
	db	0
zfu_extfile_msg			;extracting files message
	db	10,10,' Files to extract ',0
zfu_extfile_msg1			;extracting file text 1
	db	'Extracting:',13,10,'  ',0
;
;	Status messages for adding and extracting files
;
zfu_statmsg_adding			;adding files
	db	'Adding....',0
zfu_statmsg_extract			;extracting files
	db	'Extracting....',0
zfu_statmsg_chkerr			;checksum error
	vid_flashing_toggle
	db	'Checksum Error'
	vid_flashing_toggle
	db	0
;
zfu_getarcfile_msg
	db	12,10,10,' Archive file ',0
;
zfu_arcfile_extension
	db	'zfu',13
;
zfu_main_mnum			;main menu
	db	10,' ',1,'B','B',1,'B','ackups',13,10
	db	' ',1,'B','F',1,'B','iles',13,10
	db	' ',1,'B','I',1,'B','nfo',13,10
	db	' ',1,'B','O',1,'B','ptions',13,10,10
	db	' e',1,'B','X',1,'B','it'
	db	0
zfu_backup_mnutxt			;backup menu text
	db	10,' ',1,'B','F',1,'B','ull',13,10
	db	' ',1,'B','D',1,'B','iff.',13,10
	db	' ',1,'B','I',1,'B','ncr.',13,10,10
	db	' e',1,'B','X',1,'B','it'
	db	0
;
zfu_file_mnutext			;file menu text
	db	10
	db	' ',1,'B','A',1,'B','dd',13,10
	db	' ',1,'B','E',1,'B','xtract',13,10
	db	' ',1,'B','L',1,'B','ist',13,10
	db	10
	db	' e',1,'B','X',1,'B','it'
	db	0
;
zfu_mainwin
	vid_win_titled	3,80,0,10,8,'MENU'
	vid_win_titled	1,0,0,78,8,'ZFU - Z88 FILE UTILITY V2.03'
	db	0
zfu_main_win_init			;reset main window
	vid_just_normal		;normal justify
	vid_cursor_off		;cursor off
	db	12		;and clear screen
	db	0
;
zfu_menu_text_pre			;prefix for menu output
	vid_win_sel	3		;select window 3
	db	12		;and clear it
	db	0
zfu_menu_text_suf			;suffix for menu output
	vid_win_sel	1
	db	0
;
zfu_pname	db	"ZFU v2.03"
	db	0
;
zfu_main_errmsg1
	db	10,10
	vid_just_center
	db	'ZFU error - '
	db	0
;
zfu_share_text			;shareware information screen
	db	12
	vid_just_center
	vid_cursor_off
	db	'ZFU - Z88 File Utility  '
	db	'Copyright 1989,1990 Richard Haw, All Rights Reserved'
	db	13,10,10
	db	'This product is distributed as Shareware, if you use'
	db	' this product you'
	db	13,10
	db	'are required to register it. For registration send $30.00 (US) to'
	db	13,10,10
	db	'Richard Haw, P.O. Box 56, Champaign IL,61824-0056 USA'
	db	10
	vid_just_normal
	db	0
;
zfu_init_data			;startup data
	db	0,0,0,0adh,0d9h,024h	;cre date
	db	0,0,0,0adh,0d9h,024h	;mod date
;
zfu_backup_msg1			;backup screen
	db	12,10
	vid_just_center
	db	'Backup Information',10
	vid_just_normal
	vid_gotoxy	15,3
	db	'Last Full backup'
	vid_gotoxy	15,4
	db	'Last Partial backup',0
zfu_backup_file_name			;backup file name
	db	':ram.0/ZFUBackInfo.cfg',0
zfu_backup_wildstring			;backup wildcard add string
	db	':*//*',0
;
zfu_newarcfile_msg			;message about new file
	db	12,10
	vid_just_center
	db	'Creating new archive file',10
	vid_just_normal
	db	0
;
