;
; ==========================================================================
;
;	ZHyper - Hypertext output routine
;
;	Calling form:
;			call zhyper
;
;	The following must be defined for this program to work:
;
;	zhyper_text - Start of ZHyper text (must be mapped into memory)
;
;	zhyper_buf - Buffer for zhypers use
;
; ==========================================================================
;
;	Dynamic data addresses
;
; ==========================================================================
;
zhyper_offset	equ	zhyper_buf
zhyper_toffset	equ	zhyper_buf+2
zhyper_hoffset	equ	zhyper_buf+4
zhyper_coffset	equ	zhyper_buf+6
zhyper_curtopic	equ	zhyper_buf+8
zhyper_curhelp	equ	zhyper_buf+10
zhyper_numref	equ	zhyper_buf+12
zhyper_curref	equ	zhyper_buf+13
zhyper_expflag	equ	zhyper_buf+14
zhyper_lasttopic	equ	zhyper_buf+16
zhyper_reftable	equ	zhyper_buf+18
;
; ==========================================================================
;
zhyper				;main entry point
;
;	Get all of the address offsets
;
	ld	hl,zhyper_text	;get main offset
	ld	(zhyper_offset),hl	;and save it
	push	hl		;save a couple copies
	push	hl
	ld	e,(hl)		;offset to token table
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)		;offset to help
	inc	hl
	ld	b,(hl)
	inc	hl
	ld	(zhyper_coffset),hl	;offset to custom char table
	pop	hl		;get offset start
	add	hl,de		;compute token offset
	ld	(zhyper_toffset),hl	;and save
	pop	hl		;get start offset again
	add	hl,bc		;compute help offset
	ld	(zhyper_hoffset),hl	;and save
	ld	(zhyper_curtopic),hl	;set starting topic
	ld	(zhyper_lasttopic),hl	;and last topic (same)
	xor	a		;zero out expansion flag
	ld	(zhyper_expflag),a
	call	zhyper_mainwin	;init windows
;
zhyper_main				;main loop point
	ld	hl,(zhyper_curtopic)	;set current topic
	call	zhyper_drawtopic	;and output
;
zhyper_input			;input scan location
	sys_call	os_in		;get a key
	jr	c,zhyper_input	;loop on carry
	call	zhyper_a_to_lower	;make lower case
	and	a		;check for zero and if so
	jr	z,zhyper_input1	;get another byte
	cp	13		;look for enter
	jr	nz,zhyper_noret
zhyper_select
	ld	hl,(zhyper_curtopic)	;save current topic as last
	ld	(zhyper_lasttopic),hl
	ld	a,(zhyper_curref)	;get current reference number
	ld	hl,zhyper_reftable	;get table address
	rlc	a		;multiply by two
	ld	c,a		;and put in bc
	ld	b,0
	add	hl,bc		;adjust address
	ld	e,(hl)		;put address in DE
	inc	hl
	ld	d,(hl)
	ex	de,hl		;and get in hl
	ld	(zhyper_curtopic),hl	;make specified reference current
	jp	zhyper_main		;and make new
zhyper_noret
	cp	'c'		;check for table of contents
	jr	nz,zhyper_notoc
	ld	hl,(zhyper_hoffset)	;get start of text
	ld	(zhyper_curtopic),hl	;and make it the current
	jp	zhyper_main
zhyper_notoc
	cp	'q'		;check for quit
	ret	z
	cp	'b'		;check for back
	jr	nz,zhyper_input	;not a valid character
;
zhyper_selback			;go back to last selection
	ld	hl,(zhyper_lasttopic)	;make current topic last topic
	ld	(zhyper_curtopic),hl	;and update
	jr	zhyper_main
zhyper_input1
	sys_call	os_in		;get byte
	jp	c,zhyper_input	;back to start if preempt
;
	cp	252		;left arrow, go back
	jr	z,zhyper_selback
;
	cp	253		;right arrow, select
	jr	z,zhyper_select
;
	cp	251		;shift up, first ref
	jr	nz,zhyper_nosup
	xor	a		;first ref
	jr	zhyper_changeref_single
;
zhyper_nosup
	cp	250		;shift down, last ref
	jr	nz,zhyper_nosdown
	ld	a,(zhyper_numref)	;last ref
	dec	a
	jr	zhyper_changeref_single
;
zhyper_nosdown
	cp	254		;down arrow
	jr	nz,zhyper_nodown
	ld	a,(zhyper_numref)	;get number of references
	ld	c,a		;and put in c
	ld	a,(zhyper_curref)	;get current reference
	inc	a		;up to match offset
	cp	c		;check for already at end
	jr	z,zhyper_iadone
	dec	a		;adjust offset
	ld	b,a		;save current in b
	inc	a		;up index
	jr	zhyper_changeref	;and output new
zhyper_nodown
	cp	255		;up arrow
	jr	nz,zhyper_noup
	ld	a,(zhyper_curref)	;get current reference
	ld	b,a		;save in b
	and	a		;count down
	jr	z,zhyper_iadone	;at zero, change
	dec	a		;move down
	jr	zhyper_changeref	;update reference
zhyper_noup
zhyper_iadone
	jp	zhyper_input	;back to read input routine
;
zhyper_changeref_single			;change to ref (A)
	push	af		;save af
	ld	a,(zhyper_curref)	;get current reference
	ld	b,a		;and put it in b
	pop	af		;then get a back
zhyper_changeref			;change from ref (B) to (A)
	push	af		;save new
	ld	a,b		;get old
	call	zhyper_drawref_n	;put out normal
	pop	af		;get new
	ld	(zhyper_curref),a	;and save as new ref number
	call	zhyper_drawref_r	;put out reverse
	jr	zhyper_iadone	;and return
	
	
	ret			;and return
;
; ==========================================================================
;
;	Support routines
;
;	zhyper_a_to_lower - Make A lowercase
;
;	zhyper_drawref_n - Draw ref (A) normal
;	zhyper_drawref_r - Draw ref (A) inverse
;
;	zhyper_drawtopic - Make topic at (HL) current and output
;
;	zhyper_getref - Get reference offset at (DE), save adjusted at
;		(HL), returns HL=HL+2, DE=DE+2
;
;	zhyper_gotoref - Select window and position cursor to reference (A)
;
;	zhyper_mainwin - Output main window
;
;	zhyper_outchar - Output character (A) with mapping
;
;	zhyper_redraw - Redraws the screen
;
;	zhyper_sop - Output string at (HL) with expansion
;
; ==========================================================================
;	
zhyper_a_to_lower
	cp	'A'		;check if too low
	ret	c
	cp	'Z'+1		;check if too high
	jr	c,:+1
	ret
:
	or	32		;set bit
	ret
;
zhyper_drawref_n			;draw reference (normal)
	call	zhyper_drawref	;output text
	call	zhyper_drawadj	;cleanup
	ret			;and return
zhyper_drawref_r			;draw reference (reverse)
	call	zhyper_drawref	;output text
	call	zhyper_drawrvid	;reverse video
	call	zhyper_drawadj	;cleanup
	call	zhyper_drawrvid	;set reverse back
	ret			;and return
zhyper_drawadj			;apply toggle over next 12 chars
	ld	hl,zhyper_drawadj_msg
	sys_call	gn_sop		;output text
	ret			;and return
zhyper_drawrvid			;output reverse video toggle
	ld	a,1
	sys_call	os_out
	ld	a,'R'
	sys_call	os_out
	ret
zhyper_drawref			;output reference (A)
	call	zhyper_gotoref	;position cursor
	push	af		;and save again
	ld	hl,zhyper_reftable	;get table address
	rlc	a		;multiply by two
	ld	c,a		;and put in bc
	ld	b,0
	add	hl,bc		;adjust address
	ld	e,(hl)		;put address in DE
	inc	hl
	ld	d,(hl)
	ex	de,hl		;and get in hl
	call	zhyper_sop		;output string
	pop	af		;get reference back
	call	zhyper_gotoref	;and position cursor back
	ret			;and return
;
zhyper_drawtopic			;output new topic
	ld	(zhyper_curtopic),hl	;save address as current topic
	ld	a,0		;set current reference number
	ld	(zhyper_curref),a
zhyper_dtl1
	ld	a,(hl)		;get byte and search till
	and	a		;end of topic
	inc	hl		;next byte
	jr	nz,zhyper_dtl1	;loop
	xor	a		;set current ref
	ld	(zhyper_curref),a
	ld	a,(hl)		;get number of refs
	dec	a		;force num_ref<9
	and	07h		;truncate for safety
	inc	a
	ld	(zhyper_numref),a	;save number
	and	a		;set condition flags
	jr	z,zhyper_dtl3	;and skip reading if so
	ld	b,a		;save copy in b
	inc	hl		;set hl to start of refs
	ld	de,zhyper_reftable	;and de to reference table
zhyper_dtl2
	call	zhyper_getref	;load reference table
	djnz	zhyper_dtl2
zhyper_dtl3
	ld	(zhyper_curhelp),hl	;save help address
	jp	zhyper_redraw	;and jump to redraw routine
;
zhyper_getref			;get reference
	push	bc		;save bc
	ld	c,(hl)		;get offset
	inc	hl
	ld	b,(hl)
	inc	hl
	push	hl		;save hl
	ld	hl,(zhyper_hoffset)	;get help offset
	add	hl,bc		;add them
	ex	de,hl
	ld	(hl),e		;and save in (DE)
	inc	hl
	ld	(hl),d
	inc	hl
	ex	de,hl		;put them back
	pop	hl		;get adjusted HL back
	pop	bc		;get bc back
	ret			;and return
;
zhyper_gotoref			;go to reference (A)
	push	af		;save a
	ld	hl,zhyper_gotoref_prefix
	sys_call	gn_sop
	add	a,32		;adjust A
	sys_call	os_out		;and output
	pop	af		;get a back
	ret			;and return
;
zhyper_mainwin
	ld	hl,zhyper_window	;put out main windows
	sys_call	gn_sop
	ret			;and return
;
;	Output character routine, the character is handled as follows:
;
;	1 - Video control, output as a literal
;	2 - repeat space count
;	3 - Video control, a 1 is output, and the next character is output
;		without translation or token substitution.
;	128-255 - Tokenized text
;
zhyper_outchar			;output character
	push	af		;save character
	ld	a,(zhyper_expflag)	;get expansion flag and check
	and	a		;to see if working on expansion
	jr	z,zhoc_1a		;and skip if so
	cp	1		;check if one (repeat flag)
	jr	nz,zhoc_0
	pop	af		;get repeat count back
	ld	b,a
	ld	a,' '
zhoc_1
	sys_call	os_out
	djnz	zhoc_1
	xor	a		;zero out expansion flag
	ld	(zhyper_expflag),a
	ret			;and return
zhoc_0				;literal character output
	xor	a		;zero out expansion flag
	ld	(zhyper_expflag),a
	pop	af		;get character back
	jp	zhoc_norm		;and output normal
;
zhoc_1a	pop	af		;get character back
	cp	1		;check for video prefix
	jp	z,zhoc_norm		;output and return
	cp	3		;check for literal video output
	jr	nz,zhoc_1b
	ld	a,2		;set flag
	ld	(zhyper_expflag),a
	ld	a,1		;and output a video control (1)
	jp	zhoc_norm
zhoc_1b
;
	cp	2		;repeated space character
	jr	nz,zhoc_2
	ld	a,01h		;set expansion flag for the
	ld	(zhyper_expflag),a	;next call to zhyper_outchar
	ret			;and return
zhoc_2
	cp	7		;check for end of line
	jr	nz,zhoc_3
	sys_call	gn_nln
	ret
zhoc_3
	cp	16		;check for video format
	jr	nc,zhoc_4
	push	af		;save byte
	ld	a,1		;output command byte
	sys_call	os_out
	pop	af
	sub	8		;shift down
	ld	c,a
	ld	b,0
	ld	hl,zhyper_vfmat_table	;get code table
	add	hl,bc		;compute offset
	ld	a,(hl)		;get byte
	sys_call	os_out		;and output
	ret			;and return
zhoc_4
	cp	32		;check for lower line draw
	jr	nc,zhoc_5
	push	af		;save it
	ld	hl,zhyper_ldraw_prefix	;output prefix
	sys_call	gn_sop
	ld	hl,zhyper_ldraw_table	;get table
	pop	af		;get byte back
	sub	16		;fix offset
	ld	c,a		;and put in bc
	ld	b,0
	add	hl,bc		;compute address
	ld	a,(hl)		;get byte
	sys_call	os_out		;and output
	ret			;then return
zhoc_5
	cp	128		;check for token
	jr	c,zhoc_norm		;and skip token output if not
	and	a		;clear carry flag
	rl	a		;multiply A times 2
	ld	c,a		;and put in bc
	ld	b,0
	ld	hl,(zhyper_toffset)	;get offset to token table
	add	hl,bc		;start offset
	push	hl		;save it
	call	zhyper_loadhltoff
	ex	de,hl		;put in de
	pop	hl		;get table address back
	inc	hl		;next offset
	inc	hl
	call	zhyper_loadhltoff
	ex	de,hl		;swap start into hl, end into de
zhoc_tok
	ld	a,l		;check for de=hl
	cp	e
	jr	nz,zhoc_tok1
	ld	a,h
	cp	d
	jr	nz,zhoc_tok1
	ret			;end of tokens
zhoc_tok1
	ld	a,(hl)		;get byte
	push	hl		;save regs
	push	de
	sys_call	os_out		;normal output
;	call	zhoc_outchar	;mapped output
	pop	de		;get regs back
	pop	hl
	inc	hl		;next byte
	jr	zhoc_tok		;loop
;
zhoc_norm
	sys_call	os_out		;normal character output
	ret			;return
;
zhyper_loadhltoff			;load hl with (hl)+token offset
	push	de		;save de
	ld	e,(hl)		;and get (hl) in de
	inc	hl
	ld	d,(hl)
	ld	hl,(zhyper_offset)	;get token offset
	add	hl,de		;and add in offset
	pop	de		;get de back
	ret			;and return
;
zhyper_redraw			;redraw screen
	ld	hl,zhyper_topics_prefix	;select window 1
	sys_call	gn_sop
	ld	hl,(zhyper_curtopic)	;get topic address
	call	zhyper_sop		;output topic name
	ld	hl,zhyper_help_prefix	;select window 2
	sys_call	gn_sop
	ld	hl,(zhyper_curhelp)	;get help address
	call	zhyper_sop		;and output help
;
	ld	a,(zhyper_numref)	;get number of references
	and	a		;test a
	ret	z		;return if no references
	ld	b,a		;and put in b
	xor	a		;zero a
zhyper_rd3				;output all refs
	push	bc		;save b
	call	zhyper_drawref_n	;normal ref
	inc	a		;next ref
	pop	bc		;get bc back
	djnz	zhyper_rd3		;and loop
	ld	a,(zhyper_curref)	;get current reference number
	call	zhyper_drawref_r	;and make reverse video
	ret			;return
;
zhyper_sop				;output a string
	ld	a,(hl)		;get byte
	and	a		;set flags
	ret	z		;return on zero
	push	hl		;save hl
	call	zhyper_outchar	;print it
	pop	hl		;get hl back
	inc	hl		;next byte
	jr	zhyper_sop
;
; ==========================================================================
;
;	Static storage area
;
; ==========================================================================
;
zhyper_window
	vid_win	1,1,0,76,1,1
	vid_win	2,1,1,76,7,1
	vid_win	3,78,0,12,8,1
	vid_win_sel_ic	1
	vid_gotoxy	30,0
	vid_tiny_toggle
	vid_underline_toggle
	vid_reverse_toggle
	db	'ZHYPER V1.00 [C=CONTENTS B=BACK Q=QUIT]'
	vid_gotoxy	0,0
	db	1,50,65,32+76
	vid_win_sel_ic	2
	vid_win_sel_ic	3
	db	0
;
zhyper_topics_prefix			;sel win 1 and move to 0
	vid_win_sel	1
	db	13,'            ',13	;blank out topic
	db	0
zhyper_help_prefix			;init for help text output
	vid_win_sel	3		;clear the reference window
	db	12
	vid_win_sel_ic	2	;back to help window and init
	db	0
zhyper_gotoref_prefix
	vid_win_sel		3	;select window 3
	db	1,'3@',32,0
zhyper_drawadj_msg
	db	1,50,65,32+12,0
;
zhyper_vfmat_table			;formatting table
	db	66,85,84,71,82,70
zhyper_ldraw_prefix
	db	1,50,42,0
zhyper_ldraw_table
	db	69,74,79,67,70,73,76,71,77,75,78