	title	Z-Macro
;
; ==========================================================================
;
;	Z-Macro module
;
;	Takes as input a single keystroke, then looks in the file
;	:ram.0/zmacro.mac for the set of data identified by .# followed
;	by the character hit (case is NOT significant), then copies that
;	data to a .CLI file and executes it.
;
; ==========================================================================
;
zm_main
	ld	hl,0		;reset all data
	ld	(zm_mfhand),hl	;macro file handle
	ld	(zm_tfhand),hl	;temporary file
	add	hl,sp		;get stack location
	ld	(zm_stack),hl	;and save it
;
zm_keyget				;main keystroke get point
	sys_call	os_in		;get command
	jp	c,zm_done		;exit on anything but a keystroke
	call	a_to_lower		;conver to lower case
	ld	(zm_key),a		;save keystroke
	call	zm_opnmfile		;open macro file
;
;	Loop through file looking for the macro corresponding
;	to the key hit.
;
:
	call	zm_getbyte		;get a byte
	cp	'.'		;check for period
	jr	nz,zm_skipline
	call	zm_getbyte		;get next byte
	cp	'#'		;check for '#'
	jr	nz,zm_skipline
	call	zm_getbyte		;get keystroke
	call	a_to_lower		;make lower case
	ld	b,a		;save it
	ld	a,(zm_key)		;get key
	cp	b		;and compare
	jr	z,zm_keyfound
zm_skipline
	call	zm_skipcr		;skip this line
	jr	:-1
;
zm_keyfound				;found a key
	call	zm_skipcr		;skip this line
	call	zm_opncfile		;open cli file
	ld	a,'Z'		;output initial char to .CLI file
	call	zm_cliout
	ld	a,13
	call	zm_cliout
:
	call	zm_getbyte		;get a character
	cp	'.'		;check if a new block
	jr	z,:+2
	call	zm_getbyte1a	;make sure not the start of a
	call	zm_cliout		;valid command
;
;	Start of main text processing, basically anything in the
;	middle of a line, or at the start of a line which does not
;	start with a period.
;
:
	call	zm_getbyte1		;get another character
	call	zm_cliout
	cp	13		;is an EOF
	jr	z,:-2		;end of line
	jr	:-1		;part of line
:
	call	zm_getbyte		;get second character in line
	cp	'#'		;check for '#'
	jr	z,:+1		;end of this macro
	push	af		;save second character
	ld	a,'.'		;output a dot
	call	zm_cliout
	pop	af		;and the read character
	call	zm_cliout
	jr	:-2		;midline processing
:
	call	zm_cleanup		;close files
;
;	Now make a call to execute the .CLI file, command must be copied
;	into bank 0 since the cli command remaps bank three.
;
	ld	bc,000fh		;set length
	ld	hl,zm_execcli
	ld	de,zm_scratch
	push	de		;save scratch address
	ldir			;copy string
	pop	hl		;get name address
;
	ld	bc,000fh		;length
	sys_call	dc_icl
;
;	Now make a call to a system input routine to start .CLI
;	execution.
;
	ld	bc,1		;set for zero wait
	sys_call	os_tin		;call get standard input
;
;	End of processing
;
	ret
;
zm_cliout				;output to CLI file
	ld	ix,(zm_tfhand)	;get handle
	sys_call	os_pb		;put byte
	ret	nc		;no error
	ld	hl,zm_errmsg4
	jp	zm_error
;
zm_skipcr				;skip to CR
	call	zm_getbyte		;get a byte
	cp	13		;check for CR
	jr	nz,zm_skipcr	;loop if not a CR
	ret
;
;	This routine is the read character routine for data which may
;	contain imbedded commands.
;
;	All imbedded commands start with the reserved character { and
;	continue till the next } character.
;
zm_getbyte1
	call	zm_getbyte		;get the byte
zm_getbyte1a
	cp	'{'		;check for start of command
	ret	nz		;return if not
	call	zm_getbyte		;get the next byte
	cp	'{'		;check for repeated character
	ret	z		;just return single {
	ld	hl,zm_scratch	;set buffer for data
	ld	(hl),a		;save first byte
	inc	hl		;next byte
	ld	b,2fh		;set buffer length
:
	push	hl		;save registers
	push	bc
	call	zm_getbyte		;get a character
	cp	'}'		;check for end of data
	jr	z,:+2		;all done here
	pop	bc		;get registers back
	pop	hl
	dec	b		;count this
	jr	nz,:+1		;no overflow
	ld	hl,zm_errmsg5
	jp	zm_error		;jump to error routine
:
	ld	(hl),a		;save character
	inc	hl		;next address
	jr	:-2		;and loop for more
:
	pop	hl		;clear stack
	pop	hl
	xor	a		;put zero terminator
	ld	(hl),a		;in buffer
;
;	Done with command, now process according to what is in the
;	buffer, valid commands are as follows:
;
;	Dn - Current Date
;	Tn - Current Time
;
	ld	a,(zm_scratch)	;get starting byte
	call	a_to_lower		;make lower
	cp	'd'		;date command
	jp	z,zm_macdate
	cp	't'		;time command
	jp	z,zm_mactime
	cp	'?'		;input prompt
	jp	z,zm_macquestion
	jp	zm_macdone		;jump to done
;
zm_macdone
	call	zm_getbyte		;ignore command by getting next
	ret			;byte and returning
;
zm_macdate
	ld	de,zm_scratch1	;set buffer address
	sys_call	gn_gmd		;get machine date
	ld	hl,zm_scratch+1	;set buffer address
	call	hexmem_to_a		;and convert from hex
	push	af
	call	hexmem_to_a
	push	af
	call	hexmem_to_a
	ld	c,a		;save third byte
	pop	af
	ld	b,a		;save second byte
	pop	af		;get third byte
	ld	hl,zm_scratch1	;date buffer
	ld	de,zm_scratch	;buffer for date
	push	de		;save buffer address
	sys_call	gn_pdt		;convert date
	xor	a		;put trailing zero
	ex	de,hl
	ld	(hl),a
	pop	hl		;get buffer address back
:
	ld	a,(hl)		;get a byte
	cp	0		;check for end of buffer and
	jp	z,zm_macdone		;return if so
	push	hl		;save hl
	call	zm_cliout		;output character
	pop	hl
	inc	hl
	jr	:-1		;and loop for more
;
zm_mactime
	ld	de,zm_scratch1	;set buffer address
	sys_call	gn_gmt		;get machine time
	ld	hl,zm_scratch+1	;set buffer address
	call	hexmem_to_a		;and convert from hex
	and	0f7h		;ensure bit 3 reset
	ld	hl,zm_scratch1	;time buffer
	ld	de,zm_scratch	;buffer for time
	push	de		;save buffer address
	sys_call	gn_ptm		;convert time
	xor	a		;put trailing zero
	ex	de,hl
	ld	(hl),a
	pop	hl		;get buffer address back
:
	ld	a,(hl)		;get a byte
	cp	0		;check for end of buffer and
	jp	z,zm_macdone		;return if so
	push	hl		;save hl
	call	zm_cliout		;output character
	pop	hl
	inc	hl
	jr	:-1		;and loop for more
	jp	zm_macdone		;return
;
zm_macquestion			;interactive response
	ld	hl,zm_maciwin	;create prompt window
	sys_call	gn_sop
	ld	hl,zm_scratch+1	;output prompt
	sys_call	gn_sop
	ld	hl,zm_maciwin1	;and other stuff
	sys_call	gn_sop
:
	ld	a,00h		;set mode
	ld	b,028h		;maximum buffer length
	ld	de,zm_scratch1
	sys_call	gn_sip		;read in a line
;
	ld	hl,zm_scratch1
:
	ld	a,(hl)		;get a byte
	cp	0		;check for end of buffer and
	jp	z,zm_macdone		;return if so
	push	hl		;save hl
	call	zm_cliout		;output character
	pop	hl
	inc	hl
	jr	:-1		;and loop for more
;
zm_getbyte
	ld	ix,(zm_mfhand)	;get file handle
	sys_call	os_gb		;get a byte
	ret	nc		;no error
	cp	rc_eof		;check for EOF
	jp	z,zm_eof		;end of file
	jr	zm_getbyte		;pre-empt codes
;
zm_done				;exit point
	ld	hl,(zm_stack)	;get old stack
	ld	sp,hl		;and make current
	call	zm_cleanup		;cleanup all files
	call	reset_win		;reset screen
	ret			;and return
;
zm_cleanup				;exit cleanup
	ld	hl,zm_mfhand	;close files
	call	close_file
	ld	hl,zm_tfhand
	call	close_file
	ret
;
zm_eof				;end of file reached
	ld	hl,zm_errmsg2	;second error message
	jp	zm_error
;
zm_error				;error message output
	push	hl		;save text
	call	zm_mainwin		;output main window
	pop	hl
	sys_call	gn_sop		;output message text
	call	pause		;pause
	jp	zm_done		;and exit
;
zm_mainwin				;draw out main window
	ld	hl,zm_title		;put out error message
	call	title_win_full	;and output error message
	ret
;
zm_opnmfile				;open macro file
	ld	a,01h		;input mode
	ld	b,0		;zero name offset
	ld	c,20h		;scratch length
	ld	de,zm_scratch	;scratch location
	ld	hl,zm_macrofile	;file name
	sys_call	gn_opf		;open file
	jr	nc,:+1		;no error
	ld	hl,zm_errmsg1
	jp	zm_error
:
	ld	(zm_mfhand),ix	;save handle
	ret			;and return
;
zm_opncfile				;open macro file
	ld	a,02h		;input mode
	ld	b,0		;zero name offset
	ld	c,20h		;scratch length
	ld	de,zm_scratch	;scratch location
	ld	hl,zm_clifile	;file name
	sys_call	gn_opf		;open file
	jr	nc,:+1		;no error
	ld	hl,zm_errmsg3
	jp	zm_error
:
	ld	(zm_tfhand),ix	;save handle
	ret			;and return
;
; ==========================================================================
;
;	Static Data Area
;
; ==========================================================================
;
zm_macrofile			;macro file name
	db	':RAM.0/ZMACRO.MAC',0
zm_execcli				;exec cli file command
	db	'.*'
zm_clifile				;cli file name
	db	':RAM.0/E.CLI'
	db	0
zm_title				;window title
	db	'Z-MACRO v1.00',0
zm_maciwin				;input window
	vid_win_1	1,10,2,60,4
	db	0
zm_maciwin1				;other input text
	vid_win_2	1,10,2,60,4
	vid_just_normal
	db	12,10,'   >'
	db	0
zm_errmsg1				;macro file not found
	vid_just_center
	vid_cursor_off
	db	12,10,10,10
	db	'Z-Macro file :RAM.0/ZMACRO.MAC not found',10
	db	0
zm_errmsg2				;macro file not found
	vid_just_center
	vid_cursor_off
	db	12,10,10,10
	db	'End of file reached on :RAM.0/ZMACRO.MAC',10
	db	0
zm_errmsg3				;macro file not found
	vid_just_center
	vid_cursor_off
	db	12,10,10,10
	db	'Error opening temporary file',10
	db	0
zm_errmsg4				;macro file not found
	vid_just_center
	vid_cursor_off
	db	12,10,10,10
	db	'Error writing temporary file',10
	db	0
zm_errmsg5				;macro file not found
	vid_just_center
	vid_cursor_off
	db	12,10,10,10
	db	'Z-Macro command too long',10
	db	0
;
; ==========================================================================
;
;	Dynamic Data Area
;
; ==========================================================================
;
	IFDEF	BASIC
zm_dstart	equ	$
	ENDIF
;
	IFDEF	ROM
zm_dstart	equ	(01ffdh-060h)	;allocated down from 01ff
	ENDIF
;
zm_key	equ	zm_dstart		;keystroke
zm_mfhand	equ	zm_dstart+1		;macro file handle
zm_tfhand	equ	zm_dstart+3		;temporary .CLI file handle
zm_stack	equ	zm_dstart+5		;saved stack
zm_scratch	equ	zm_dstart+7		;scratch buffer
zm_scratch1	equ	zm_dstart+037h
;