	title	Z88 Menu Master
;
; ==========================================================================
;
;	Z88 Menu Program
;
; ==========================================================================
;
;	Version 1.00 written November 1990 by Richard Haw under
;	contract from MAC Systems Inc.
;
;	This program is a simple BASIC runnable program which 
;	presents the user with a menu which can be used to execute
;	other programs.
;
;	The program is self-modifying, allowing the editing of the
;	user menus. The program name is stored internally, allowing
;	the creation of multiple menu programs, each callable from the
;	main program. (The only limitation here is that the called
;	programs link back to the main program, and so you end up at
;	the main level at all times, which can't be avoided without
;	a further modification of the main program each time, and the
;	file open and close overhead is at this point considered
;	prohibative).
;
; ==========================================================================
;
;	The menu consists of 30 fixed length records in
;	memory, which are initially empty.
;
;	Record format:
;
;	Byte	Contents
;	0	Hot-key (A-Z,0-9, NULL if none)
;	1	Type byte
;		0 - Basic program
;		1 - Chained CLI file
;	2-11	Short name (displayed in menu)
;	12-63	Long description (displayed at bottom of screen)
;
; ==========================================================================
;
zmm_entry
	ld	hl,0		;save the stack state
	add	hl,sp
	ld	(zmm_stack),hl
	ld	hl,0		;zero out handles
	ld	(zmm_file_handle),hl
	xor	a		;zero counters
	ld	(zmm_col),a
	ld	(zmm_row),a
;
	z88_name	'ZMM v1.00'		;set program name
	ld	hl,zmm_mainwin	;print out window header
	sys_call	gn_sop
;
; ==========================================================================
;
;	Main menu
;
; ==========================================================================
;
zmm_main
	ld	hl,zmm_menumsg	;put out menu screen
	sys_call	gn_sop
:
	sys_call	os_in		;get a key
	jr	c,:-1
	call	a_to_lower		;make lower case
	cp	'b'		;burn ?
	jp	z,zmm_burnit
	cp	'd'		;Dump ?
	jp	z,zmm_dump
	cp	'v'		;Verify ?
	jp	z,zmm_verify
	cp	'q'		;Quit ?
	jp	z,zmm_alldone
	jr	:-1
;
; ==========================================================================
;
;	Burn a file into EPROM
;
; ==========================================================================
;
zmm_burnit
	xor	a		;zero verify error flag
	ld	a,(zmm_epberr)
	ld	hl,zmm_burnmsg	;put out window prompt
	sys_call	gn_sop
	ld	a,20h		;single line lock
	ld	b,080h		;buffer length
	ld	l,020h		;window input length
	ld	de,zmm_file_name	;file name buffer
	sys_call	gn_sip		;read input line
	jp	c,zmm_main		;any problems (inc. preempt)
	ld	a,01h		;output mode
	ld	bc,0080h		;local and buffer size
	ld	de,zmm_buf		;scratch space
	ld	hl,zmm_file_name	;file name
	sys_call	gn_opf		;and open the file
	jp	c,zmm_main		;any problems
	ld	(zmm_file_handle),ix	;save handle
	ld	hl,zmm_burnmsg_1	;output 'working' message
	sys_call	gn_sop
	ld	a,02		;get the extent of the file
	ld	de,0		;in DEBC
	sys_call	os_frm
	ld	a,e		;check for 128
	cp	02h
	jr	nz,:+1
	ld	a,069h		;mask for 128K
	jr	:+3
:
	ld	a,b		;check for 32K
	cp	080h
	jr	nz,:+1
	ld	a,048h		;mask for 32K
	jr	:+2
:
	ld	a,048h		;mask (default)
:
	ld	(zmm_eprom_mask),a	;save mask for later
;
	rl	b		;take byte 3 of bytes 2&3 of
	rl	e		;the file size * 4 (to return
	rl	b		;the number of banks).
	rl	e
	ld	a,e
	ld	(zmm_num_bank),a	;save the number of banks
	ld	b,a		;and put in b
	cpl	a		;get starting bank
	inc	a
	ld	c,a		;and save in c
epbb_bank_loop
	push	bc		;save counter and bank
	ld	b,c		;get bank counter
	ld	c,3		;segment number
	sys_call	os_mpb		;map bank
	push	bc		;save old mapping
	ld	hl,00h		;move block from file to memory
	ld	de,04000h		;(full segment 1)
	ld	bc,04000h		;block length (full bank)
	sys_call	os_mv		;move it
	jr	c,epbb_badread	;error on write
	ld	hl,04000h		;source for write
	ld	de,0c000h		;destination
	call	zmm_epbwrite	;burn 'em
	pop	bc		;get mapping back
	sys_call	os_mpb		;remap
	pop	bc		;get counter back
	inc	c		;next bank
	djnz	epbb_bank_loop	;and loop for all banks
	ld	hl,zmm_file_handle	;close file
	call	close_file
	ld	a,(zmm_epberr)	;get error flag
	and	a		;set condition flags
	jr	z,zmm_burn_ok	;and set address of the
	ld	hl,zmm_burnmsg_bad	;appropriate info message
	jr	zmm_burn_bad
zmm_burn_ok
	ld	hl,zmm_burnmsg_ok
zmm_burn_bad
	sys_call	gn_sop		;output closing message
	pause
	jp	zmm_main		;and back to main menu
;
epbb_badread			;error during read, cleanup
	pop	bc		;get old mapping
	sys_call	os_mpb		;and restore it
	pop	bc		;clean up stack
	ld	hl,zmm_file_handle	;close file
	call	close_file
	ld	hl,epbb_badread_msg	;and output error message
	sys_call	gn_sop
	pause
	jp	zmm_main
;
; ==========================================================================
;
;	Dump an EPROM to a file (binary)
;
; ==========================================================================
;
zmm_dump
	call	zmm_chk_eprom	;check for a valid eprom
	jp	c,zmm_main		;back to main on error
;
;	EPROM looks good, get a file and dump the EPROM.
;
epbd_doit
	ld	hl,zmm_dumpmsg	;put out window prompt
	sys_call	gn_sop
	ld	a,20h		;single line lock
	ld	b,080h		;buffer length
	ld	l,020h		;window input length
	ld	de,zmm_file_name	;file name buffer
	sys_call	gn_sip		;read input line
	jp	c,zmm_main		;any problems (inc. preempt)
	ld	a,02h		;output mode
	ld	bc,0080h		;local and buffer size
	ld	de,zmm_buf		;scratch space
	ld	hl,zmm_file_name	;file name
	sys_call	gn_opf		;and open the file
	jp	c,zmm_main		;any problems
	ld	(zmm_file_handle),ix	;save handle
	ld	hl,zmm_dumpmsg_1	;output 'working' message
	sys_call	gn_sop
;
	ld	a,(zmm_num_bank)	;get number of banks
	ld	b,a		;and put in b
	cpl	a		;complement and then increment
	inc	a		;to get starting bank
	ld	c,a		;bank number is in C
epbd_bank_loop
	push	bc		;save counter and bank
	ld	b,c		;get bank counter
	ld	c,3		;segment number
	sys_call	os_mpb		;map bank
	push	bc		;save old mapping
	ld	hl,0c000h		;mapped address
	ld	de,0		;move it
	ld	bc,04000h		;block length (full bank)
	sys_call	os_mv		;move it
	jr	c,epbd_badwrite	;error on write
	pop	bc		;get mapping back
	sys_call	os_mpb		;remap
	pop	bc		;get counter back
	inc	c		;next bank
	djnz	epbd_bank_loop	;and loop for all banks
	ld	hl,zmm_file_handle	;close file
	call	close_file
	jp	zmm_main		;and back to main menu
;
epbd_badwrite			;error during write, cleanup
	pop	bc		;get old mapping
	sys_call	os_mpb		;and restore it
	pop	bc		;clean up stack
	ld	hl,zmm_file_handle	;close file
	call	close_file
	ld	hl,epbd_badwrite_msg	;and output error message
	sys_call	gn_sop
	pause
	jp	zmm_main
;
; ==========================================================================
;
;	Verify EPROM erase
;
; ==========================================================================
;
zmm_verify
	ld	hl,zmm_epvwin	;output verify message
	sys_call	gn_sop
	ld	b,0c0h		;start bank (full addr space)
	ld	c,02		;segment to map to
epbv_bankloop
	push	bc		;save registers
	sys_call	os_mpb		;map bank in
	push	bc		;save mapping
	ld	hl,08000h		;set start address
epbv_byteloop
	ld	a,(hl)		;get byte
	cp	0ffh		;check for valid
	jr	nz,epbv_bad		;no good
	inc	hl		;next address
	ld	a,h		;get high byte
	cp	0c0h		;and check for 'overflow'
	jr	nz,epbv_byteloop
	pop	bc		;get mapping and remap
	sys_call	os_mpb
	pop	bc		;get bank map
	inc	b		;next bank
	ld	a,b
	and	a		;and continue till all
	jr	nz,epbv_bankloop	;banks are done
	jr	epbv_ok		;it's zero
epbv_bad
	pop	bc		;get mapping and remap
	sys_call	os_mpb
	pop	bc		;clean stack
	ld	hl,zmm_epv_badmsg
epbv_bad_1
	sys_call	gn_sop
	pause
	jp	zmm_main		;and return to main menu
epbv_ok
	ld	hl,zmm_epv_okmsg	;put out OK message
	jr	epbv_bad_1
;
; ==========================================================================
;
;	Exit the program
;
; ==========================================================================
;
zmm_alldone
	ld	hl,(zmm_stack)	;restore stack (just to be safe)
	ld	sp,hl
	ret			;and return to BASIC handler
;
; ==========================================================================
;
;	Support routines
;
; ==========================================================================
;
;	zmm_epbwrite - Write a block of bytes into the EPROM
;
;	Entry:
;		HL - points to source buffer
;		DE - points to EPROM address (must be mapped to seg 3)
;		zmm_block_size - defined as block length to burn
;		(zmm_eprom_mask) - set as appropriate for size of EPROM
;
;	Exit:
;		(zmm_epberr) set to 0ffh if burn verify error
;
; ==========================================================================
;
;	zmm_chk_eprom - Check to make sure its a Z88 EPROM
;
;	Exit:
;		Carry set and message output if the EPROM is not a
;		recognizable Z88 EPROM.
;
;		If a Z88 EPROM then CF=0 and (zmm_num_bank) holds the
;		number of banks in the EPROM.
;
; ==========================================================================
;
zmm_epbwrite			;write out block of bytes
	xor	a		;zero c
	ld	c,a
	ld	b,75		;number of trys for EPROM
	ld	a,(zmm_eprom_mask)	;set eprom signal
	out	(0b3h),a
:
	ld	a,0eh		;set Vpp and program bits
	out	(0b0h),a
	call	zmm_epbw1		;write block
	call	zmm_epbvfy		;verify
	jr	z,:+1		;block OK
	djnz	:-1		;loop till max count
:
	ld	a,04h		;reset Vpp and program bits
	out	(0b0h),a		;(but not screen)
	ld	a,76		;now go back and reburn the
	sub	b		;the same number of times
	ld	b,a
:
	ld	a,02eh		;set Vpp and program bits
	out	(0b0h),a
	call	zmm_epbw1
	ld	a,04h		;reset Vpp and program bits
	out	(0b0h),a		;(but not screen)
	djnz	:-1		;loop till done
	call	zmm_epbvfy		;verify final write
	jr	z,:+1
	ld	a,0ffh		;write error, set flag
	ld	(zmm_epberr),a
:
	ld	a,05h		;turn screen back on
	out	(0b0h),a
	ret			;and return
;
zmm_epbw1				;copy block of data
	push3			;save registers
	ld	bc,zmm_block_size	;set block length
	ldir			;copy bytes
	pop3			;get registers back
	ret			;and return
;
zmm_epbvfy				;verify data (NZ if fails)
	push3			;save registers
	ld	bc,zmm_block_size	;set block size
:
	ld	a,(de)		;get data byte
	cp	(hl)		;compare to ROM
	jr	nz,:+1		;is bad
	inc	de		;next byte
	inc	hl
	dec	bc		;count this byte
	ld	a,c		;and loop till done
	and	a
	jr	nz,:-1
	ld	a,b
	and	a
	jr	nz,:-1
:
	pop3			;get registers back
	ret			;and return
;
; ==========================================================================
;
zmm_chk_eprom
	ld	bc,0ff03h		;bind top bank to segment 3
	sys_call	os_mpb
	push	bc		;save old mapping
	ld	a,(0fffeh)		;get type byte 1
	cp	'o'
	jr	z,zmm_chk_eprom_1
	cp	'O'
	jr	z,zmm_chk_eprom_1
	jr	zmm_bad_eprom
zmm_chk_eprom_1
	ld	a,(0ffffh)
	cp	'z'
	jr	z,zmm_chk_eprom_2
	cp	'Z'
	jr	z,zmm_chk_eprom_2
	jr	zmm_bad_eprom
zmm_chk_eprom_2
;
;	Eprom seems to be a Z88 EPROM
;
	ld	a,(0fffch)		;get number of banks
	ld	(zmm_num_bank),a	;and save it for later
	pop	bc		;get old mapping back
	sys_call	os_mpb		;and restore it
	ret			;and return
;
;	Bad header on the EPROM, doesn't seem to be anything known,
;	(must have 'OZ' for application or 'oz' for file system at
;	FF:3FFE, so put out a message and go back to the main menu.
;
zmm_bad_eprom
	pop	bc		;get mapping back
	sys_call	os_mpb		;and restore
	ld	hl,zmm_badmsg	;load message
	sys_call	gn_sop
	pause
	scf			;set carry flag
	ret			;and return
