;
; ==========================================================================
;
;	Z88 Outline program (ZOP)
;
; ==========================================================================
;
zop_main				;main entry point
	ld	hl,0		;reset all data
	add	hl,sp		;get stack location
	ld	(zop_stack),hl	;and save it
;
	ld	hl,zop_pname	;set program name
	sys_call	dc_nam
;
;	Put out initial screen message
;
	ld	hl,zop_mainwin	;put out main windows
	sys_call	gn_sop
	ld	hl,zop_share_text
	sys_call	gn_sop
	jp	zop_file_menu
;
; ==========================================================================
;
;	Main loop
;
; ==========================================================================
;
zop_main_loop
	call	zop_menu_clear
zop_main_loop_1
	sys_call	os_in
	jr	c,zop_main_loop_1
	call	a_to_lower
;
;	Hot keys
;
	cp	cntrl_x		;exit
	jp	z,zop_done
	cp	cntrl_e		;edit current topic
	jp	z,zop_editcur
	cp	cntrl_o		;main menu
	jp	z,zop_sel_menu
	jp	zop_main_loop_1
;
; ==========================================================================
;
;	Main selection menu
;
; ==========================================================================
;
zop_sel_menu
	ld	hl,zop_sel_menu_msg
	call	zop_menu_out
zop_smenu_1
	sys_call	os_in		;get character
	jr	c,zop_smenu_1	;loop if need be
	call	a_to_lower		;make lower case
	cp	'f'		;file menu
	jp	z,zop_file_menu
	cp	'x'		;back to main
	jp	z,zop_main_loop
	jr	zop_smenu_1
;
; ==========================================================================
;
;	File menu
;
; ==========================================================================
;
zop_file_menu
	ld	hl,zop_file_menu_msg
	call	zop_menu_out
zop_fmenu_1
	sys_call	os_in		;get character
	jr	c,zop_fmenu_1	;loop if need be
	call	a_to_lower		;make lower case
	cp	'n'		;new file
	jp	z,zop_file_new
	cp	'o'		;open file
	jp	z,zop_file_open
	cp	's'		;save file
	jp	z,zop_file_save
	cp	'p'		;print file
	jp	z,zop_file_print
	cp	'x'		;previous menu
	jp	z,zop_sel_menu
	jr	zop_fmenu_1
;
zop_file_new			;new file
	jp	zop_main_loop
;
zop_file_open			;open file
	jp	zop_main_loop
;
zop_file_save			;save file
	jp	zop_main_loop
;
zop_file_print			;print file
	jp	zop_main_loop
;
; ==========================================================================
;
;	Edit current topic
;
; ==========================================================================
;
zop_editcur
	clear_screen
	ld	bc,0101h		;set cursor position
	call	zop_readbuf		;read line into buffer
	jp	zop_main_loop	;go back to loop
;
; ==========================================================================
;
;	Utility and support routines
;
; ==========================================================================
;
zop_output_menu			;output menu text in (HL)
	push	hl		;save pointer
	ld	hl,zop_menu_text_pre	;output prefix
	sys_call	gn_sop
	pop	hl		;get text
	sys_call	gn_sop
	ld	hl,zop_menu_text_suf	;and output suffix
	sys_call	gn_sop
	ret			;return
;
zop_cleanup				;cleanup open files/wildcards
	ret
;
zop_done				;exit point
	ld	hl,(zop_stack)	;get old stack
	ld	sp,hl		;and make current
	call	zop_cleanup		;cleanup all files
	ret			;and return
;
zop_main_error			;main error handler
	ld	sp,(zop_stack)	;restore the stack
	jp	zop_main_loop
;
zop_readbuf				;read data into zop_buf
	push	bc		;save location
	call	zop_cursor_on	;turn cursor on
	pop	bc		;get bc back
	push	bc
	call	gotoxy		;position cursor
	ld	de,zop_buf		;set address
	ld	a,021h		;set mode
	ld	b,240		;buffer length
	ld	l,74		;locked line length
	sys_call	gn_sip		;call routine
	jr	c,zop_readbuf1	;error occured
	pop	bc		;clean up stack
	call	zop_cursor_off	;turn cursor off
	ret			;and return
zop_readbuf1
	pop	bc		;get cursor loc
	push	bc		;and save again
	call	gotoxy		;position cursor
	jr	zop_readbuf		;redraw
;
zop_cursor_on			;turn cursor on
	ld	hl,zop_curson_msg
	sys_call	gn_sop
	ret
zop_cursor_off			;turn cursor off
	ld	hl,zop_cursoff_msg
	sys_call	gn_sop
	ret
;
zop_menu_out			;output menu text
	push	hl		;save address
	ld	hl,zop_menu_text_pre	;init it
	sys_call	gn_sop
	pop	hl		;get data
	sys_call	gn_sop
	ld	hl,zop_menu_text_suf	;back to main win
	sys_call	gn_sop
	ret			;and return to caller
;
zop_menu_clear			;output menu text
	ld	hl,zop_menu_text_pre	;init it
	sys_call	gn_sop
	ld	hl,zop_menu_text_suf	;back to main win
	sys_call	gn_sop
	ret			;and return to caller
;
; ==========================================================================
;
;	Static storage area
;
; ==========================================================================
;
zop_mainwin
	vid_win		3,79,0,11,8,1
	vid_win		1,0,0,78,8,1
	db	0
;
zop_menu_text_pre			;prefix for menu output
	vid_win_sel	3		;select window 3
	db	12		;and clear it
	db	0
zop_menu_text_suf			;suffix for menu output
	vid_win_sel	1
	db	0
;
zop_pname	db	"ZOP v0.90"
	db	0
;
zop_share_text			;shareware information screen
	db	12
	vid_just_center
	vid_cursor_off
	db	'ZOP - Z88 Outline Program  '
	db	'Copyright 1990 Richard Haw, All Rights Reserved'
	db	13,10,10
	db	'This product is distributed as Shareware, if you use'
	db	' this product you'
	db	13,10
	db	'are required to register it. For registration send $30.00 (US) to'
	db	13,10,10
	db	'Richard Haw, P.O. Box 56, Champaign IL,61824-0056 USA'
	db	10,10
	db	'Type ','H for Help'
	vid_just_normal
	db	0
;
zop_curson_msg
	vid_cursor_on
	db	0
zop_cursoff_msg
	vid_cursor_off
	db	0
;
zop_sel_menu_msg			;main menu
	db	1,'U',1,'T',' MAIN MENU ',1,'U',1,'T'
	db	1,'B','F',1,'B','ile',13,10
	db	1,'B','E',1,'B','dit',13,10
	db	1,'B','S',1,'B','earch',13,10
	db	1,'B','O',1,'B','ptions',13,10
	db	1,'B','T',1,'B','ools',13,10
	db	10
	db	'e',1,'B','X',1,'B','it menu'
	db	0
;
zop_file_menu_msg			;file menu
	db	1,'U',1,'T','   FILES   ',1,'U',1,'T'
	db	1,'B','N',1,'B','ew',13,10
	db	1,'B','O',1,'B','pen',13,10
	db	1,'B','S',1,'B','ave',13,10
	db	1,'B','P',1,'B','rint',13,10
	db	10,10
	db	'e',1,'B','X',1,'B','it menu'
	db	0
;
; ==========================================================================
;
;	Dynamic storage area
;
; ==========================================================================
;
	IFDEF	BASIC
zop_dstart	equ	$
	bsz	0b50h
	ENDIF
;
	IFDEF	ROM
zop_dstart	equ	02100h
	ENDIF
;
zop_stack	equ	zop_dstart		;stack save pointer
zop_fhand	equ	zop_stack+2		;file handle
zop_deffmt	equ	zop_fhand+2		;default formats tables
zop_defattr	equ	zop_deffmt+16	;default attributes
zop_mh_h	equ	zop_defattr+16	;handle for memory handle tree
zop_mh_ptr	equ	zop_mh_h+2		;ptr to first memory handel ele.
zop_mt_ptr	equ	zop_mh_ptr+3	;ptr to first template
zop_mo_ptr	equ	zop_mt_ptr+3	;ptr to first outline alloc.
zop_buf	equ	zop_mo_ptr+3	;buffer
zop_eod	equ	zop_buf+100h	;end of data
;
