;
; ==========================================================================
;
;	Graphics support routines
;
; ==========================================================================
;
;	graph_create - create a graphics window
;	graph_delete - remove the graphics window
;	graph_erase - erase the screen
;		hl points to 32 byte scratch buffer on entry
;	graph_draw - Draw graph, starting
;		(hl) start of display data
;		(de) width of buffer (32 for one screen)
;	graph_line - output line at (hl) to screen (c)
;
; ==========================================================================
;
graph_create
	ld	bc,mp_gra		;set to define a map
	ld	a,'6'		;set as window 6
	ld	hl,0ffh		;make 255 wide
	sys_call	os_map		;do it
	ret
;
graph_delete
	ld	a,'6'		;window 6
	ld	bc,mp_del		;set for delete
	sys_call	os_map		;do it
	ret
;
graph_erase
	ld	b,32		;and count
	xor	a		;zero a
	push	hl		;save buffer addr
graph_erase_1
	ld	(hl),a		;zero byte
	inc	hl		;next address
	djnz	graph_erase_1	;loop till done
	pop	hl		;get buf addr
	ld	b,64		;set row count
	ld	e,0		;start at row 0
graph_erase_2
	call	graph_outline	;draw out line
	inc	e		;next row
	djnz	graph_erase_2	;loop till done
	ret			;all done, return
;
graph_draw				;draw out an image
	ld	c,0		;row pointer
	ld	b,64		;row counter
graph_draw_1
	push	hl		;save buffer
	push	de		;and increment
	push	bc		;and counters
	ld	e,c		;set row
	call	graph_outline	;and draw line
	pop	bc		;get back all
	pop	de
	pop	hl
	add	hl,de		;adjust offset
	inc	c		;next row
	djnz	graph_draw_1	;loop till done
	ret			;and return
;
graph_outline			;output line e at (hl)
	pushall			;save all registers
	ld	a,'6'		;window 6
	ld	bc,mp_wr
	ld	d,0		;zero d (e should be row)
	sys_call	os_map
	popall			;get registers back
	ret			;return