	title	Z88 RLE Display
;
; ==========================================================================
;
;	This module performs graphics display of RLE images
;
; ==========================================================================
;
zrle_entry
	ld	hl,0		;reset all data
	add	hl,sp		;get stack location
	ld	(zrle_stack),hl	;and save it
	ld	hl,zrle_pname	;set program name
	sys_call	dc_nam
	ld	hl,0		;zero out all handles
	ld	(zrle_file_handle),hl
	xor	a		;and graph flag
	ld	(zrle_graph_flag),a
;
;	Get file name and read in the .RLE file
;
zrle_get_file
	call	reset_win
	ld	hl,zrle_gfile_msg	;prompt to use
	ld	de,zrle_scratch	;buffer to use
	ld	c,080h		;and length
	call	get_file_name	;and get the file name
	jp	c,zrle_alldone	;exit on error
	ld	a,(zrle_scratch)	;check for null entry
	and	a
	jp	z,zrle_alldone
	ld	a,01h		;open for input
	ld	b,0		;local name
	ld	c,80h		;scratch length
	ld	hl,zrle_scratch	;file name
	ld	de,zrle_file_name	;fully qualified name
	sys_call	gn_opf		;and open the file
	jp	c,zrle_bad_file
	ld	(zrle_file_handle),ix	;save handle
	call	reset_win		;clear up display
	ld	hl,zrle_main_win_msg	;set up main window
	sys_call	gn_sop
	ld	hl,zrle_reading_msg	;put out reading file message
	call	zrle_sop
;
;	Now read in RLE header, header simply consists of the characters
;	GH.
;
:
	sys_call	os_gb		;get character
	jp	c,zrle_bad_file	;exit if error
	cp	'G'		;look for G
	jr	nz,:-1		;loop if not found
:
	sys_call	os_gb		;get character
	jp	c,zrle_bad_file	;exit if error
	cp	'H'		;look for H
	jr	nz,:-1		;loop if not found
;
;	Initialize data for the dot setting loop, the registers are
;	used as counters as follows:
;
;	B - 8 count for bit shift operations (count down)
;	C - 32 count for line width
;	D - row number being drawn (0 to 191)
;	HL - buffer address
;
	ld	b,8
	ld	c,32
	ld	e,0
	ld	hl,zrle_buffer
	xor	a		;zero first buffer byte
	ld	(hl),a
;
zrle_rle_loop
	call	zrle_get_byte	;get byte
	cp	0		;check for zero
	jr	z,zrle_rle_2
zrle_rle_1
	scf			;set carry flag
	call	zrle_setdot		;set dot
	dec	a		;count it
	jr	nz,zrle_rle_1
zrle_rle_2
	call	zrle_get_byte	;get another byte
	cp	0		;check for zero
	jr	z,zrle_rle_4
zrle_rle_3
	scf			;set carry flag
	call	zrle_clrdot		;clear dot
	dec	a		;count it
	jr	nz,zrle_rle_3
zrle_rle_4
	jr	zrle_rle_loop	;loop till EOF
;
zrle_get_byte
	ld	ix,(zrle_file_handle)	;get file handle
	sys_call	os_gb		;get byte
	jp	c,zrle_bad_file
	sub	020h		;take away offset
	jr	c,zrle_get_byte	;ignore control chars
	ret			;and return
;
zrle_clrdot
	ccf			;complement carry flag (0)
zrle_setdot
	push	af		;save a
	rl	(hl)		;rotate memory left
	dec	b		;count bit
	jr	nz,zrle_sd_2
	inc	hl		;next byte in buffer
	dec	c
	jr	nz,zrle_sd_1	;this line is full
	inc	e		;increment line
	ld	a,e		;check for end of data
	cp	192
	jr	z,zrle_rle_eod	;end of data
	ld	c,32		;reset counter
zrle_sd_1
	xor	a		;zero out byte
	ld	(hl),a
	ld	b,8		;and reset bit counter
zrle_sd_2
	pop	af		;get a back
	ret			;return
;
;	End of data, clean up stack, close file
;
zrle_rle_eod
	pop	af		;clean stack (af)
	pop	af		;and  return address
	ld	hl,zrle_file_handle	;and close file
	call	close_file
;
;	Main display menu
;
	ld	a,00h		;starting row
	ld	(zrle_currow),a
	call	graph_create	;create a graph
	ld	a,0ffh		;and set flag
	ld	(zrle_graph_flag),a
	call	zrle_display	;output first screen
zrle_main_menu
	ld	hl,zrle_main_menu_msg	;output main menu
	call	zrle_sop
zrle_main_menu_1
	sys_call	os_in		;get a character
	jr	c,zrle_main_menu_1	;loop if need be
	call	a_to_lower		;make it lower
	cp	'q'		;quit?
	jp	z,zrle_alldone
	cp	'n'		;display another ?
	jp	z,zrle_get_file
	cp	'a'		;auto scroll
	jp	z,zrle_auto_scroll
;
	ifdef	zrle_print_asm
	cp	'p'		;print out
	jp	z,zrle_print_menu
	endif
;
	cp	0		;check for arrow key
	jr	nz,zrle_main_menu_1
	sys_call	os_in		;get next key
	jr	c,zrle_main_menu_1	;some error
	cp	255		;up arrow
	jr	z,zrle_up_one
	cp	254		;down arrow
	jr	z,zrle_down_one
	cp	251		;shift up arrow
	jr	z,zrle_up_eight
	cp	250		;shift down arrow
	jr	z,zrle_down_eight
	cp	247		;diamond up arrow
	jr	z,zrle_up_page
	cp	246		;diamond down arrow
	jr	z,zrle_down_page
	cp	243		;square up arrow
	jr	z,zrle_up_full
	cp	242		;square down arrow
	jr	z,zrle_down_full
	jr	zrle_main_menu_1	;unknown input
;
zrle_up_one
	ld	a,01h		;count down
	jr	zrle_scroll_down
zrle_down_one
	ld	a,01h		;up count
	jr	zrle_scroll_up
zrle_up_eight
	ld	a,08h		;count down
	jr	zrle_scroll_down
zrle_down_eight
	ld	a,08h		;up count
	jr	zrle_scroll_up
zrle_up_page
	ld	a,040h		;count down
	jr	zrle_scroll_down
zrle_down_page
	ld	a,040h		;up count
	jr	zrle_scroll_up
zrle_up_full
	xor	a		;zero
	jr	zrle_update
zrle_down_full
	ld	a,080h		;up count
	jr	zrle_update
zrle_scroll_down			;scroll down
	ld	b,a		;save a
	ld	a,(zrle_currow)	;get current row
	sub	b
	jr	nc,zrle_update
	xor	a		;down to zero
	jr	zrle_update
zrle_scroll_up			;scroll up
	ld	b,a		;save a
	ld	a,(zrle_currow)	;get current row
	add	a,b		;add it
	cp	080h		;check for over the limit
	jr	c,zrle_update
	ld	a,080h		;set to last
	jr	zrle_update
zrle_update
	ld	(zrle_currow),a	;save current row
	call	zrle_display	;update graphics
	jp	zrle_main_menu_1	;and loop to menu
;
zrle_auto_scroll			;auto scroll
	ld	a,00h		;zero count
	ld	(zrle_currow),a
	call	zrle_display
	ld	hl,zrle_currow	;set address of row
	ld	b,040h		;scroll count
zrle_autos_1
	push	bc		;save counter
	push	hl		;and buffer
	inc	(hl)		;next row
	inc	(hl)
	call	zrle_display
	pop	hl		;get hl back
	pop	bc		;and counter
	djnz	zrle_autos_1	;loop
	jp	zrle_main_menu_1	;and back to menu
;
	ifdef	zrle_print_asm
;
zrle_print_menu
	ld	hl,zrle_print_menu_msg	;put out menu
	call	zrle_sop
zrle_prntm_1
	sys_call	os_in		;get character
	jr	c,zrle_prntm_1
	call	a_to_lower		;make lower
	cp	'q'		;check for quite
	jp	zrle_main_menu_1
	cp	'c'		;screen
	jr	nz,zrle_prntm_2
	ld	a,(zrle_currow)	;get start row
	ld	b,a
	ld	c,040h		;and extent
	jp	zrle_print_image	;and print it
zrle_prntm_2
	cp	'f'		;full image
	jr	nz,zrle_prntm_3
	ld	bc,00c0h		;start 0 length c0
	jp	zrle_print_image
zrle_prntm_3
	jr	zrle_prntm_1
;
zrle_print_image			;print image
	push	bc		;save count
	ld	bc,0080h		;local and scratch length
	ld	hl,zrle_com_name	;com name
	ld	de,zrle_scratch	;scratch
	ld	a,02h		;open for output
	sys_call	gn_opf		;open it
	jp	c,zrle_alldone
	ld	(zrle_file_handle),ix	;save handle
;
;	put out printer line header
;
	ld	a,esc		;graphics command
	sys_call	os_out
	ld	a,'K'
	sys_call	os_out
	ld	a,00h		;and length, lsb first
	sys_call	os_out
	ld	a,01h
	sys_call	os_out
zrle_prnti_1
	pop	bc		;get start and count back
	ld	d,0		;and compute offset
	ld	e,b
	ld	hl,0020h		;line length
	sys_call	gn_m16		;multiply
	ld	de,zrle_buffer	;buffer
	add	hl,de		;compute address
	push	bc		;save count
	push	hl		;save start addr
	ld	b,08h		;set bit count
zrle_prnti_2
	ld	c,00h		;zero c
	push	hl		;save address
zrle_prnti_3
	push	bc		;save offset count
	ld	a,(hl)		;get byte
zrle_prnti_4
	rrca			;rotate
	djnz	zrle_printi_4	;till
	and	080h		;mask out low bits
	or	c		;and set in c
	rrca			;then move over
	pop	bc		;get offset back
	ld	c,a		;save in c
	dec	b		;count down
	jr	z,zrle_prnti_6
	ld	de,0020h		;offset to next byte
	add	hl,de
	jr	zrle_prnti_3
zrle_prnti_6
	ld	ix,(zrle_file_handle)	;get handle
	pushall			;save all registers
	ld	a,c		;and output byte
	sys_call	os_out
	popall			;get back registers
	pop	hl		;get hl
	inc	hl		;next byte
	djnz	zrle_prnti_2	;loop for 8
;
;	Now have output one byte block
;
	endif
;
; ==========================================================================
;
;	Support routines
;
; ==========================================================================
;
zrle_display			;display data starting at currow
	ld	h,0		;set count in hl
	ld	a,(zrle_currow)
	ld	l,a
	ld	de,0020h		;bytes/row
	sys_call	gn_m16		;get offset
	ld	de,zrle_buffer	;and add in address
	add	hl,de
	ld	de,0020h		;now set increment
	call	graph_draw		;and output data
	ret			;then return
;
zrle_sop				;standard output
	push	hl		;save message
	ld	hl,zrle_sel_mainwin	;set for main window
	sys_call	gn_sop		;and select
	pop	hl		;get message
	sys_call	gn_sop		;and output
	ret			;then return
;
zrle_alldone			;normal exit routine
	call	zrle_cleanup	;close out all files
	ld	hl,(zrle_stack)	;get old stack
	ld	sp,hl		;and make current
	ret			;and return
;
zrle_cleanup
	ld	hl,zrle_file_handle	;close file if need be
	call	close_file
	ld	a,(zrle_graph_flag)	;check for graph
	and	a
	ret	z		;return if no problem
	call	graph_delete	;and delete map area if one
	ret			;and return
;
zrle_bad_file			;put out bad file message
	ld	hl,zrle_bad_file_msg	;put out message
	call	zrle_sop
	call	pause		;and pause
	ld	hl,(zrle_stack)	;restore stack
	ld	sp,hl
	jp	zrle_get_file	;and go back to getting file
;
; ==========================================================================
;
;	Static data area
;
; ==========================================================================
;
zrle_pname		db	'ZRLE v1.01',0
zrle_com_name	db	':COM.0',0
zrle_gfile_msg			;file input prompt
	db	'RLE File to display',0
zrle_main_win_msg			;main window definition
	vid_win_1	1,0,0,49,8
	db	'ZRLE V1.01 - COPYRIGHT 1990 RICHARD HAW'
	vid_win_2	1,0,0,49,8
	vid_cursor_off
	db	0
zrle_sel_mainwin			;select main window
	vid_win_sel	1
	db	0
;
zrle_main_menu_msg			;main menu
	db	12,10
	vid_gotoxy	3,1
	db	1,'B','A',1,'B','uto scroll'
	vid_gotoxy	3,2
	db	1,'B','N',1,'B','ew file'
;
	ifdef	zrle_print_asm
	vid_gotoxy	3,3
	db	1,'B','P',1,'B','rint'
	endif
;
	vid_gotoxy	3,5
	db	1,'B','Q',1,'B','uit ZRLE'
	vid_gotoxy	24,1
	db	1,242,1,243		;arrows
	db	' 1 pixel scroll'
	vid_gotoxy	20,2
	db	1,45,' '		;shift
	db	1,242,1,243		;arrows
	db	' 8 pixel scroll'
	vid_gotoxy	21,3
	db	1,43,'  '		;shift
	db	1,242,1,243		;arrows
	db	' 1 page scroll'
	vid_gotoxy	21,4
	db	1,42,'  '		;shift
	db	1,242,1,243		;arrows
	db	' top/bottom'
	db	0
;
zrle_print_menu_msg
	db	12,10
	vid_gotoxy	3,1
	db	1,'B','C',1,'B','urrent screen'
	vid_gotoxy	3,2
	db	1,'B','F',1,'B','ull image'
	vid_gotoxy	3,5
	db	1,'B','Q',1,'B','uit to main menu'
	db	0
;
zrle_bad_file_msg
	db	12,10,10
	vid_just_center
	db	'Error reading file',10
	vid_just_normal
	db	0
zrle_reading_msg			;reading in file message
	db	12,10,10
	vid_just_center
	db	1,'B',1,'F'
	db	'Reading in RLE file',10
	db	1,'F',1,'B'
	vid_just_normal
	db	0
